<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia
*  @copyright 2019 - 2020 Prestalia
*  @license   This is proprietary software thus it cannot be distributed or reselled
*/

use Prestalia\ExclusiveLock\Locks\FlockLock;

require_once dirname(__FILE__).'/../autoload.php';
require_once './doExportFunctions.php';

if (Tools::isSubmit('ajax')) {
    error_reporting(0);
} else {
    error_reporting(E_ALL);
}

header('Content-Type: application/json; charset=utf-8');
header('Access-Control-Allow-Origin: *');
header("Cache-Control: no-cache, must-revalidate");
header("Expires: Sat, 26 Jul 1997 05:00:00 GMT");

/*
 * Script start
 */

try {
/*
 * Checks and config
 */

/* Set script time values */
    $time = array(
        'start'  => (double)microtime(true),
        'max'    => (int)ini_get('max_execution_time'),
        'margin' => 5,
    );

    // ini_set('memory_limit', '512M');

/* Get memory limit */
    $memLimit = iniStringToBytes(ini_get('memory_limit'));

/* Get shop id */
    $idShop = (int)Tools::getValue('shop');

    if (!$idShop || !is_numeric($idShop)) {
        term(array(
            'success' => false,
            'msg'     => 'invalid_shop_id',
        ));
    }

/* Check endpoint */
    $endpoint = $idLang = Tools::getValue('lang');

    if (!$endpoint) {
        term(array(
            'success' => false,
            'msg'     => 'invalid_endpoint',
        ));
    }

/* Set export status (will be eventually changed in the code below) */
    SniperCommon::setConfig('last_export_status', 'error', $idShop, $endpoint);

/* Check token */
    $token = Tools::getValue('token');

    if (!$token || $token != SniperCommon::getConfig('ajax_token', $idShop, $idLang)) {
        term(array(
            'success' => false,
            'msg'     => 'invalid_token',
        ));
    }

    $recursive = Tools::getValue('recursive');
    $cycle = Tools::getValue('cycle', 0);

/* Set employee object into context */
    $idEmployee = (int)Tools::getValue('employee');

    if (!$idEmployee || !is_numeric($idEmployee)) {
        term(array(
            'success' => false,
            'msg'     => 'invalid_employee_id',
        ));
    }
    $module = new Sniperfast();
    $moduleVersion = $module->version;

    $context = Context::getContext();
    $context->cart = null;
    $context->customer = null;
    $context->language = new Language($idLang, $idLang);
    if (Tools::version_compare(_PS_VERSION_, '1.6.1.0', '<')) {
        $context->shop = new Shop($idShop, null, $idShop);
    } else {
        $context->shop = new Shop($idShop, $idLang, $idShop);
    }

    if (_PS_VERSION_ < '1.6') {
        $context->employee = new Employee($idEmployee, null, $idShop);
    } else {
        $context->employee = new Employee($idEmployee, $idLang, $idShop);
    }

    $idCurrency = SniperCommon::getConfig('sniper_currency', $idShop, $idLang);
    if (empty($idCurrency)) {
        $context->currency = Currency::getDefaultCurrency();
    } else {
        $context->currency = new Currency($idCurrency, $idLang, $idShop);
    }

/* Check the output directory */
    $moduleDir      = _PS_MODULE_DIR_.'sniperfast';
    $outputDir      = $moduleDir.'/output/';
    $outputFileName = $idShop.'_'.$idLang.'_export.csv';

    if (!file_exists($outputDir)) {
        umask(0);
        if (!mkdir($outputDir, 0777, true)) {
            term(array(
                'success' => false,
                'msg'     => 'cannot_create_output_dir',
            ));
        }
    } elseif (!is_writable($outputDir)) {
        term(array(
            'success' => false,
            'msg'     => 'output_dir_not_writable',
        ));
    }

    $tmp_dir = $moduleDir."/tmp/";
    $sniperfastModule = new Sniperfast();
    $lock = new FlockLock($tmp_dir.'sync.lockfile');
    if (!$lock->acquire()) {
        $infoMsg1 = $sniperfastModule->l('Another export is currently running.', 'doExport');
        $infoMsg2 = $sniperfastModule->l('Please wait for it to end before starting a new one.', 'doExport');
        term(array(
            'success' => false,
            'msg'     => 'another_instance_running',
            'info'    => $infoMsg1.' '.$infoMsg2
        ));
    }

/* Init stats array */
    $stats = array(
        'products_to_export'       => 0,
        'skipped_already_exported' => 0,
        'skipped_price'            => 0,
        'skipped_quantity'         => 0,
        'skipped_inactive'         => 0,
        'skipped_no_attributes'    => 0
    );

/* Get config from DB */
    $config = SniperCommon::getConfigKeys($idShop, $idLang);
    if (empty($config)) {
        term(array(
            'success' => false,
            'msg'     => 'missing_configuration',
        ));
    }

/* Get and set last product id and last export date */
    $lastIdProduct  = SniperCommon::getConfig('last_id_product', $idShop, $idLang);
    $lastExportDate = SniperCommon::getConfig('last_export_date', $idShop, $idLang);

    if ($lastIdProduct == "" || Tools::isSubmit('restart')) {
        $lastIdProduct = 0;
        SniperCommon::setConfig('last_id_product', '0', $idShop, $idLang);
    }

    if ($lastExportDate == "" || Tools::isSubmit('restart')) {
        $lastExportDate = 1;
        SniperCommon::setConfig('last_export_date', '1', $idShop, $idLang);
    }

    $lastIdProduct  = (int)$lastIdProduct;
    $lastExportDate = (int)$lastExportDate;

/* Stop here if specified gap between exports is not yet elapsed */
    if (Tools::isSubmit('cron') && time() < $lastExportDate + ($config['exports_gap']*60)) {
        SniperCommon::setConfig('last_export_status', 'exports_gap_not_elapsed', $idShop, $idLang);

        $infoMsg1 = $sniperfastModule->l('Another export was recently performed.', 'doExport');
        $infoMsg2 = $sniperfastModule->l('You need to wait at least', 'doExport');
        $infoMsg3 = $sniperfastModule->l('minutes befores of running another CRON.', 'doExport');
        $infoMsg4 =
        $sniperfastModule->l('You can still force the export manually using the dedicated button within the module.', 'doExport');

        term(array(
            'success' => false,
            'msg'     => 'exports_gap_not_elapsed',
            'info'    => $infoMsg1.' '.$infoMsg2.' '.$config['exports_gap'].' '.$infoMsg3.' '.$infoMsg4
        ));
    }

    /* Clear output file and cache if new export */
    $log_prefix = $idShop.'_'.$idLang;
    if ($lastIdProduct == 0) {
        file_put_contents($outputDir.$outputFileName, '');

        if (file_exists($tmp_dir.$log_prefix."_enabled_products.dat")) {
            @unlink($tmp_dir.$log_prefix."_enabled_products.dat");
        }

        SniperCommon::setConfig('skipped_no_attributes', '0', $idShop, $idLang);

        // if (file_exists($tmp_dir.$log_prefix."_boost.dat")) {
        //     @unlink($tmp_dir.$log_prefix."_boost.dat");
        // }
        // $boost = getBoostData($idShop, $idLang);
        // file_put_contents($tmp_dir.$log_prefix."_boost.dat", serialize($boost));
    }
    // else {
    //     if (file_exists($tmp_dir.$log_prefix."_boost.dat")) {
    //         $boost = unserialize(Tools::file_get_contents($tmp_dir.$log_prefix."_boost.dat"));
    //     } else {
    //         $boost = getBoostData($idShop, $idLang);
    //         file_put_contents($tmp_dir.$log_prefix."_boost.dat", serialize($boost));
    //     }
    // }

/*
 * Non product-dependent export data
 */
    /* Get enabled products */
    $product_filter = new SniperfastProductFilter((int)$idShop, (int)$endpoint);
    $enabledProducts = getEnabledProducts($product_filter, $idShop, $idLang);

    if (empty($enabledProducts)) {
        term(array(
            'success' => true,
            'msg'     => 'nothing_to_do',
        ));
    }

    $stats['products_to_export'] = count($enabledProducts);

    // Set category navigation pipe
    $catNavPipe = Configuration::get('PS_NAVIGATION_PIPE');

    if ($catNavPipe == '' || $catNavPipe == false || $catNavPipe == null) {
        $catNavPipe = '>';
    }

    // check if we have to export features/attributes and build enabled features/attributes array
    $exportFeatures = false;
    if ($config['sniper_features'] != '') {
        $exportFeatures = true;
        $enabledIdFeatures = array_map('intval', explode(',', $config['sniper_features']));
    } elseif ($config['sniper_free_ship'] > 0 || $config['sniper_export_review'] > 0) {
        $exportFeatures = true;
    }

    $exportAttributes = false;
    if ($config['sniper_attributes'] != '') {
        $exportAttributes = true;
        $enabledIdAttributeGroups = array_map('intval', explode(',', $config['sniper_attributes']));
    }

    // Get if https is requested for images
    // $useHttpsForImages = SniperCommon::getConfig('https_image_protocol', $idShop, $idLang);
    $config['use_https'] = Configuration::get('PS_SSL_ENABLED');
    $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
    if (Configuration::get('PS_SSL_ENABLED')) {
        $baseUrl = str_replace('http://', 'https://', $baseUrl);
    }
    $config['base_url'] = $baseUrl;
/*
 * Export
 */
    $rowCount             = 0;
    $categoryPathCache    = array();
    $categoryDefaultCache = array();

    // Get max categories per product
    $maxCategories = 4;
    $rootCategory = Category::getRootCategory();
    $rootCategoryId = $rootCategory->id_category;

    // Write layout to file
    if ($lastIdProduct == 0) {
        $header = "id_product|id_category_default|reference|ean13|pname|description|".
        "description_short|supplier_name|category_link_rewrite|plink|mname|quantity|".
        "supplier_reference|cover_image_url|psales|category|category/lvl1|category/lvl2|".
        "category/lvl3|category/lvl4|price_deleted|price_showed|features|attributes|tags|alternate_image_url";
        file_put_contents(
            $outputDir.$outputFileName,
            $header."\n",
            FILE_APPEND
        );
    }

    $filePointer = fopen($outputDir.$outputFileName, "a");

    // Sales cache
    $sql = "SELECT id_product, quantity FROM "._DB_PREFIX_."product_sale";
    $sales = Db::getInstance()->executeS($sql);
    $id_sales_cache = array();
    if (!empty($sales)) {
        foreach ($sales as $tmp) {
            $id_sales_cache[$tmp['id_product']] = $tmp['quantity'];
        }
    }

    // Process products
    $processedProducts = 0;

    $sniperfastProductExported = new SniperfastProductExporter($idShop, $idLang, $config, $id_sales_cache);

    foreach ($enabledProducts as $idProd) {
        $data = array();
        // Skip current product if already exported
        if ($idProd <= $lastIdProduct) {
            $stats['skipped_already_exported']++;
        } else {
            try {
                $data = $sniperfastProductExported->exportData($idProd, $stats);
            } catch (Exception $e) {
                $stats['skipped_no_attributes']++;
            }
        }

        if (!empty($data)) {
            if (writeCsvRow(
                $data,
                $filePointer
            )) {
                $rowCount++;
            } else {
                $stats['skipped_no_attributes']++;
            }
        }

        if (Tools::isSubmit('cron')) {
            // Calculate elapsed time
            $timeElapsed = microtime(true) - $time['start'];

            // Stop here if time/memory limit has been reached
            if (($time['max'] > 0 && $time['max'] - $timeElapsed <= $time['margin'])
                || ($memLimit > 0 && memory_get_usage() >= $memLimit - (20*1048576))
                || ($recursive && $rowCount >= $config['export_batch_size'])
            ) {
                $lastIdProduct = $idProd;
                SniperCommon::setConfig('last_id_product', $lastIdProduct, $idShop, $endpoint);
                break;
            }
        } else {
            // If exporting from the module GUI execute batches of products
            // to give a better work feedback to the user
            if ($rowCount >= $config['export_batch_size']) {
                $lastIdProduct = $idProd;
                SniperCommon::setConfig('last_id_product', $lastIdProduct, $idShop, $endpoint);
                break;
            }
        }
        unset($data);
        cleanMemory();
    } /* End products loop */

    fclose($filePointer);

    $processedProducts = $stats['skipped_already_exported']
        + $stats['skipped_price']
        + $stats['skipped_quantity']
        + $stats['skipped_inactive']
        + $stats['skipped_no_attributes']
        + $rowCount;

    // Check if export is completed
    $additionalInfo = '';
    if ($processedProducts >= $stats['products_to_export']) {
        $outMsg = "export_done";
        SniperCommon::setConfig('last_id_product', '0', $idShop, $endpoint);
        SniperCommon::setConfig('last_export_date', time(), $idShop, $endpoint);
        SniperCommon::setConfig('last_export_status', 'ok', $idShop, $endpoint);
    } else {
        $outMsg = "export_cycle_done";
        SniperCommon::setConfig('last_export_status', 'products_left', $idShop, $endpoint);

        if (Tools::isSubmit('cron') && $recursive) {
            $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
            if (Configuration::get('PS_SSL_ENABLED')) {
                $baseUrl = str_replace('http://', 'https://', $baseUrl);
            }

            ++$cycle;

            $ajaxExportUrl =
            "{$baseUrl}modules/sniperfast/ajax/doExport.php?shop={$idShop}&lang={$endpoint}".
            "&employee={$idEmployee}&token={$token}&cron=1&recursive=1&cycle=".$cycle;

            Tools::redirect($ajaxExportUrl);
        }

        $infoMsg1 = $sniperfastModule->l('The export process is not complete.', 'doExport');
        $infoMsg2 = $sniperfastModule->l('You must run another CRON to resume it.', 'doExport');
        $infoMsg3 =
        $sniperfastModule->l('Repeat this operation as many times as required to complete the export.', 'doExport');
        $additionalInfo = $infoMsg1.' '.$infoMsg2.' '.$infoMsg3;
    }

    if (Tools::isSubmit('cron') && $outMsg == 'export_done') {
        $idJob = 0;
        $fetchRequestRes = sendFetchRequest($idShop, $idLang, $token, $idJob, $moduleVersion);
        SniperCommon::setConfig('sniper_server_communication', $fetchRequestRes, $idShop, $idLang, $idJob);
        if ($fetchRequestRes) {
            SniperCommon::setConfig('last_job_id', $idJob, $idShop, $idLang);
        }
    }

    term(array(
        'success'           => true,
        'msg'               => $outMsg,
        'info'              => $additionalInfo,
        'time_total'        => round(microtime(true) - $time['start'], 3),
        'time_product_avg'  => round((microtime(true) - $time['start']) / $processedProducts, 5),
        'max_used_memory'   => round(memory_get_peak_usage() / 1048576, 2) . ' MB',
        'exported_csv_rows' => $rowCount,
        'last_id_product'   => $lastIdProduct,
    ) + $stats);
} catch (Exeption $ex) {
    term(array(
        'success' => false,
        'msg'     => 'exeption_msg='.$ex->getMessage(),
    ));
}


/*
 * Service functions
 */

function term($jsonVals)
{
    echo Tools::jsonEncode($jsonVals);
    exit();
}

function iniStringToBytes($val)
{
    $val  = trim((string)$val);
    $last = Tools::strtolower(Tools::substr($val, -1));
    $val  = (int)$val;

    // Fall-through is intentional
    switch ($last) {
        case 'g':
        case 'G':
            $val *= 1024; // go on multiplying
        case 'm':
        case 'M':
            $val *= 1024; // go on multiplying
        case 'k':
        case 'K':
            $val *= 1024;
    }

    return (int)$val;
}
