<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia
*  @copyright 2019 - 2020 Prestalia
*  @license   This is proprietary software thus it cannot be distributed or reselled
*/

function writeCsvRow($data, $filePointer)
{
    $csvRow = array(
        $data['id_product'],
        $data['id_category_default'],
        $data['reference'],
        $data['ean13'],
        $data['pname'],
        $data['description'],
        $data['description_short'],
        $data['supplier_name'],
        $data['category_link_rewrite'],
        $data['plink'],
        $data['mname'],
        $data['quantity'],
        $data['supplier_reference'],
        $data['cover_image_url'],
        $data['psales'],
        $data['category'],
        $data['category/lvl1'],
        $data['category/lvl2'],
        $data['category/lvl3'],
        $data['category/lvl4'],
        $data['price_deleted'],
        $data['price_showed'],
        $data['features'],
        $data['attributes'],
        $data['tags'],
        $data['alternate_image_url']
    );

    return fputcsv($filePointer, $csvRow, '|', '"');
}

function getEnabledProducts(SniperfastProductFilter $filter, $idShop, $idLang)
{
    $cacheDir = dirname(__FILE__)."/../tmp/";
    $cacheFile = "{$idShop}_{$idLang}_enabled_products.dat";

    if (file_exists($cacheDir.$cacheFile)) {
        return unserialize(Tools::file_get_contents($cacheDir.$cacheFile));
    }

    $out = null;
    $out = $filter->getEnabledProducts();

    if (!file_exists($cacheDir)) {
        $old_mask = umask();
        umask(0);
        mkdir($cacheDir, 0775);
        umask($old_mask);
    }

    file_put_contents($cacheDir.$cacheFile, serialize($out));

    return $out;
}

function getBoostData($idShop, $idLang)
{
    $dbp = _DB_PREFIX_;
    $db  = Db::getInstance();

    $boost = array(
        'product'       => array(),
        'category'      => array(),
        'manufacturer'  => array(),
        'supplier'      => array()
    );

    $elements = array(
        'product',
        'category',
        'manufacturer',
        'supplier'
    );

    foreach ($elements as $el) {
        $sql = "SELECT id_".pSQL($el).", boost
            FROM {$dbp}sniperfast_".pSQL($el)."
            WHERE id_shop = ".(int)$idShop.
            " AND endpoint = ".(int)$idLang;
        $res = $db->executeS($sql);
        if (!empty($res)) {
            foreach ($res as $r) {
                $boost[$el][$r['id_'.$el]] = $r['boost'];
            }
        }
    }

    return $boost;
}

function cleanMemory()
{
    static $last_memory = false;

    if ($last_memory === false) {
        $last_memory = memory_get_usage();
    }

    $current_memory = memory_get_usage();
    if ($current_memory - $last_memory < (32 * 1048576)) {
        return false;
    }

    Cache::clean('*');
    Product::flushPriceCache();
    $last_memory = $current_memory;

    return true;
}

function sendFetchRequest($idShop, $idLang, $token, &$idJob, $moduleVersion)
{
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL."://index.sniperfast.".CDN_ENDPOINT."/schedule");
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query(array(
        'shop'  => $idShop,
        'lang'  => $idLang,
        'key'   => SniperCommon::getConfig('sniper_key', $idShop, $idLang),
        'index' => SniperCommon::getConfig('sniper_index_name', $idShop, $idLang),
        'token' => $token,
        'ip'    => $_SERVER['SERVER_ADDR'],
        'v'     => $moduleVersion
    )));

    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $res = curl_exec($ch);
    curl_close($ch);
    $res = Tools::jsonDecode($res, true);

    if (isset($res['id_job'])) {
        $idJob = $res['id_job'];
    }

    if (isset($res['status']) && $res['status'] == 'ok') {
        return 1;
    }
    return 0;
}
