<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia
*  @copyright 2019 - 2020 Prestalia
*  @license   This is proprietary software thus it cannot be distributed or reselled
*/

require_once dirname(__FILE__).'/../autoload.php';

header('Content-Type: application/json; charset=utf-8');
header('Access-Control-Allow-Origin: *');
header("Cache-Control: no-cache, must-revalidate");
header("Expires: Sat, 26 Jul 1997 05:00:00 GMT");

$idShop    = (int)Tools::getValue('shop');
$endpoint  = (int)Tools::getValue('ep');

/* Check shop id */
if (!is_numeric($idShop)) {
    echo Tools::jsonEncode(array(
        'success' => false,
        'error'  => 'wrong id_shop',
    ));
    die;
}

/* Check endpoint id */
if (!is_numeric($endpoint)) {
    echo Tools::jsonEncode(array(
        'success' => false,
        'error'  => 'wrong endpoint',
    ));
    die;
}

/* Check token */
$token = Tools::getValue('token');

if (!$token || $token != SniperCommon::getConfig('ajax_token', $idShop, $endpoint)) {
    echo Tools::jsonEncode(array(
        'success' => false,
        'error'     => 'invalid_token',
    ));
    die;
}

/* Check type */
$type = Tools::getValue('type');

if (!$type) {
    echo Tools::jsonEncode(array(
        'success' => false,
        'error'     => 'invalid_type',
    ));
    die;
}

$filter = new SniperfastProductFilter((int)$idShop, (int)$endpoint);

switch ($type) {
    case 'cat':
        /* Check category id */
        $idCategory = (int)Tools::getValue('id_category');
        if (!is_numeric($idCategory)) {
            echo Tools::jsonEncode(array(
                'success' => false,
                'error'  => 'wrong id_category',
            ));
            die;
        }
        if (!$filter->isCategoryEnabled($idCategory)) {
            $filter->enableCategory($idCategory);
            $current_status = 1;
        } else {
            $filter->disableCategory($idCategory);
            $current_status = 0;
        }
        break;
    case 'manu':
        /* Check manufacturer id */
        $idManufacturer = (int)Tools::getValue('id_manufacturer');
        if (!is_numeric($idManufacturer)) {
            echo Tools::jsonEncode(array(
                'success' => false,
                'error'  => 'wrong id_manufacturer',
            ));
            die;
        }
        if (!$filter->isManufacturerEnabled($idManufacturer)) {
            $filter->enableManufacturer($idManufacturer);
            $current_status = 1;
        } else {
            $filter->disableManufacturer($idManufacturer);
            $current_status = 0;
        }
        break;
    case 'prod':
        /* Check product id */
        $idProduct = (int)Tools::getValue('id_product');
        if (!is_numeric($idProduct)) {
            echo Tools::jsonEncode(array(
                'success' => false,
                'error'  => 'wrong id_product',
            ));
            die;
        }
        if (!$filter->isProductEnabled($idProduct)) {
            $filter->enableProduct($idProduct);
            $current_status = 1;
        } else {
            $filter->disableProduct($idProduct);
            $current_status = 0;
        }
        break;
    case 'supp':
        /* Check supplier id */
        $idSupplier = (int)Tools::getValue('id_supplier');
        if (!is_numeric($idSupplier)) {
            echo Tools::jsonEncode(array(
                'success' => false,
                'error'  => 'wrong id_supplier',
            ));
            die;
        }
        if (!$filter->isSupplierEnabled($idSupplier)) {
            $filter->enableSupplier($idSupplier);
            $current_status = 1;
        } else {
            $filter->disableSupplier($idSupplier);
            $current_status = 0;
        }
        break;
}

echo Tools::jsonEncode(array(
    'success' => true,
    'status'  => $current_status,
));
