<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia
*  @copyright 2019 - 2020 Prestalia
*  @license   This is proprietary software thus it cannot be distributed or reselled
*/

require_once dirname(__FILE__).'/../autoload.php';
require_once dirname(__FILE__).'/saveDataFunctions.php';

header('Content-Type: application/json; charset=utf-8');
header('Access-Control-Allow-Origin: *');
header("Cache-Control: no-cache, must-revalidate");
header("Expires: Sat, 26 Jul 1997 05:00:00 GMT");

/* Get shop id */
$idShop = (int)Tools::getValue('shop');
if (!$idShop || !is_numeric($idShop)) {
    exit(Tools::jsonEncode(array(
        'success' => false,
        'msg'     => 'invalid_shop_id'
    )));
}

/* Get language id */
$idLang = Tools::getValue('lang');
if (!$idLang) {
    exit(Tools::jsonEncode(array(
        'success' => false,
        'msg'     => 'invalid_lang'
    )));
}

/* Check token */
$token = Tools::getValue('token');
if (!$token || $token != SniperCommon::getConfig('ajax_token', $idShop, $idLang)) {
    exit(Tools::jsonEncode(array(
        'success' => false,
        'msg'     => 'invalid_token'
    )));
}

/* Check type */
$type = Tools::getValue('type');
if (!$type) {
    exit(Tools::jsonEncode(array(
        'success' => false,
        'msg'     => 'invalid_type',
    )));
}

$success = false;
$msg = '';
$result = null;

switch ($type) {
    case 'account':
        $success = registerRecoverAccount($idShop, $idLang, $msg);
        break;
    case 'boost':
        $elements = Tools::getValue('el');
        $boost  = Tools::getValue('val');
        $source = Tools::getValue('sel');
        $success = saveBoostData($idShop, $idLang, $elements, $boost, $source, $msg);
        break;
    case 'boost_reset':
        $source = Tools::getValue('sel');
        $success = resetBoostData($idShop, $idLang, $source, $msg);
        break;
    case 'f_range':
        $range = Tools::getValue('val');
        $success = saveFeatureRange($idShop, $idLang, $range, $msg);
        break;
    case 'page_url':
        $success    = getResultPageUrl($idShop, $idLang, $result, $msg);
        break;
    case 'save_job':
        $idJob      = Tools::getValue('id_job');
        $success    = saveJobId($idJob, $idShop, $idLang, $msg);
        break;
    case 'save_stats_settings':
        $exludedIps = Tools::getValue('excluded_ips');
        $success    = saveStatsOptions($exludedIps, $idShop, $idLang, $result, $msg);
        break;
}

if ($success) {
    exit(Tools::jsonEncode(array(
        'success' => true,
        'msg'     => $msg,
        'result'  => $result
    )));
} else {
    exit(Tools::jsonEncode(array(
        'success' => false,
        'msg'     => $msg
    )));
}
