<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia
*  @copyright 2019 - 2020 Prestalia
*  @license   This is proprietary software thus it cannot be distributed or reselled
*/

function getSourceName($source)
{
    $sourceName = '';
    switch ($source) {
        case 'prod':
            $sourceName = 'product';
            break;
        case 'cat':
            $sourceName = 'category';
            break;
        case 'manu':
            $sourceName = 'manufacturer';
            break;
        case 'supp':
            $sourceName = 'supplier';
            break;
    }
    return $sourceName;
}

function registerRecoverAccount($idShop, $idLang, &$msg)
{
    $storeAddress = getStoreAddress($idShop, $idLang);

    $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
    if (Configuration::get('PS_SSL_ENABLED')) {
        $baseUrl = str_replace('http://', 'https://', $baseUrl);
    }
    $baseUrl = rtrim($baseUrl, '/');

    $shopName = str_replace(array('http://', 'https://'), '', $baseUrl);
    $shopName = preg_replace('/[^a-z\d]+/i', '', $shopName);
    $shopName = Tools::substr($shopName, 0, 52);

    $params = array(
        'shop_id'  => $idShop,
        'shop_name' => $shopName,
        'shop_url' => $baseUrl,
        'shop_email' => Configuration::get('PS_SHOP_EMAIL'),
        'shop_address' => Tools::jsonEncode($storeAddress),
        'type'  => 'Prestashop',
        'lang'  => $idLang,
        'shop_ip' => $_SERVER['SERVER_ADDR']
    );

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL.'://index.sniperfast.'.CDN_ENDPOINT.'/autoregister');
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));

    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $res = curl_exec($ch);
    curl_close($ch);
    $res = json_decode($res, true);

    if (empty($res)) {
        $msg = 'autoregister_error';
        return false;
    }
    if (!$res['success']) {
        $msg = $res['msg'];
        return false;
    }
    SniperCommon::setConfig('sniper_index_name', $res['sniper_index'], $idShop, $idLang);
    SniperCommon::setConfig('sniper_key', $res['sniper_key'], $idShop, $idLang);

    SniperCommon::updateMainData($idShop, $idLang, $msg);

    $msg = $res['msg'];
    return true;
}

function getStoreAddress($idShop, $idLang)
{
    $dbp = _DB_PREFIX_;
    $storeAddress = array();
    if (Tools::version_compare(_PS_VERSION_, '1.7.3.0', '<')) {
        $sql = "SELECT so.address1, so.address2, so.city, so.postcode AS zip,
            so.phone, cl.name AS country
            FROM {$dbp}store AS so
            LEFT JOIN {$dbp}store_shop AS ss ON so.id_store = ss.id_store
            LEFT JOIN {$dbp}country_lang AS cl ON so.id_country = cl.id_country
            WHERE ss.id_shop = ".(int)$idShop.
            " AND cl.id_lang = ".(int)$idLang.
            " LIMIT 1";
        $storeAddress = Db::getInstance()->executeS($sql);
    } else {
        $sql = "SELECT so.city, sl.address1, sl.address2, so.postcode AS zip,
            so.phone, cl.name AS country
            FROM {$dbp}store AS so
            LEFT JOIN {$dbp}store_shop AS ss ON so.id_store = ss.id_store
            LEFT JOIN {$dbp}store_lang AS sl ON so.id_store = sl.id_store
            LEFT JOIN {$dbp}country_lang AS cl ON so.id_country = cl.id_country
            WHERE ss.id_shop = ".(int)$idShop.
            " AND sl.id_lang = ".(int)$idLang.
            " AND cl.id_lang = ".(int)$idLang.
            " LIMIT 1";
        $storeAddress = Db::getInstance()->executeS($sql);
    }
    if (empty($storeAddress)) {
        return array();
    }

    return $storeAddress[0];
}

function resetBoostData($idShop, $idLang, $source, &$msg)
{
    $table = 'sniperfast_';
    $sourceName = getSourceName($source);
    $table .= $sourceName;
    $sql = "UPDATE "._DB_PREFIX_.pSQL($table)."
        SET boost = 0
        WHERE id_shop = ".(int)$idShop."
        AND endpoint = ".(int)$idLang;
    $msg = 'reset_boost_success';
    return Db::getInstance()->execute($sql);
}

function saveBoostData($idShop, $idLang, $elements, $boost, $source, &$msg)
{
    if (empty($elements)) {
        $msg = 'invalid_elements';
        return false;
    }
    if (!is_numeric($idShop) || !is_numeric($idLang)) {
        $msg = 'invalid_shop_lang';
        return false;
    }
    $table = 'sniperfast_';
    $sourceName = getSourceName($source);
    $table .= $sourceName;
    $sql = "INSERT INTO "._DB_PREFIX_.pSQL($table).
        " (id_".pSQL($sourceName).", id_shop, endpoint, boost) VALUES ";
    $sqlDuplicate = " ON DUPLICATE KEY UPDATE boost = VALUES(boost)";
    $values = array();
    foreach ($elements as $el) {
        $tempValue = array(
            (int)$el,
            (int)$idShop,
            (int)$idLang,
            (int)$boost
        );
        $values[] = '('.implode(',', $tempValue).')';
    }
    $values = implode(',', $values);
    Db::getInstance()->execute($sql.$values.$sqlDuplicate);

    return true;
}

function saveFeatureRange($idShop, $idLang, $range, &$msg)
{
    $msg = 'feature_success';
    $featureRangeFile = dirname(__FILE__).'/../output/'.$idShop.'_'.$idLang.'_range_feature.json';
    $errorMsg = '';
    if (!file_exists($featureRangeFile) && empty($range)) {
        return true;
    } elseif (!file_exists($featureRangeFile) && !empty($range)) {
        file_put_contents($featureRangeFile, $range);
        SniperCommon::updateMainData($idShop, $idLang, $errorMsg);
        return true;
    } else {
        $previousRange = Tools::file_get_contents($featureRangeFile);
        if ($previousRange == $range['features_details']) {
            return true;
        }
        file_put_contents($featureRangeFile, $range);
        SniperCommon::updateMainData($idShop, $idLang, $errorMsg);
        return true;
    }
}

function getResultPageUrl($idShop, $idLang, &$result, &$msg)
{
    $isDedicatedPageEnabled = SniperCommon::getConfig('dedi_page_enable', $idShop, $idLang);
    if (!$isDedicatedPageEnabled) {
        $result = '';
        $msg = 'dedicated_disabled';
        return false;
    }
    $dedicatedPageId = SniperCommon::getConfig('dedi_page_id', $idShop, $idLang);
    $link = new Link();
    $cms = new CMS($dedicatedPageId);
    $result = $link->getCMSLink($cms);
    $msg = "dedicated_success";
    return true;
}

function saveJobId($idJob, $idShop, $idLang, &$msg)
{
    if (!is_numeric($idJob)) {
        $msg = 'invalid_id_job';
        return false;
    }
    SniperCommon::setConfig('last_job_id', $idJob, $idShop, $idLang);
    $msg = 'id_job_save_success';
    return true;
}

function saveStatsOptions($exludedIps, $idShop, $idLang, &$result, &$msg)
{
    $exludedIps = explode(',', $exludedIps);
    $validIps = array();
    foreach ($exludedIps as $exclIp) {
        $exclIp = trim($exclIp);
        if (filter_var($exclIp, FILTER_VALIDATE_IP)) {
            $validIps[] = $exclIp;
        }
    }
    $validIps = implode(',', $validIps);
    SniperCommon::setConfig('excluded_ips', $validIps, $idShop, $idLang);
    $msg = 'valid_ip_success';
    $result = $validIps;
    return true;
}
