<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Context;
use \Customer;
use \Product;
use \Cart;

class CreateCartHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_customer', 'is_numeric')) {
            return error('invalid_id_customer');
        }

        if (!$request->has('key')) {
            return error('invalid_key');
        }

        if (!$request->has('id_currency', 'is_numeric')) {
            return error('invalid_id_currency');
        }

        if (!$request->has('id_lang', 'is_numeric')) {
            return error('invalid_id_lang');
        }

        if (!$request->has('cart', 'is_array')) {
            return error('invalid_cart_data');
        }

        $cart_detail = $request->get('cart');

        $id_customer = (int)$request->get('id_customer');
        $id_currency = (int)$request->get('id_currency');
        $id_lang = (int)$request->get('id_lang');
        $customer = new Customer($id_customer);

        if ($customer->secure_key != $request->get('key')) {
            return error('invalid_key');
        }

        $context = Context::getContext();
        $customer->logged = 1;
        $context->customer = $customer;

        $cart = new Cart();
        $context->cart = $cart;
        $cart->id_currency = $id_currency; //mandatory field
        $cart->id_customer = (int)$customer->id;
        $cart->id_lang = $id_lang;
        $cart->secure_key = $customer->secure_key;
        $cart->save();
        // for now we assume the cart always contains at least one product
        foreach ($cart_detail as $cart_product) {
            // check if product has attributes - if yes set the attribute and send it to the cart
            if (isset($cart_product['id_product_attribute']) && $cart_product['id_product_attribute'] > 0) {
                $id_product_attribute = $cart_product['id_product_attribute'];
            } else {
                /* we make sure that default attribute is always set.
                If product have no attribute "0" will be returned, so it's ok. */
                $id_product_attribute = Product::getDefaultAttribute($cart_product['id_product']);
            }
            $cart->updateQty($cart_product['quantity'], $cart_product['id_product'], (int)$id_product_attribute);
        }

        return ['id_cart' => (int)$cart->id];
    }
}
