<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;


use \Tools;
use \CustomerThread;
use \CustomerMessage;
use \Validate;

class CustomerMessageHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if ($request->isPost()) {
            return $this->newMessage($request);
        }

        return error('Invalid request');
    }

    private function newMessage(Request $request)
    {
        if (!$request->has('id_shop', 'is_numeric')) {
            return error('invalid_shop_id');
        }
        if (!$request->has('id_lang', 'is_numeric')) {
            return error('invalid_lang_id');
        }
        if (!$request->has('email', 'is_string')) {
            return error('invalid_email');
        }
        if (!$request->has('message', 'is_string')) {
            return error('invalid_message');
        }
        if (!$request->has('reason', 'is_numeric')) {
            return error('invalid_reason');
        }

        $shopId = (int)$request->get('id_shop');
        $langId = (int)$request->get('id_lang');
        $email = $request->get('email');
        $msg = $request->get('message');
        $reasonId = (int)$request->get('reason');

        if (!Validate::isEmail($email)) {
            return error('invalid_email');
        }

        $thread = new CustomerThread();
        $thread->token = Tools::passwdGen(12, 'ALPHANUMERIC');
        $thread->id_shop = $shopId;
        $thread->id_lang = $langId;
        $thread->id_contact = $reasonId;
        $thread->email = $email;
        $thread->status = 'open';

        if ($request->has('id_customer', 'is_numeric')) {
            $customerId = (int)$request->get('id_customer');
            $thread->id_customer = $customerId;
        }

        try {
            $thread->save();
        } catch (Exception $e) {
            return error('error_saving_thread');
        }

        $message = new CustomerMessage();
        $message->id_customer_thread = $thread->id;
        $message->message = $msg;
        $message->user_agent = "prestalia_app";

        try {
            $message->save();
        } catch (Exception $e) {
            return error('error_saving_message');
        }

        return success();
    }
}
