<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Configuration;
use \Currency;
use \Db;
use \Module;
use \SniperCommon;
use \SniperTemplateLoader;

class GetAppConfigHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_shop', 'is_numeric')) {
            return error('missing_shop_id');
        }
        // if (!$request->has('id_lang', 'is_numeric')) {
        //     return error('Invalid or missing id_lang');
        // }

        $p = _DB_PREFIX_;
        $shopId = (int)$request->get('id_shop');
        $langId = (int)$request->get('id_lang');

        if (empty($langId)) {
            $langId = Configuration::get('PS_LANG_DEFAULT', null, null, $shopId);
        }

        $sniperKey = SniperCommon::getConfig('sniper_key', $shopId, $langId);
        $sniperIndex = SniperCommon::getConfig('sniper_index_name', $shopId, $langId);
        $priceToken = SniperCommon::getConfig('price_token', $shopId, $langId);

        $sql = "select conf_key, conf_value from {$p}sniperfast_app where id_shop = {$shopId}";

        $res = Db::getInstance()->executeS($sql);

        if (!$res) {
            return error('fetch_error');
        }

        $c = array(
            'app_contact_email' => '',
            'app_contact_gmaps' => '',
            'app_contact_phone' => '',
            'app_contact_whatsapp' => '',
            'app_img_logo' => '',
            'app_social_fb' => '',
            'app_social_ig' => '',
            'app_social_tw' => '',
            'app_social_yt' => '',
            'app_tpl_name' => 'White'
        );

        foreach ($res as $e) {
            $c[$e['conf_key']] = $e['conf_value'];
        }

        $sniperTemplateLoader = new SniperTemplateLoader($shopId);
        $templateData = $sniperTemplateLoader->getTemplateData($c['app_tpl_name']);
        $estimatedDeliveryModuleIsInstalled = Module::isInstalled('estimateddelivery');

        $idCurrency = (int)Configuration::get('PS_CURRENCY_DEFAULT');
        $currency = new Currency($idCurrency);

        $out = [
            'sniperfast' => [
              'indexName' => $sniperIndex,
              'indexKey' => $sniperKey,
              'priceToken' => $priceToken
            ],
            'social' => [
                'facebookUrl' => $c['app_social_fb'],
                'instagramUrl' => $c['app_social_ig'],
                'twitterUrl' => $c['app_social_tw'],
                'youtubeUrl' => $c['app_social_yt'],
            ],
            'shopinfo' => [
                'whatsapp' => $c['app_contact_whatsapp'],
                'map' => $c['app_contact_gmaps'],
                'phoneNumber' => $c['app_contact_phone'],
                'mail' => $c['app_contact_email']
            ],
            'color' => $templateData,
            'additional' => [
                'estimated_delivery_installed' => $estimatedDeliveryModuleIsInstalled
            ],
            'currency' => [
                'code' => $currency->iso_code,
                'id' => $currency->id,
                'name' => $currency->name,
            ]
        ];

        return ['config' => $out];
    }
}
