<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Db;

class GetAttributesForGroupHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_lang', 'is_numeric')) {
            return error('invalid_id_lang');
        }

        if (!$request->has('id_product', 'is_numeric')) {
            return error('invalid_id_product');
        }

        if (!$request->has('id_attribute_group', 'is_numeric')) {
            return error('invalid_id_attribute_group');
        }

        if (!$request->has('attributes', 'is_array')) {
            return error('invalid_attributes');
        }

        $id_lang = (int)$request->get('id_lang');
        $id_product = (int)$request->get('id_product');
        $id_attribute_group = (int)$request->get('id_attribute_group');
        $p = _DB_PREFIX_;
        $attributes = $request->get('attributes');

        if (empty($attributes)) {
            $append = " AND pagl.id_attribute_group = {$id_attribute_group}";
        } else {
            $attributesList = array_map(
                function ($e) {
                    return (int)$e['idAttribute'];
                },
                $attributes
            );

            $attributesCount = count($attributesList);
            $attributesList = implode(',', $attributesList);

            $append = "AND pagl.id_attribute_group = {$id_attribute_group}
                AND ppac.id_product_attribute IN
                (SELECT ppac.id_product_attribute
                FROM {$p}product_attribute ppa
                JOIN {$p}product_attribute_combination ppac
	                  ON ppa.id_product_attribute = ppac.id_product_attribute
                JOIN {$p}stock_available s
                ON ppa.id_product_attribute = s.id_product_attribute
                WHERE ppa.id_product = {$id_product}
                AND ppac.id_attribute IN ({$attributesList})
                AND s.quantity > 0
                GROUP BY ppa.id_product_attribute
                HAVING count(*) = {$attributesCount})";
        }

        $sql = "SELECT DISTINCT  pal.name, pal.id_attribute
            FROM {$p}attribute_group_lang pagl
            JOIN {$p}attribute_group pag
            ON pag.id_attribute_group = pagl.id_attribute_group
            JOIN {$p}attribute pa
            ON pa.id_attribute_group = pag.id_attribute_group
            JOIN {$p}attribute_lang pal
            ON pal.id_attribute = pa.id_attribute
            JOIN {$p}product_attribute_combination ppac
            ON ppac.id_attribute = pa.id_attribute
            JOIN {$p}product_attribute ppa
            ON ppa.id_product_attribute = ppac.id_product_attribute
            WHERE pal.id_lang = {$id_lang}
            AND ppa.id_product = {$id_product} {$append}";

        $res = Db::getInstance()->executeS($sql);

        if (!$res) {
            return ['attributes' => []];
        }

        $out = [];

        foreach ($res as $e) {
            $out[] = [
                'idAttribute' => (int)$e['id_attribute'],
                'name' => $e['name']
            ];
        }

        return ['attributes' => $out];
    }
}
