<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Db;
use \Context;
use \Customer;
use \Product;
use \Language;
use \Shop;
use \Employee;
use \Currency;

class GetCombinationInfoHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_lang', 'is_numeric')) {
            return error('invalid_id_lang');
        }

        if (!$request->has('id_product', 'is_numeric')) {
            return error('invalid_id_product');
        }

        if (!$request->has('id_currency', 'is_numeric')) {
            return error('invalid_id_currency');
        }

        if (!$request->has('attributes', 'is_array')) {
            return error('invalid_attributes');
        }

        $id_lang = (int)$request->get('id_lang');
        $id_product = (int)$request->get('id_product');
        $id_currency = (int)$request->get('id_currency');
        $attributes = $request->get('attributes');
        $id_customer = null;

        if ($request->has('id_customer', 'is_numeric')) {
            $id_customer = (int)$request->get('id_customer');
        }

        if (empty($attributes)) {
            return error('empty_attributes');
        }

        $attributesList = array_map(
            function ($e) {
                return (int)$e['idAttribute'];
            },
            $attributes
        );

        $attributesCount = count($attributesList);
        $attributesList = implode(',', $attributesList);

        $p = _DB_PREFIX_;
        $sql = "SELECT ppa.id_product_attribute
            FROM {$p}product_attribute_combination ppac
            JOIN {$p}product_attribute ppa
            ON ppac.id_product_attribute = ppa.id_product_attribute
            WHERE id_attribute IN ({$attributesList})
            AND ppa.id_product = {$id_product}
            GROUP BY ppa.id_product_attribute
            HAVING COUNT(*) = {$attributesCount}";

        $res = Db::getInstance()->executeS($sql);

        if (!$res) {
            return error('no_id_product_attribute');
        }

        if (count($res) > 1) {
            return error('multiple_id_product_attribute');
        }

        $id_product_attribute = (int)$res[0]['id_product_attribute'];

        $id_employee = ID_EMPLOYEE;
        $context = Context::getContext();
        $context->cart = null;
        $context->customer = new Customer($id_customer);
        $context->language = new Language($id_lang, $id_lang);
        $context->shop = new Shop(1, $id_lang, 1);
        $context->employee = new Employee($id_employee, null, $id_employee);
        $context->currency = new Currency($id_currency);

        $prod = new Product($id_product, true, $id_lang);
        $result = $prod->getAttributeCombinationsById($id_product_attribute, $id_lang);

        $output = array();
        foreach ($result as $tmp) {
            // parse and semplify details for the retrieved id_product_attribute
            $output['idProductAttribute'] = (int)$tmp['id_product_attribute'];
            $output['idProduct'] = (int)$id_product;
            $output['reference'] = $tmp['reference'];
            $output['price'] =
                Product::getPriceStatic(
                    (int)$id_product,
                    true,
                    (int)$tmp['id_product_attribute'],
                    2,
                    null,
                    false,
                    false,
                    1,
                    false,
                    $id_customer
                );
            $output['priceDiscount'] =
                Product::getPriceStatic(
                    (int)$id_product,
                    true,
                    (int)$tmp['id_product_attribute'],
                    2,
                    null,
                    false,
                    true,
                    1,
                    false,
                    $id_customer
                );
        }

        return ['combination' => $output];
    }
}
