<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Validate;
use \Customer;

class GetCustomerHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('email') || !$request->has('password')) {
            return error('invalid_user_or_password');
        }

        $email = (string)$request->get('email');
        $password = (string)$request->get('password');

        if (!Validate::isEmail($email)) {
            return error('invalid_email');
        }

        if (!Validate::isPasswd($password)) {
            return error('invalid_password');
        }

        $customer = new Customer();

        $res = $customer->getByEmail($email, $password);
        if (!$res || !$res->active) {
            return error('invalid_credentials');
        }

        return ['customer' => [
            'id' => (int)$res->id,
            'secureKey' => $res->secure_key,
            'idShop' => (int)$res->id_shop,
            'idShopGroup' => (int)$res->id_shop_group,
            'email' => $res->email,
            'firstname' => $res->firstname,
            'lastname' => $res->lastname
        ]];
    }
}
