<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Configuration;
use \Context;
use \Language;
use \Module;
use \Estimateddelivery;

class GetDeliveryTimeHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_lang', 'is_numeric')) {
            return error('invalid_id_lang');
        }

        if (!$request->has('id_product', 'is_numeric')) {
            return error('invalid_id_product');
        }

        if (!$request->has('id_product_attribute', 'is_numeric')) {
            return error('invalid_id_product_attribute');
        }

        $id_product = (int)$request->get('id_product');
        $id_product_attribute = (int)$request->get('id_product_attribute');

        if (!$request->has('id_carrier', 'is_numeric')) {
            $id_carrier = (int)Configuration::get('PS_CARRIER_DEFAULT');
        } else {
            $id_carrier = (int)$request->get('id_carrier');
        }

        $id_lang = (int)$request->get('id_lang');
        $context = Context::getContext();
        $context->language = new Language($id_lang, $id_lang);

        try {
            $ees = new Estimateddelivery();
        } catch (Exception $e) {
            $output = array(
                "success" => 0,
                'reason' => 'no_data',
                "daily_picking_limit" => '',
                "carrier_name" => '',
                "delivery_diff" => '',
                "delivery_min" => '',
                "delivery_max" => '',
            );
            return ['delivery' => $output];
        }
        $ees->setCarriersFromIds(array($id_carrier));
        $data = $ees->showEstimatedDelivery(null, $id_product, $id_product_attribute, null, 'array', null);

        $output = array();
        if (isset($data['deliveries'])) {
            foreach ($data['deliveries'] as $tmp) {
                $output['success'] = 1;
                $output['daily_picking_limit'] = $tmp['picking_limit'];
                $output['carrier_name'] = $tmp['name'];
                $output['delivery_diff'] = $tmp['delivery_diff'];
                $output['delivery_min'] = $tmp['delivery_min'];
                $output['delivery_max'] = $tmp['delivery_max'];
            }
        } else {
            $output = array(
                "success" => 0,
                'reason' => 'no_data',
                "daily_picking_limit" => '',
                "carrier_name" => '',
                "delivery_diff" => '',
                "delivery_min" => '',
                "delivery_max" => '',
            );
        }

        return ['delivery' => $output];
    }
}
