<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Db;
use \SniperCommon;
use \Tools;

class GetInfoPagesHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_shop', 'is_numeric')) {
            return error('Invalid or missing id_shop');
        }
        if (!$request->has('id_lang', 'is_numeric')) {
            return error('Invalid or missing id_lang');
        }
        if (!$request->has('pages', 'is_array')) {
            return error('Invalid or missing pages');
        }

        $shopId = (int)$request->get('id_shop');
        $langId = (int)$request->get('id_lang');

        $cmsPages = SniperCommon::getAppConfig('sniper_app_cms', $shopId);

        $p = _DB_PREFIX_;
        if (Tools::version_compare(_PS_VERSION_, '1.6.1.0', '>=')) {
            $sql = "select id_cms as id, meta_title as title, content from {$p}cms_lang
                where id_shop = {$shopId} and id_lang = {$langId} and id_cms in (".pSQL($cmsPages).")";
        } else {
            $sql = "select id_cms as id, meta_title as title, content from {$p}cms_lang
                where id_lang = {$langId} and id_cms in (".pSQL($cmsPages).")";
        }
        $res = Db::getInstance()->executeS($sql);

        $response = ['pages' => []];
        foreach ($res as $page) {
            $response['pages'][] = [
                'id' => (int)$page['id'],
                'title' => $page['title'],
                'content' => $page['content']
            ];
        }

        return $response;
    }
}
