<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Context;
use \Db;
use \Language;
use \Tools;
use \SniperCommonApp;

class GetProductSlidersHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        ob_start();

        if (!$request->has('id_shop', 'is_numeric')) {
            return error('Invalid or missing id_shop');
        }
        if (!$request->has('id_lang', 'is_numeric')) {
            return error('Invalid or missing id_lang');
        }

        $shopId = (int)$request->get('id_shop');
        $langId = (int)$request->get('id_lang');
        $context = Context::getContext();
        $context->language = new Language($langId, $langId);

        $res = SniperCommonApp::getAllAppSlider($shopId, $langId);

        $response = ['sliders' => []];
        foreach ($res as $slider) {
            $query = array('q' => '');
            $params = array();
            $productQuery = null;
            $type = '';
            switch ($slider['source']) {
                case 1:
                    $params = array('sort' => 'best');
                    $type = 'mostSold';
                    break;
                case 2:
                    $params = SniperCommonApp::getCategoryPathForQuery($slider['source_data']);
                    $productQuery = array();
                    foreach ($params as $key => $val) {
                        $productQuery[] = $val;
                    }
                    $type = 'category';
                    break;
                case 3:
                    $params = SniperCommonApp::getManufacturerForQuery($slider['source_data']);
                    $productQuery = $params['snmanu'];
                    $type = 'manufacturer';
                    break;
                case 4:
                    $productQuery = explode(',', $slider['source_data']);
                    $type = 'products';
                    break;
                case 5:
                    $params = SniperCommonApp::getCategoryPathForQuery($slider['source_data']);
                    $productQuery = array();
                    foreach ($params as $key => $val) {
                        $productQuery[] = $val;
                    }
                    $params['sort'] = 'id_prod';
                    $type = 'newarrivals_cat';
                    break;
                case 6:
                    $params = SniperCommonApp::getManufacturerForQuery($slider['source_data']);
                    $productQuery = $params['snmanu'];
                    $params['sort'] = 'id_prod';
                    $type = 'newarrivals_manu';
                    break;
                case 7:
                    $params = array('sort' => 'id_prod');
                    $type = 'newarrivals_prod';
                    break;
            }

            $query = array_merge($query, $params);

            $response['sliders'][] = [
                'name' => $slider['name'],
                'source' => $type,
                'source_data' => $productQuery,
                'query' => $query
            ];
        }

        ob_get_clean();
        return $response;
    }
}
