<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Context;
use \Db;
use \Language;
use \Tools;
use \SniperCommonApp;

class GetSearchBannersHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        ob_start();

        if (!$request->has('id_shop', 'is_numeric')) {
            return error('Invalid or missing id_shop');
        }
        if (!$request->has('id_lang', 'is_numeric')) {
            return error('Invalid or missing id_lang');
        }

        $shopId = (int)$request->get('id_shop');
        $langId = (int)$request->get('id_lang');
        $context = Context::getContext();
        $context->language = new Language($langId, $langId);

        $res = SniperCommonApp::getAllAppBanners($shopId, $langId);

        $response = ['banners' => []];
        foreach ($res as $banner) {
            $query = array('q' => '');
            $params = array();
            $type = 'data';
            $additionalData = '';
            switch ($banner['source']) {
                case 0:
                    $type = 'img';
                    break;
                case 1:
                    $params = array('sort' => 'best');
                    break;
                case 2:
                    $params = SniperCommonApp::getCategoryPathForQuery($banner['source_data']);
                    break;
                case 3:
                    $params = SniperCommonApp::getManufacturerForQuery($banner['source_data']);
                    break;
                case 4:
                    $additionalData = $banner['source_data'];
                    $type = 'prod';
                    break;
                case 5:
                    $params = SniperCommonApp::getCategoryPathForQuery($banner['source_data']);
                    $params['sort'] = 'id_prod';
                    break;
                case 6:
                    $params = SniperCommonApp::getManufacturerForQuery($banner['source_data']);
                    $params['sort'] = 'id_prod';
                    break;
                case 7:
                    $params = array('sort' => 'id_prod');
                    break;
            }

            $query = array_merge($query, $params);

            $response['banners'][] = [
                'name' => $banner['name'],
                'image' => $banner['image'],
                'query' => $query,
                'type' => $type,
                'additional' => $additionalData
            ];
        }

        ob_get_clean();
        return $response;
    }
}
