<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Tools;
use \SniperCommon;

class GetTranslationFileHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_shop', 'is_numeric')) {
            header("HTTP/1.0 404 Not Found");
            return error('Invalid or missing id_shop');
        }
        if (!$request->has('lang', 'is_string')) {
            header("HTTP/1.0 404 Not Found");
            return error('Invalid or missing lang');
        }

        $shopId = (int)$request->get('id_shop');
        $lang = (string)$request->get('lang');
        $appDefaultLanguage = SniperCommon::getAppConfig('app_default_language', $shopId);

        if (file_exists(dirname(__FILE__).'/../../resources/app/translations/'.$shopId.'_'.$lang.'.json')) {
            $jsonDecoded = json_decode(
                Tools::file_get_contents(
                    dirname(__FILE__).'/../../resources/app/translations/'.$shopId.'_'.$lang.'.json'
                ),
                true
            );
            return $jsonDecoded;
        } elseif (file_exists(dirname(__FILE__).'/../../resources/app/translations/'.$lang.'.json')) {
            $jsonDecoded = json_decode(
                Tools::file_get_contents(
                    dirname(__FILE__).'/../../resources/app/translations/'.$lang.'.json'
                ),
                true
            );
            return $jsonDecoded;
        } elseif (file_exists(dirname(__FILE__).'/../../resources/app/translations/'.$appDefaultLanguage.'.json')) {
            $jsonDecoded = json_decode(
                Tools::file_get_contents(
                    dirname(__FILE__).'/../../resources/app/translations/'.$appDefaultLanguage.'.json'
                ),
                true
            );
            return $jsonDecoded;
        } else {
            header("HTTP/1.0 404 Not Found");
            return error('Missing language file');
        }
    }
}
