<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Context;
use \Customer;
use \Cart;
use \CartRule;
use \Tools;

class RedirectCheckoutHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_customer', 'is_numeric')) {
            return error('invalid_id_customer');
        }

        if (!$request->has('key')) {
            return error('invalid_key');
        }

        if (!$request->has('id_currency', 'is_numeric')) {
            return error('invalid_id_currency');
        }

        if (!$request->has('id_cart', 'is_numeric')) {
            return error('invalid_id_cart');
        }

        $id_customer = (int)$request->get('id_customer');
        $id_currency = (int)$request->get('id_currency');
        $id_cart = (int)$request->get('id_cart');
        $customer = new Customer($id_customer);

        if ($customer->secure_key != $request->get('key')) {
            return error('invalid_key');
        }

        $context = Context::getContext();
        $context->smarty->assign('confirmation', 1);
        $context->cookie->id_currency = $id_currency;
        $context->cookie->id_customer = (int)$customer->id;
        $context->cookie->customer_lastname = $customer->lastname;
        $context->cookie->customer_firstname = $customer->firstname;
        $context->cookie->passwd = $customer->passwd;
        $context->cookie->logged = 1;
        $context->cookie->email = $customer->email;
        $customer->logged = 1;
        $context->customer = $customer;
        $context->cart = new Cart($id_cart);
        $context->cookie->id_cart = (int)$context->cart->id;

        // custom cookie parameter in order to dinamically load app.css from cssinjector module
        $context->cookie->in_app = 1;

        // according to others programmatic cart creator samples
        // it looks like it's good to have those two methods called
        CartRule::autoRemoveFromCart($context);
        CartRule::autoAddToCart($context);

        $context->cookie->write();
        Tools::redirect('index.php?controller=order');
    }
}
