<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \pSQL;
use \Validate;
use \Customer;

class SignupHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        $keys = [
            ['key' => 'firstname', 'check' => 'Validate::isName'],
            ['key' => 'lastname', 'check' => 'Validate::isName'],
            ['key' => 'email', 'check' => 'Validate::isEmail'],
            ['key' => 'password', 'check' => 'Validate::isPasswd'],
            ['key' => 'newsletter', 'check' => 'is_bool'],
        ];

        foreach ($keys as $key) {
            if (!$request->has($key['key'], $key['check'])) {
                return error("invalid_{$key['key']}");
            }
        }

        if (Customer::customerExists($request->get('email'), true)) {
            return error('email_already_exists');
        }

        $customer = new Customer();
        $customer->lastname = $request->get('lastname');
        $customer->firstname = $request->get('firstname');
        $customer->passwd = md5(pSQL(_COOKIE_KEY_ . $request->get('password')));
        $customer->email = $request->get('email');
        $customer->newsletter = (int)$request->get('newsletter');
        $customer->active = 1;

        if ($customer->add()) {
            return ['id_customer' => (int)$customer->id];
        } else {
            return error('error_creating_user');
        }
    }
}
