<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

class SniperCommon
{
    public static function installTab($parent, $className, $name, $thisName)
    {
        // Create new admin tab
        $tab = new Tab();

        $tab->id_parent = (int)Tab::getIdFromClassName($parent);
        $tab->name      = array();

        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $name;
        }

        $tab->class_name = $className;
        $tab->module     = $thisName;
        $tab->active     = 1;

        return $tab->add();
    }

    public static function uninstallTab($className)
    {
        // Retrieve Tab ID
        $idTab = (int)Tab::getIdFromClassName($className);

        // Load tab
        $tab = new Tab((int)$idTab);

        // Delete it
        return $tab->delete();
    }

    public static function checkConfig($idShop, $idLang)
    {
        // 'key' => 'default value' pairs
        $config = array(
            'sniper_default' => '1',
            'exports_gap' => '5',
        );

        foreach ($config as $key => $val) {
            if (self::getConfig($key, $idShop, $idLang) == '') {
                self::setConfig($key, $val, $idShop, $idLang);
            }
        }

        if (self::getConfig('ajax_token', $idShop, $idLang) == '') {
            self::generateNewAjaxToken($idShop, $idLang);
        }

        if (self::getConfig('price_token', $idShop, $idLang) == '') {
            self::generateNewPriceToken($idShop, $idLang);
        }
    }

    public static function setConfig($key, $value, $idShop, $idLang)
    {
        if ($key == null || $key == "") {
            return false;
        }

        $table    = _DB_PREFIX_.'sniperfast_conf';
        $key      = pSQL($key);
        $value    = pSQL($value);
        $idShop   = (int)$idShop;
        $idLang   = (int)$idLang;

        // All the variables in this query are already casted right above
        $sql = "INSERT INTO $table
            VALUES ('$key', $idShop, $idLang, '$value')
            ON DUPLICATE KEY
            UPDATE conf_value = '$value'";

        return Db::getInstance()->execute($sql);
    }

    public static function setAppConfig($key, $value, $idShop)
    {
        if ($key == null || $key == "") {
            return false;
        }

        $table    = _DB_PREFIX_.'sniperfast_app';
        $key      = pSQL($key);
        $value    = pSQL($value);
        $idShop   = (int)$idShop;

        // All the variables in this query are already casted right above
        $sql = "INSERT INTO $table
            VALUES ($idShop, '$key', '$value')
            ON DUPLICATE KEY
            UPDATE conf_value = '$value'";

        return Db::getInstance()->execute($sql);
    }

    public static function getConfig($key, $idShop, $idLang)
    {
        if (empty($key) || empty($idShop) || empty($idLang)) {
            return false;
        }

        $table    = _DB_PREFIX_.'sniperfast_conf';
        $key      = pSQL($key);
        $idShop   = (int)$idShop;
        $idLang   = (int)$idLang;

        // All the variables in this query are already casted right above
        $sql = "SELECT conf_value
            FROM $table
            WHERE conf_key = '$key'
            AND id_shop = $idShop
            AND id_lang = $idLang";
        $res = Db::getInstance()->executeS($sql);
        if (empty($res)) {
            return '';
        }

        return $res[0]['conf_value'];
    }

    public static function getAppConfig($key, $idShop)
    {
        if (empty($key) || empty($idShop)) {
            return false;
        }

        $table    = _DB_PREFIX_.'sniperfast_app';
        $key      = pSQL($key);
        $idShop   = (int)$idShop;

        // All the variables in this query are already casted right above
        $sql = "SELECT conf_value
            FROM $table
            WHERE conf_key = '$key'
            AND id_shop = $idShop";
        $res = Db::getInstance()->executeS($sql);
        if (empty($res)) {
            return '';
        }

        return $res[0]['conf_value'];
    }

    public static function generateNewAjaxToken($idShop, $idLang)
    {
        $token = bin2hex(openssl_random_pseudo_bytes(16));
        return self::setConfig('ajax_token', $token, $idShop, $idLang);
    }

    public static function generateNewPriceToken($idShop, $idLang)
    {
        $token = bin2hex(openssl_random_pseudo_bytes(16));
        return self::setConfig('price_token', $token, $idShop, $idLang);
    }

    public static function cmsPageExists($dedicatedPageId, $idShop)
    {
        $sql = "SELECT * FROM "._DB_PREFIX_."cms
            WHERE id_cms = ".(int)$dedicatedPageId;
        $cmsRes = Db::getInstance()->executeS($sql);
        if (empty($cmsRes)) {
            $sql = "UPDATE "._DB_PREFIX_."sniperfast_conf
                SET conf_value = 0
                WHERE conf_key = 'dedi_page_id'
                AND id_shop = ".(int)$idShop;
            Db::getInstance()->execute($sql);
            return false;
        }
        return true;
    }

    public static function updateMainData($idShop, $idLang, &$errorMsg)
    {
        $sourceFile = dirname(__FILE__).'/../views/js/sniperfast_data_source.js';
        $template = Tools::file_get_contents($sourceFile);
        if ($template === false) {
            $errorMsg = "Can\'t open template file";
            return false;
        }

        // Features
        $featureRangeFile = dirname(__FILE__).'/../output/'.$idShop.'_'.$idLang.'_range_feature.json';
        $featureRange = '';
        if (file_exists($featureRangeFile)) {
            $featureRangeFileEncoded = Tools::file_get_contents($featureRangeFile);
            $featureRangeFileDecoded = Tools::jsonDecode($featureRangeFileEncoded, true);
            if (count($featureRangeFileDecoded) > 0) {
                $tempFeatures = array();
                foreach ($featureRangeFileDecoded as $f) {
                    if ($f['is_range'] == 1) {
                        $tempFeatures[$f['name']] = 1;
                    }
                }
                if (count($tempFeatures) > 0) {
                    $featureRange = Tools::jsonEncode($tempFeatures);
                }
            }
        }

        // Dedicated page
        $dedicatedPageUrl = null;
        if (SniperCommon::getConfig('dedi_page_enable', $idShop, $idLang)) {
            $dedicatedPageId = SniperCommon::getConfig('dedi_page_id', $idShop, $idLang);
            $cmsPage = new CMS((int)$dedicatedPageId);
            $link = new Link();
            $dedicatedPageUrl = $link->getCMSLink($cmsPage, null, null, $idLang);
        }

        $idCurrency = SniperCommon::getConfig('sniper_currency', $idShop, $idLang);
        $sql = "SELECT iso_code FROM "._DB_PREFIX_."currency
            WHERE id_currency = ".(int)$idCurrency;
        $currencyIsoCode = Db::getInstance()->getValue($sql);

        $submittedValues = array(
            '[[sniper_div_dsk]]' => SniperCommon::getConfig('sniper_div_dsk', $idShop, $idLang),
            '[[sniper_div_mob]]' => SniperCommon::getConfig('sniper_div_mob', $idShop, $idLang),
            '[[sniper_index_name]]' => SniperCommon::getConfig('sniper_index_name', $idShop, $idLang),
            '[[sniper_key]]' => SniperCommon::getConfig('sniper_key', $idShop, $idLang),
            '[[sniper_export_review]]' => SniperCommon::getConfig('sniper_export_review', $idShop, $idLang),
            '[[sniper_page_width]]' => 992,
            '[[warehouse_compatibility]]' => SniperCommon::getConfig('warehouse_compatibility', $idShop, $idLang),
            '[[megashop_compatibility]]' => SniperCommon::getConfig('megashop_compatibility', $idShop, $idLang),
            '[[sniperfast_lang_iso]]' => Language::getIsoById($idLang),
            '[[sniperfast_dedicate_page]]' => $dedicatedPageUrl,
            '[[sniperfast_feature_range]]' => $featureRange ,
            '[[sniper_result_width]]' => SniperCommon::getConfig('sniper_result_width', $idShop, $idLang),
            '[[sniper_result_align]]' => SniperCommon::getConfig('sniper_result_align', $idShop, $idLang),
            '[[sniper_price_token]]' => SniperCommon::getConfig('price_token', $idShop, $idLang),
            '[[sniper_currency_iso]]' => $currencyIsoCode,
            '[[sniper_hide_cat]]' => SniperCommon::getConfig('hide_cat', $idShop, $idLang),
            '[[sniper_hide_manu]]' => SniperCommon::getConfig('hide_manu', $idShop, $idLang)
        );

        $template = str_replace(
            array_keys($submittedValues),
            $submittedValues,
            $template
        );

        $date               = new DateTime();
        $dateFormat         = $date->format("Y-m-d_H-i-s");
        $newTemplateName    = md5($dateFormat);
        $newTemplate        = dirname(__FILE__).'/../views/js/'.$idShop.'_'.$idLang.'_'.$newTemplateName.'.js';
        if (!file_put_contents($newTemplate, $template)) {
            $errorMsg = "Can\'t write template file";
            return false;
        } else {
            $oldFile = SniperCommon::getConfig('source_data_file', $idShop, $idLang);
            $oldFilePath = dirname(__FILE__).'/../views/js/'.$idShop.'_'.$idLang.'_'.$oldFile.'.js';
            if (file_exists($oldFilePath)) {
                unlink($oldFilePath);
            }
            SniperCommon::setConfig('source_data_file', $newTemplateName, $idShop, $idLang);
        }
        return true;
    }

    public static function addProductToQueue($idProduct, $time, $idShop)
    {
        $sql = "INSERT INTO "._DB_PREFIX_."sniperfast_queue
            (id_product, id_shop, updated_on) VALUES (".
                (int)$idProduct.", ".
                (int)$idShop.", ".
                (int)$time.
            ") ON DUPLICATE KEY UPDATE updated_on = ".(int)$time;
        try {
            return Db::getInstance()->execute($sql);
        } catch (Exception $e) {
            //Do nothing
        }
    }

    public static function getLastIncrementalUpdate($idShop)
    {
        $sql = "SELECT conf_value FROM "._DB_PREFIX_."sniperfast_conf
            WHERE conf_key = 'last_incremental_update'
            AND id_shop = ".(int)$idShop.
            " AND id_lang = 131000";
        $lastUpdate = Db::getInstance()->getValue($sql);
        if (empty($lastUpdate)) {
            return '0_0';
        } else {
            return $lastUpdate;
        }
    }

    public static function setLastIncrementalUpdate($currentTime, $idProduct, $idShop)
    {
        self::setConfig('last_incremental_update', $idProduct.'_'.$currentTime, $idShop, 131000);
    }

    public static function getConfiguredLanguages($idShop)
    {
        $sql = "SELECT DISTINCT(id_lang) FROM  "._DB_PREFIX_."sniperfast_conf
            WHERE conf_key = 'sync_on_save'
            AND conf_value = 1
            AND id_shop = ".(int)$idShop;
        $res = Db::getInstance()->executeS($sql);
        $configuredLanguages = array();
        if (!empty($res)) {
            foreach ($res as $r) {
                $configuredLanguages[] = $r['id_lang'];
            }
        }
        return $configuredLanguages;
    }

    public static function getQueuedProducts($idShop)
    {
        $sql = "SELECT id_product FROM "._DB_PREFIX_."sniperfast_queue
            WHERE id_shop = ".(int)$idShop;
        $res = Db::getInstance()->executeS($sql);
        if (empty($res)) {
            return array();
        }

        $queuedProducts = array();
        foreach ($res as $r) {
            $queuedProducts[] = $r['id_product'];
        }
        return $queuedProducts;
    }

    public static function getConfigKeys($idShop, $idLang)
    {
        $configKeys = array(
            'exports_gap'                 => 'int',
            'sniper_features'             => 'string',
            'sniper_attributes'           => 'string',
            'image_format'                => 'int',
            'only_in_stock'               => 'int',
            'sniper_currency'             => 'int',
            'export_batch_size'           => 'int',
            'sniper_free_ship'            => 'int',
            'sniper_export_review'        => 'int'
        );
        $config = array();

        foreach ($configKeys as $key => $type) {
            $config[$key] = self::getConfig($key, $idShop, $idLang);

            if ($key == 'only_in_stock' && $config[$key] == '') {
                $config[$key] = 0;
                continue;
            }

            if ($key == 'sniper_currency' && $config[$key] == '') {
                $config[$key] = 0;
                continue;
            }

            if ($key == 'export_batch_size' && $config[$key] == '') {
                $config[$key] = 300;
                continue;
            }

            if ($key == 'sniper_free_ship' && $config[$key] == '') {
                $config[$key] = 0;
                continue;
            }
            if ($key == 'sniper_export_review' && $config[$key] == '') {
                $config[$key] = 0;
                continue;
            }

            if ($config[$key] == '') {
                if ($key == 'sniper_features' || $key == 'sniper_attributes') {
                    continue;
                }
                return array();
            }

            // Cast config value to the proper type
            settype($config[$key], $type);
        }

        $imgTypes    = ImageType::getImagesTypes();

        foreach ($imgTypes as $imgType) {
            if ($config['image_format'] == $imgType['id_image_type']) {
                $config['img_type'] = $imgType['name'];
                break;
            }
        }

        return $config;
    }

    public static function sendOrderDetails($idShop, $idLang, $orderData)
    {
        if (!self::getConfig('sniper_index_name', $idShop, $idLang)) {
            return;
        }
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL."://api.sniperfast.".CDN_ENDPOINT."/stats?type=send_order");
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query(array(
            'shop'  => $idShop,
            'lang'  => $idLang,
            'key'   => self::getConfig('sniper_key', $idShop, $idLang),
            'index' => self::getConfig('sniper_index_name', $idShop, $idLang),
            'order_data'  => $orderData,
            'ip'    => $_SERVER['HTTP_X_REAL_IP']
        )));

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_exec($ch);
        curl_close($ch);
    }
}
