<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

class SniperTemplateLoader
{
    private $table_prefix;
    private $db;
    private $idShop;
    private $templateFolder;
    private $customTemplateFolder;

    public function __construct($idShop)
    {
        $this->table_prefix = _DB_PREFIX_.'sniperfast_';
        $this->db = Db::getInstance();
        $this->idShop = $idShop;
        $this->templateFolder = _PS_MODULE_DIR_.'sniperfast/resources/app/template/';
        $this->customTemplateFolder = _PS_MODULE_DIR_.'sniperfast/resources/app/template_custom/';
    }

    public function getAvailableTemplates()
    {
        $availableTemplates = array();

        if (!file_exists($this->templateFolder)) {
            return $availableTemplates;
        }

        foreach (glob($this->templateFolder."*.json") as $fileName) {
            $fileNameExpl = explode('/', $fileName);
            $fileNameExpl = array_pop($fileNameExpl);
            $fileNameExpl = explode('.', $fileNameExpl);
            $fileNameExpl = array_shift($fileNameExpl);
            $availableTemplates[] = $fileNameExpl;
        }

        return $availableTemplates;
    }

    public function getDefaultTemplateData($templateName)
    {
        $defaultValues =
            Tools::jsonDecode(Tools::file_get_contents($this->templateFolder.$templateName.'.json'), true);
        return $defaultValues;
    }

    public function getTemplateData($templateName)
    {
        $defaultValues = $this->getDefaultTemplateData($templateName);
        if (!file_exists($this->customTemplateFolder.$this->idShop.'_'.$templateName.'.json')) {
            return $defaultValues;
        }

        $templateData = array();
        $templateKeys = array_keys($defaultValues);
        $customValues =
            Tools::jsonDecode(
                Tools::file_get_contents($this->customTemplateFolder.$this->idShop.'_'.$templateName.'.json'),
                true
            );

        foreach ($templateKeys as $key) {
            if (isset($customValues[$key]) && !empty($customValues[$key])) {
                $templateData[$key] = $customValues[$key];
            } else {
                $templateData[$key] = $defaultValues[$key];
            }
        }

        return $templateData;
    }

    public function saveTemplateSettings($appConfig, $templateName)
    {
        $defaultValues = $this->getDefaultTemplateData($templateName);
        $templateData = array();
        $templateKeys = array_keys($defaultValues);

        foreach ($templateKeys as $key) {
            if (isset($appConfig[$key]) && !empty($appConfig[$key])) {
                $templateData[$key] = $appConfig[$key];
            } else {
                $templateData[$key] = $defaultValues[$key];
            }
        }

        file_put_contents(
            $this->customTemplateFolder.$this->idShop.'_'.$templateName.'.json',
            Tools::jsonEncode($templateData)
        );
    }
}
