<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

require_once dirname(__FILE__).'/../autoload.php';

class SniperUpgradeHelper
{
    private $idShop;
    private $languages;
    
    public function __construct($idShop)
    {
        $this->idShop = $idShop;
        $this->languages = $this->getWebsiteLanguages($idShop);
    }
    
    public function convertDiv()
    {
        foreach ($this->languages as $lang) {
            $sniperDiv = SniperCommon::getConfig('sniper_div_id', $this->idShop, $lang['id']);
            if (empty($sniperDiv)) {
                continue;
            }
            
            $sniperDiv = explode(',', $sniperDiv);
            if (isset($sniperDiv[0])) {
                SniperCommon::setConfig('sniper_div_dsk', $sniperDiv[0], $this->idShop, $lang['id']);
            }
            if (isset($sniperDiv[1])) {
                SniperCommon::setConfig('sniper_div_mob', $sniperDiv[1], $this->idShop, $lang['id']);
            }
        }
    }
    
    public function updateMainFile()
    {
        $path = _PS_MODULE_DIR_.'sniperfast';
        foreach ($this->languages as $lang) {
            $sourceFile = '';
            if ($lang['iso_code'] == 'it') {
                $sourceFile = $path.'/views/js/main_it.js';
            } else {
                $sourceFile = $path.'/views/js/main_en.js';
            }
            
            if (!file_exists($sourceFile)) {
                continue;
            }

            $sniperIndex  = SniperCommon::getConfig('sniper_index_name', $this->idShop, $lang['id']);
            if (empty($sniperIndex)) {
                continue;
            }
            $submittedValues = array();
            $template = Tools::file_get_contents($sourceFile);
            if ($template === false) {
                continue;
            }

            $template = str_replace(
                array_keys($submittedValues),
                $submittedValues,
                $template
            );

            $date               = new DateTime();
            $dateFormat         = $date->format("Y-m-d_H-i-s");
            $newTemplateName    = md5($dateFormat).$lang['id'];
            $newTemplate        = $path.'/views/js/'.$newTemplateName.'.js';
            if (!file_put_contents($newTemplate, $template)) {
                continue;
            } else {
                $oldFile = SniperCommon::getConfig('main_data_file', $this->idShop, $lang['id']);
                $oldFilePath = $path.'/views/js/'.$oldFile.'.js';
                if (file_exists($oldFilePath)) {
                    unlink($oldFilePath);
                }
                SniperCommon::setConfig('main_data_file', $newTemplateName, $this->idShop, $lang['id']);
            }
        }
    }
    
    public function updateMainFile105()
    {
        $path = _PS_MODULE_DIR_.'sniperfast';
        $sourceFile = $path.'/views/js/sniperfast_data_source.js';
        if (!file_exists($sourceFile)) {
            return;
        }
        
        $template = Tools::file_get_contents($sourceFile);
        if ($template === false) {
            return;
        }
        foreach ($this->languages as $lang) {
            $submittedValues = array();
            $submittedValues['[[sniper_index_name]]']  =
                SniperCommon::getConfig('sniper_index_name', $this->idShop, $lang['id']);
            if (empty($submittedValues['[[sniper_index_name]]'])) {
                continue;
            }
            $submittedValues['[[sniper_key]]']  = SniperCommon::getConfig('sniper_key', $this->idShop, $lang['id']);
            if (empty($submittedValues['[[sniper_key]]'])) {
                continue;
            }
            
            $lookupValues       = array(
                'sniper_div_dsk',
                'sniper_div_mob',
                'warehouse_compatibility',
                'megashop_compatibility'
            );
            
            foreach ($lookupValues as $lVal) {
                $submittedValues['[['.$lVal.']]'] = SniperCommon::getConfig($lVal, $this->idShop, $lang['id']);
            }
            
            $submittedValues['[[sniperfast_lang_iso]]'] = $lang['iso_code'];
            $submittedValues['[[sniper_page_width]]'] = '';
            $submittedValues['[[sniperfast_dedicate_page]]'] = '';
            
            $template = str_replace(
                array_keys($submittedValues),
                $submittedValues,
                $template
            );

            $date               = new DateTime();
            $dateFormat         = $date->format("Y-m-d_H-i-s");
            $newTemplateName    = md5($dateFormat);
            $newTemplate        = $path.'/views/js/'.$this->idShop.'_'.$lang['id'].'_'.$newTemplateName.'.js';
            if (!file_put_contents($newTemplate, $template)) {
                return false;
            } else {
                $oldFile = SniperCommon::getConfig('source_data_file', $this->idShop, $lang['id']);
                $oldFilePath = $path.'/views/js/'.$this->idShop.'_'.$lang['id'].'_'.$oldFile.'.js';
                if (file_exists($oldFilePath)) {
                    unlink($oldFilePath);
                }
                SniperCommon::setConfig('source_data_file', $newTemplateName, $this->idShop, $lang['id']);
            }
        }
    }
    
    public function updateMainData110()
    {
        $path = _PS_MODULE_DIR_.'sniperfast';
        $sourceFile = $path.'/views/js/sniperfast_data_source.js';
        foreach ($this->languages as $lang) {
            $submittedValues = array(
                '[[sniper_div_dsk]]' => SniperCommon::getConfig('sniper_div_dsk', $this->idShop, $lang['id']),
                '[[sniper_div_mob]]' => SniperCommon::getConfig('sniper_div_mob', $this->idShop, $lang['id']),
                '[[sniper_index_name]]' => SniperCommon::getConfig('sniper_index_name', $this->idShop, $lang['id']),
                '[[sniper_key]]' => SniperCommon::getConfig('sniper_key', $this->idShop, $lang['id']),
                '[[sniper_page_width]]' => 992,
                '[[warehouse_compatibility]]' =>
                    SniperCommon::getConfig('warehouse_compatibility', $this->idShop, $lang['id']),
                '[[megashop_compatibility]]' =>
                    SniperCommon::getConfig('megashop_compatibility', $this->idShop, $lang['id']),
                '[[sniperfast_lang_iso]]' => $lang['iso_code'],
                '[[sniperfast_dedicate_page]]' => $this->getDedicatedPageUrl($this->idShop, $lang['id']),
                '[[sniperfast_feature_range]]' => ''
            );
            
            if (empty($submittedValues['[[sniper_div_dsk]]']) ||
                empty($submittedValues['[[sniper_index_name]]']) ||
                empty($submittedValues['[[sniper_key]]'])
            ) {
                continue;
            }
            
            $template = Tools::file_get_contents($sourceFile);
            if ($template === false) {
                $errorMsg = $this->l('Can\'t open template file');
                file_put_contents($path.'/upgrade/sniper110.txt', $errorMsg."\n", FILE_APPEND);
                return false;
            }

            $template = str_replace(
                array_keys($submittedValues),
                $submittedValues,
                $template
            );

            $date               = new DateTime();
            $dateFormat         = $date->format("Y-m-d_H-i-s");
            $newTemplateName    = md5($dateFormat).$lang['id'];
            $newTemplate        = $path.'/views/js/'.$this->idShop.'_'.$lang['id'].'_'.$newTemplateName.'.js';
            if (!file_put_contents($newTemplate, $template)) {
                $errorMsg = $this->l('Can\'t write template file');
                file_put_contents($path.'/upgrade/sniper110.txt', $errorMsg."\n", FILE_APPEND);
                return false;
            } else {
                $oldFile = SniperCommon::getConfig('source_data_file', $this->idShop, $lang['id']);
                $oldFilePath = $path.'/views/js/'.$this->idShop.'_'.$lang['id'].'_'.$oldFile.'.js';
                if (file_exists($oldFilePath)) {
                    unlink($oldFilePath);
                }
                SniperCommon::setConfig('source_data_file', $newTemplateName, $this->idShop, $lang['id']);
            }
        }
        return true;
    }
    
    protected function getWebsiteLanguages($idShop)
    {
        $languages = Language::getLanguages(true, $idShop);
        $languageForMenu = array();
        
        foreach ($languages as $lang) {
            $l = array();
            $l['id']    = $lang['id_lang'];
            $l['iso_code'] = $lang['iso_code'];
            $languageForMenu[] = $l;
        }
        return $languageForMenu;
    }
    
    protected function getDedicatedPageUrl($idShop, $idLang)
    {
        $dedicatedPageEnable = SniperCommon::getConfig('dedi_page_enable', $idShop, $idLang);
        if (!$dedicatedPageEnable) {
            return null;
        }
        
        $dedicatedPageId = SniperCommon::getConfig('dedi_page_id', $idShop, $idLang);
        if (empty($dedicatedPageId)) {
            return null;
        }

        if (!empty($dedicatedPageId) && !SniperCommon::cmsPageExists($dedicatedPageId, $idShop)) {
            return null;
        }

        $cmsPage = new CMS((int)$dedicatedPageId);
        $link = new Link();
        return $link->getCMSLink($cmsPage);
    }
}
