<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

class SniperfastProductFilter
{
    private $enabled_products = null;
    private $enabled_categories = null;
    private $enabled_manufacturers = null;
    private $enabled_suppliers = null;

    private $products_by_category_and_status = null;
    private $products_by_manufacturer_and_status = null;
    private $products_by_supplier_and_status = null;

    private $table_prefix;
    private $db;

    private $id_shop;

    public function setIdShop($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        $this->id_shop = $id;
        return $this;
    }

    private $endpoint;

    public function setEndpoint($endpoint)
    {
        if (!is_int($endpoint)) {
            throw new InvalidArgumentException('$endpoint must be INT');
        }
        $this->endpoint = $endpoint;
        return $this;
    }

    public function __construct($id_shop, $endpoint)
    {
        $this->table_prefix = _DB_PREFIX_.'sniperfast_';
        $this->db = Db::getInstance();

        $this->setIdShop($id_shop)
            ->setEndpoint($endpoint);
    }

    public function prefetch($idProducts = null)
    {
        $this->enabled_products = array();
        $this->enabled_categories = array();
        $this->enabled_manufacturers = array();
        $this->enabled_suppliers = array();

        $this->products_by_category_and_status = array();
        $this->products_by_manufacturer_and_status = array();
        $this->products_by_supplier_and_status = array();

        $psp = _DB_PREFIX_;
        $mp = $this->table_prefix;
        $providedProducts = '';
        if (!empty($idProducts)) {
            $providedProducts = ' AND p.id_product IN ('.pSQL(implode(',', $idProducts)).')';
        }
        $query = "SELECT p.id_product, ps.active, ap.enabled AS product_enabled,
            p.id_category_default, IFNULL(ac.enabled, 1) AS category_enabled,
            p.id_manufacturer, IFNULL(am.enabled, 1) AS manufacturer_enabled,
            p.id_supplier, IFNULL(asup.enabled, 1) AS supplier_enabled
            FROM {$psp}product p
            JOIN {$psp}product_shop ps ON ps.id_product = p.id_product
            LEFT JOIN {$mp}product ap ON ap.id_product = p.id_product
                AND ap.id_shop = ps.id_shop AND ap.endpoint = '$this->endpoint'
            LEFT JOIN {$mp}category ac ON ac.id_category = p.id_category_default
                AND ac.id_shop = ps.id_shop AND ac.endpoint = '$this->endpoint'
            LEFT JOIN {$mp}manufacturer am ON am.id_manufacturer = p.id_manufacturer
                AND am.id_shop = ps.id_shop AND am.endpoint = '$this->endpoint'
            LEFT JOIN {$mp}supplier asup ON asup.id_supplier = p.id_supplier
                AND asup.id_shop = ps.id_shop AND asup.endpoint = '$this->endpoint'
            WHERE ps.id_shop = $this->id_shop".
            $providedProducts.
            " ORDER BY p.id_product";

        $res = $this->db->executeS($query);

        if (!is_array($res)) {
            throw new RuntimeException('Unable to prefetch');
        }

        foreach ($res as $row) {
            $this->prefetchProcessRow($row);
        }

        return $this;
    }

    private function prefetchProcessRow($row)
    {
        $id_product = (int)$row['id_product'];
        $product_active = $row['active'] == '1' ? true : false;

        $id_category = 0;
        if (is_numeric($row['id_category_default'])) {
            $id_category = (int)$row['id_category_default'];
        }

        $id_manufacturer = 0;
        if (is_numeric($row['id_manufacturer'])) {
            $id_manufacturer = (int)$row['id_manufacturer'];
        }

        $id_supplier = 0;
        if (is_numeric($row['id_supplier'])) {
            $id_supplier = (int)$row['id_supplier'];
        }

        $category_enabled     = $row['category_enabled'] == '1' ? true : false;
        $manufacturer_enabled = $row['manufacturer_enabled'] == '1' ? true : false;
        $supplier_enabled     = $row['supplier_enabled'] == '1' ? true : false;

        if ($category_enabled) {
            $this->enabled_categories[$id_category] = true;
        }
        if ($manufacturer_enabled) {
            $this->enabled_manufacturers[$id_manufacturer] = true;
        }
        if ($supplier_enabled) {
            $this->enabled_suppliers[$id_supplier] = true;
        }

        // the product's status always has precedence over category,
        // manufacturer and supplier status
        $product_enabled = false;
        if ($product_active) {
            switch ($row['product_enabled']) {
                case '1':
                    $product_enabled = true;
                    $this->enabled_products[$id_product] = true;
                    break;
                case '0':
                    $product_enabled = false;
                    break;
                default:
                    $product_enabled = false;
                    if ($category_enabled && $manufacturer_enabled && $supplier_enabled) {
                        $product_enabled = true;
                        $this->enabled_products[$id_product] = true;
                    }
            }
        }

        if ($id_category != 0) {
            if (!isset($this->products_by_category_and_status[$id_category][(int)$product_enabled])) {
                $this->products_by_category_and_status[$id_category][(int)$product_enabled] = array();
            }
            $this->products_by_category_and_status[$id_category][(int)$product_enabled][] = $id_product;
        }

        if ($id_manufacturer != 0) {
            if (!isset($this->products_by_manufacturer_and_status[$id_manufacturer][(int)$product_enabled])) {
                $this->products_by_manufacturer_and_status[$id_manufacturer][(int)$product_enabled] = array();
            }
            $this->products_by_manufacturer_and_status[$id_manufacturer][(int)$product_enabled][] = $id_product;
        }

        if ($id_supplier != 0) {
            if (!isset($this->products_by_supplier_and_status[$id_supplier][(int)$product_enabled])) {
                $this->products_by_supplier_and_status[$id_supplier][(int)$product_enabled] = array();
            }
            $this->products_by_supplier_and_status[$id_supplier][(int)$product_enabled][] = $id_product;
        }
    }

    // section Product

    public function enableProduct($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        $this->setProductStatus($id, 1);

        if (is_array($this->enabled_products)) {
            $this->enabled_products[$id] = true;
        }
    }

    public function disableProduct($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        $this->setProductStatus($id, 0);

        if (is_array($this->enabled_products)) {
            unset($this->enabled_products[$id]);
        }
    }

    public function isProductEnabled($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        if (!is_array($this->enabled_products)) {
            $this->prefetch();
        }

        return isset($this->enabled_products[$id]);
    }

    public function getEnabledProducts()
    {
        if (!is_array($this->enabled_products)) {
            $this->prefetch();
        }

        return array_keys($this->enabled_products);
    }

    public function getEnabledProductsInterval($interval)
    {
        if (!is_array($this->enabled_products)) {
            $this->prefetchInterval($interval);
        }

        return array_keys($this->enabled_products);
    }

    private function setProductStatus($id, $status)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }
        if (!is_int($status) && !is_bool($status)) {
            throw new InvalidArgumentException('$status must be INT or BOOL');
        }

        $status = (int)$status;
        $mp = $this->table_prefix;
        $query = "INSERT INTO {$mp}product(id_product, id_shop, endpoint, enabled)
            VALUES ($id, $this->id_shop, $this->endpoint, $status)
            ON DUPLICATE KEY
            UPDATE enabled = $status";

        return $this->db->execute($query);
    }

    private function clearProductStatus($id)
    {
        $mp = $this->table_prefix;
        $query = "UPDATE {$mp}product
            SET enabled = NULL
            WHERE id_product = $id AND id_shop = $this->id_shop AND endpoint = $this->endpoint";

        return $this->db->execute($query);
    }

    private function clearProductStatusBatch(array $id_list)
    {
        if (empty($id_list)) {
            return true;
        }

        $mp = $this->table_prefix;
        $ids = pSQL(implode(',', $id_list));
        $query = "UPDATE {$mp}product
            SET enabled = NULL
            WHERE id_product IN ($ids)
            AND id_shop = $this->id_shop AND endpoint = $this->endpoint";

        return $this->db->execute($query);
    }

    private function getProductsByCategoryAndStatus($id_category, $status)
    {
        if (!is_int($id_category)) {
            throw new InvalidArgumentException('$id_category must be INT');
        }
        if (!is_int($status) && !is_bool($status)) {
            throw new InvalidArgumentException('$status must be INT or BOOL');
        }

        $status = (int)$status;

        if (!is_array($this->products_by_category_and_status)) {
            $this->prefetch();
        }

        if (isset($this->products_by_category_and_status[$id_category][$status])) {
            return $this->products_by_category_and_status[$id_category][$status];
        }

        return array();
    }

    private function getProductsByManufacturerAndStatus($id_manufacturer, $status)
    {
        if (!is_int($id_manufacturer)) {
            throw new InvalidArgumentException('$id_manufacturer must be INT');
        }
        if (!is_int($status) && !is_bool($status)) {
            throw new InvalidArgumentException('$status must be INT or BOOL');
        }

        $status = (int)$status;

        if (!is_array($this->products_by_manufacturer_and_status)) {
            $this->prefetch();
        }

        if (isset($this->products_by_manufacturer_and_status[$id_manufacturer][$status])) {
            return $this->products_by_manufacturer_and_status[$id_manufacturer][$status];
        }

        return array();
    }

    private function getProductsBySupplierAndStatus($id_supplier, $status)
    {
        if (!is_int($id_supplier)) {
            throw new InvalidArgumentException('$id_supplier must be INT');
        }
        if (!is_int($status) && !is_bool($status)) {
            throw new InvalidArgumentException('$status must be INT or BOOL');
        }

        $status = (int)$status;

        if (!is_array($this->products_by_supplier_and_status)) {
            $this->prefetch();
        }

        if (isset($this->products_by_supplier_and_status[$id_supplier][$status])) {
            return $this->products_by_supplier_and_status[$id_supplier][$status];
        }

        return array();
    }

    // end section Product

    // section Category

    public function enableCategory($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        $this->setCategoryStatus($id, 1);

        if (is_array($this->enabled_categories)) {
            $this->enabled_categories[$id] = true;
        }
    }

    public function disableCategory($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        $this->setCategoryStatus($id, 0);

        if (is_array($this->enabled_categories)) {
            unset($this->enabled_categories[$id]);
        }
    }

    public function isCategoryEnabled($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        if (!is_array($this->enabled_categories)) {
            $this->prefetch();
        }

        return isset($this->enabled_categories[$id]);
    }

    private function setCategoryStatus($id, $status)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }
        if (!is_int($status) && !is_bool($status)) {
            throw new InvalidArgumentException('$status must be INT or BOOL');
        }

        $this->clearProductStatusBatch($this->getProductsByCategoryAndStatus($id, $status));

        $status = (int)$status;
        $mp = $this->table_prefix;
        $query = "INSERT INTO {$mp}category(id_category, id_shop, endpoint, enabled)
            VALUES ($id, $this->id_shop, $this->endpoint, $status)
            ON DUPLICATE KEY
            UPDATE enabled = $status";

        return $this->db->execute($query);
    }

    // end section Category

    // section Manufacturer

    public function enableManufacturer($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        $this->setManufacturerStatus($id, 1);

        if (is_array($this->enabled_manufacturers)) {
            $this->enabled_manufacturers[$id] = true;
        }
    }

    public function disableManufacturer($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        $this->setManufacturerStatus($id, 0);

        if (is_array($this->enabled_manufacturers)) {
            unset($this->enabled_manufacturers[$id]);
        }
    }

    public function isManufacturerEnabled($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        if (!is_array($this->enabled_manufacturers)) {
            $this->prefetch();
        }

        return isset($this->enabled_manufacturers[$id]);
    }

    private function setManufacturerStatus($id, $status)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }
        if (!is_int($status) && !is_bool($status)) {
            throw new InvalidArgumentException('$status must be INT or BOOL');
        }

        $this->clearProductStatusBatch($this->getProductsByManufacturerAndStatus($id, $status));

        $status = (int)$status;
        $mp = $this->table_prefix;
        $query = "INSERT INTO {$mp}manufacturer(id_manufacturer, id_shop, endpoint, enabled)
            VALUES ($id, $this->id_shop, $this->endpoint, $status)
            ON DUPLICATE KEY
            UPDATE enabled = $status";

        return $this->db->execute($query);
    }

    // end section Manufacturer

    // section Supplier

    public function enableSupplier($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        $this->setSupplierStatus($id, 1);

        if (is_array($this->enabled_suppliers)) {
            $this->enabled_suppliers[$id] = true;
        }
    }

    public function disableSupplier($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        $this->setSupplierStatus($id, 0);

        if (is_array($this->enabled_suppliers)) {
            unset($this->enabled_suppliers[$id]);
        }
    }

    public function isSupplierEnabled($id)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }

        if (!is_array($this->enabled_suppliers)) {
            $this->prefetch();
        }

        return isset($this->enabled_suppliers[$id]);
    }

    private function setSupplierStatus($id, $status)
    {
        if (!is_int($id)) {
            throw new InvalidArgumentException('$id must be INT');
        }
        if (!is_int($status) && !is_bool($status)) {
            throw new InvalidArgumentException('$status must be INT or BOOL');
        }

        $this->clearProductStatusBatch($this->getProductsBySupplierAndStatus($id, $status));

        $status = (int)$status;
        $mp = $this->table_prefix;
        $query = "INSERT INTO {$mp}supplier(id_supplier, id_shop, endpoint, enabled)
            VALUES ($id, $this->id_shop, $this->endpoint, $status)
            ON DUPLICATE KEY
            UPDATE enabled = $status";

        return $this->db->execute($query);
    }

    // end section Supplier
}
