<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

class AdminSniperfastController extends ModuleAdminController
{
    protected $idLang;
    protected $product_filter;

    public function __construct()
    {
        require_once dirname(__FILE__).'/../../autoload.php';

        $this->allow_export       = false;
        $this->bootstrap          = true;
        $this->path               = _PS_MODULE_DIR_.'sniperfast';
        $this->list_no_link       = true;
        $this->somethingSubmitted = false;
        $this->className          = 'SniperFast';

        parent::__construct();

        $idShop                   = $this->context->shop->id;
        // $this->idLang             = Configuration::get('PS_LANG_DEFAULT', null, null, $idShop);

        $langs = $this->getWebsiteLanguages($idShop, true);
        $cookie = new Cookie('sniperfast');
        if (!isset($cookie->selected_lang)) {
            $firstLang = array_shift($langs);
            $this->idLang = $firstLang['id'];
            $cookie->selected_lang = $this->idLang;
        } else {
            $this->idLang = $cookie->selected_lang;
        }

        $this->product_filter =
            new SniperfastProductFilter((int)$this->context->shop->id, (int)$this->idLang);
    }

    public function setMedia($isNewTheme = false)
    {
        if (version_compare(phpversion(), '7.2', '<')) {
            parent::setMedia();
        } else {
            parent::setMedia($isNewTheme);
        }
        $this->addCSS($this->path.'/views/css/prestalia_header.css');
        $this->addCSS($this->path.'/views/css/back_142.css');
        $this->addJQuery();
        $this->context->controller->addJqueryPlugin('colorpicker');
        $this->addJS($this->path.'/views/js/back_142.js');
    }

    public function postProcess()
    {
        $this->processFormGeneral();
        $this->processFormSynonims();
        $this->processFormFeatures();
        $this->processFormAttributes();
        $this->processFormCss();
        $this->processListProducts();
        $this->processListCategoryMapping();
        $this->processListManufacturers();
        $this->processListSuppliers();
        $this->processFormAppImg();
        $this->processFormAppSocial();
        $this->processFormAppBanner();
        $this->processFormAppSlider();
        $this->processFormAppNotification();
        $this->processFormAppNotificationOrder();
        $this->processFormAppCms();
        $this->processFormAppContacts();
        $this->processFormAppLanguage();
        $this->processPosition();
    }

    protected function processFormGeneral()
    {
        if (!Tools::isSubmit('submit_form_general') && !Tools::isSubmit('submitAddform_general')) {
            return false;
        }
        $idShop = $this->context->shop->id;
        $lookupValues       = array(
            'sniper_index_name',
            'sniper_key',
            'sniper_div_dsk',
            'sniper_div_mob',
            'only_in_stock',
            'sniper_currency',
            'image_format',
            'sniper_free_ship',
            'sniper_export_review',
            'sync_on_save'
        );
        $requiredValues     = array('sniper_index_name', 'sniper_key', 'sniper_div_dsk');
        $requiredValueMissing = false;

        foreach ($lookupValues as $lookupVal) {
            $val = Tools::getValue($lookupVal);
            $val = trim($val);

            if (empty($val) && in_array($lookupVal, $requiredValues)) {
                if ($lookupVal == 'sniper_index_name' &&
                    SniperCommon::getConfig('sniper_index_name', $idShop, $this->idLang) != ''
                ) {
                    continue;
                }
                if ($lookupVal == 'sniper_key' &&
                    SniperCommon::getConfig('sniper_key', $idShop, $this->idLang) != ''
                ) {
                    continue;
                }
                $requiredValueMissing = true;
                continue;
            }
            SniperCommon::setConfig($lookupVal, $val, $idShop, $this->idLang);
        }

        if (Tools::version_compare(_PS_VERSION_, '1.7', '<')) {
            $shop = new Shop($idShop, null, $idShop);
            $currentTheme = $shop->getTheme();
        } else {
            $shop = new Shop($idShop, $this->idLang, $idShop);
            $currentTheme = $shop->theme->getDirectory();
        }

        $currentTheme = explode('/', $currentTheme);
        array_pop($currentTheme);
        $currentTheme = array_pop($currentTheme);

        if ($currentTheme == 'warehouse') {
            SniperCommon::setConfig('warehouse_compatibility', 1, $idShop, $this->idLang);
        } else {
            SniperCommon::setConfig('warehouse_compatibility', 0, $idShop, $this->idLang);
        }

        if (Tools::strpos($currentTheme, 'MegaShop') !== false) {
            SniperCommon::setConfig('megashop_compatibility', 1, $idShop, $this->idLang);
        } else {
            SniperCommon::setConfig('megashop_compatibility', 0, $idShop, $this->idLang);
        }

        // Dedicated page
        $dedicatedPageEnable = Tools::getValue('dedi_page_enable');
        $dedicatedPageTitle = Tools::getValue('dedi_page_title');
        $dedicatedPageTitle = trim($dedicatedPageTitle);
        if ($dedicatedPageEnable && empty($dedicatedPageTitle)) {
            $this->errors[] = $this->l('Page title can\'t be empty');
            return;
        }
        $dedicatedPageRewrite = Tools::getValue('dedi_page_rewrite');
        $dedicatedPageRewrite = trim($dedicatedPageRewrite);
        if ($dedicatedPageEnable && empty($dedicatedPageRewrite)) {
            $this->errors[] = $this->l('Link rewrite can\'t be empty');
            return;
        }
        $dedicatedPageId = SniperCommon::getConfig('dedi_page_id', $idShop, $this->idLang);
        if (empty($dedicatedPageId)) {
            $sql = "SELECT conf_value
                FROM "._DB_PREFIX_."sniperfast_conf
                WHERE conf_key = 'dedi_page_id'
                AND id_shop = ".(int)$idShop;
            $dedicatedIds = Db::getInstance()->executeS($sql);
            if (!empty($dedicatedIds)) {
                foreach ($dedicatedIds as $dediId) {
                    $dedicatedPageId = $dediId['conf_value'];
                    break;
                }
            }
        }

        if (!empty($dedicatedPageId) && !SniperCommon::cmsPageExists($dedicatedPageId, $idShop)) {
            $dedicatedPageId = null;
        }

        if ($dedicatedPageEnable) {
            $cmsPage = null;
            try {
                if (empty($dedicatedPageId)) {
                    $cmsPage = new CMS();
                    $cmsPage->meta_title[$this->idLang] = $dedicatedPageTitle;
                    $cmsPage->link_rewrite[$this->idLang] = $dedicatedPageRewrite;
                    $sql = "SELECT c.`id_cms_category`
                        FROM `"._DB_PREFIX_."cms_category` c
                        LEFT JOIN `"._DB_PREFIX_."cms_category_lang` cl ON c.`id_cms_category` = cl.`id_cms_category`
                        WHERE `id_lang` = ".(int)$this->idLang."
                        AND c.`id_parent` = 0
                        AND `active` = 1
                        ORDER BY `name` ASC";
                    $homeCategoryId = Db::getInstance()->getValue($sql);
                    $cmsPage->id_cms_category = $homeCategoryId;
                    $cmsPage->content[$this->idLang] = '<div id="sniperfast_search_container"></div>';
                    $cmsPage->indexation = 0;
                    $cmsPage->active = 1;
                    $cmsPage->add();
                } else {
                    $cmsPage = new CMS((int)$dedicatedPageId);
                    $cmsPage->meta_title[$this->idLang] = $dedicatedPageTitle;
                    $cmsPage->link_rewrite[$this->idLang] = $dedicatedPageRewrite;
                    $cmsPage->content[$this->idLang] = '<div id="sniperfast_search_container"></div>';
                    $cmsPage->active = $dedicatedPageEnable;
                    $cmsPage->update();
                }
            } catch (Exception $e) {
                $this->errors[] = $this->l('Could not add the page');
                return;
            }

            SniperCommon::setConfig('dedi_page_id', $cmsPage->id, $idShop, $this->idLang);
            SniperCommon::setConfig('dedi_page_enable', $dedicatedPageEnable, $idShop, $this->idLang);
            SniperCommon::setConfig('dedi_page_title', $dedicatedPageTitle, $idShop, $this->idLang);
            SniperCommon::setConfig('dedi_page_rewrite', $dedicatedPageRewrite, $idShop, $this->idLang);
        } else {
            SniperCommon::setConfig('dedi_page_enable', $dedicatedPageEnable, $idShop, $this->idLang);
        }

        // Aspect data
        $hideCat = Tools::getValue('hide_cat');
        $hideManu = Tools::getValue('hide_manu');
        SniperCommon::setConfig('hide_cat', $hideCat, $idShop, $this->idLang);
        SniperCommon::setConfig('hide_manu', $hideManu, $idShop, $this->idLang);

        // File update
        $errorMsg = '';
        $updateSuccess =
            SniperCommon::updateMainData($idShop, $this->idLang, $errorMsg);
        if (!$updateSuccess) {
            $requiredValueMissing = true;
            $this->errors[] = $errorMsg;
        }

        if ($requiredValueMissing) {
            $this->errors[] = $this->l('One ore more required values are missing');
        } else {
            $this->somethingSubmitted = true;
        }
    }

    protected function processFormFeatures()
    {
        $idShop = $this->context->shop->id;

        if (!Tools::isSubmit('submit_form_features') && !Tools::isSubmit('submitAddform_features')) {
            return false;
        }

        $submittedFeatures = Tools::getValue('sniper_features');
        $commaSeparatedFeatures = '';

        if ($submittedFeatures) {
            foreach ($submittedFeatures as $tmp) {
                $commaSeparatedFeatures.=$tmp.',';
            }
            $commaSeparatedFeatures = rtrim($commaSeparatedFeatures, ',');
        };

        SniperCommon::setConfig('sniper_features', $commaSeparatedFeatures, $idShop, $this->idLang);
        $this->somethingSubmitted = true;
    }

    protected function processFormAttributes()
    {
        $idShop = $this->context->shop->id;

        if (!Tools::isSubmit('submit_form_attributes') && !Tools::isSubmit('submitAddform_attributes')) {
            return false;
        }

        $submittedAttributes = Tools::getValue('sniper_attributes');
        $commaSeparatedAttributes = '';

        if ($submittedAttributes) {
            foreach ($submittedAttributes as $tmp) {
                $commaSeparatedAttributes.=$tmp.',';
            }
            $commaSeparatedAttributes = rtrim($commaSeparatedAttributes, ',');
        };

        SniperCommon::setConfig('sniper_attributes', $commaSeparatedAttributes, $idShop, $this->idLang);
        $this->somethingSubmitted = true;
    }

    protected function processFormSynonims()
    {
        if (!Tools::isSubmit('submit_form_synonims') && !Tools::isSubmit('submitAddform_synonims')) {
            return false;
        }

        $idShop = $this->context->shop->id;

        // Check synonims validity
        $val = '';
        $tmpPath = $this->path.'/tmp/'.$idShop.'_'.$this->idLang.'_synonims_tmp.txt';

        if ($_FILES["sniper_synonims_file"]["name"] != '') {
            $finfo = finfo_open(FILEINFO_MIME_TYPE);
            $mtype = finfo_file($finfo, $_FILES["sniper_synonims_file"]["tmp_name"]);
            finfo_close($finfo);

            if ($mtype != 'text/plain') {
                $this->errors[] = $this->l('Invalid file');
                return false;
            } elseif ($_FILES['sniper_synonims_file']['error'] == UPLOAD_ERR_OK) {
                if (!move_uploaded_file(
                    $_FILES['sniper_synonims_file']['tmp_name'],
                    $tmpPath
                )) {
                    $this->errors[] = $this->l('Cannot upload file');
                    return false;
                } else {
                    $val = Tools::file_get_contents($tmpPath);
                }
            }
        } else {
            $val = Tools::getValue('sniper_synonims_text');
            $val = trim($val);
        }

        if (empty($val)) {
            if (file_exists($this->path.'/output/'.$idShop.'_'.$this->idLang.'_synonims.txt')) {
                unlink($this->path.'/output/'.$idShop.'_'.$this->idLang.'_synonims.txt');
            }
            if (file_exists($this->path.'/log/'.$idShop.'_'.$this->idLang.'_synonims_error.txt')) {
                unlink($this->path.'/log/'.$idShop.'_'.$this->idLang.'_synonims_error.txt');
            }
            $this->somethingSubmitted = true;
            return;
        }

        $synonimsCheck = $this->getValidSynonims($val, $idShop, $this->idLang);
        switch ($synonimsCheck) {
            case 1:
                $this->somethingSubmitted = true;
                break;
            case 0:
                $this->warnings[] = $this->l('Some invalid synonims found, check report');
                break;
            case -1:
                $this->errors[] = $this->l('No valid synonims found');
                break;
        }
        if (file_exists($tmpPath)) {
            unlink($tmpPath);
        }
    }

    protected function processFormCss()
    {
        if (!Tools::isSubmit('submit_form_css') && !Tools::isSubmit('submitAddform_css')) {
            return false;
        }

        $idShop = $this->context->shop->id;

        // Check CSS validity
        $val = Tools::getValue('sniper_css_text');
        $val = trim($val);
        $cssHeight = Tools::getValue('sniper_result_height');
        $cssWidth  = Tools::getValue('sniper_result_width');
        $cssAlign  = Tools::getValue('sniper_result_align');
        $cssHeight = preg_replace('/\D/', '', $cssHeight);
        SniperCommon::setConfig('sniper_result_height', $cssHeight, $idShop, $this->idLang);
        SniperCommon::setConfig('sniper_result_width', $cssWidth, $idShop, $this->idLang);
        SniperCommon::setConfig('sniper_result_align', $cssAlign, $idShop, $this->idLang);
        $errorMsg = '';
        SniperCommon::updateMainData($idShop, $this->idLang, $errorMsg);

        if (file_exists($this->path.'/views/css/'.$idShop.'_'.$this->idLang.'_custom.css')) {
            unlink($this->path.'/views/css/'.$idShop.'_'.$this->idLang.'_custom.css');
        }
        if (file_exists($this->path.'/views/css/'.$idShop.'_'.$this->idLang.'_sniperfast.css')) {
            unlink($this->path.'/views/css/'.$idShop.'_'.$this->idLang.'_sniperfast.css');
        }

        if (empty($val) && empty($cssHeight) && $cssWidth == 100) {
            $this->somethingSubmitted = true;
            return;
        }

        $cssHeightFile = $this->path.'/views/css/sniperfast.css';
        if (file_exists($cssHeightFile)) {
            $cssHeightTemp = Tools::file_get_contents($cssHeightFile);
            if (empty($cssHeight)) {
                $cssHeight = 180;
            }
            $cssHeightTemp = str_replace('[[sniper_result_height]]', $cssHeight, $cssHeightTemp);
            $cssHeightTemp = str_replace('[[sniper_result_width]]', $cssWidth, $cssHeightTemp);
            file_put_contents($this->path.'/views/css/'.$idShop.'_'.$this->idLang.'_sniperfast.css', $cssHeightTemp);
        }

        $this->getValidCss($val, $idShop, $this->idLang);
        $this->somethingSubmitted = true;
    }

    protected function processFormConfiguration()
    {
        if (!Tools::isSubmit('submit_form_configuration') &&
            !Tools::isSubmit('submitAddform_configuration')
        ) {
            return false;
        }

        $idShop = $this->context->shop->id;
        $submittedValues = array('sniper_name_weight');

        foreach ($submittedValues as $submittedVal) {
            $val = Tools::getValue($submittedVal);
            SniperCommon::setConfig($submittedVal, $val, $idShop, $this->idLang);
        }

        $this->somethingSubmitted = true;
    }

    protected function processFormUtilities()
    {
        if (!Tools::isSubmit('submit_form_utilities') &&
            !Tools::isSubmit('submitAddform_utilities')
        ) {
            return false;
        }

        $idShop = $this->context->shop->id;
        $submittedValues = array('sniper_bar_lookup');

        foreach ($submittedValues as $submittedVal) {
            $val = Tools::getValue($submittedVal);
            SniperCommon::setConfig($submittedVal, $val, $idShop, $this->idLang);
        }

        $this->somethingSubmitted = true;
    }

    public function initContent()
    {
        if (!$this->viewAccess()) {
            $this->errors[] = $this->l('You do not have permission to view this.');
            return;
        }

        $this->getLanguages();
        $this->initToolbar();
        $this->initTabModuleList();

        // This function does not exist on PS 1.5
        if (Tools::version_compare(_PS_VERSION_, '1.6', '>=')) {
            $this->initPageHeaderToolbar();
        }

        /* Set ajax url  */
        $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }

        $idShop = (int)$this->context->shop->id;
        SniperCommon::checkConfig($idShop, $this->idLang);
        $ajaxToken = SniperCommon::getConfig('ajax_token', $idShop, $this->idLang);

        $psVersion = 6;
        if (Tools::version_compare(_PS_VERSION_, '1.7', '>=')) {
            $psVersion = 7;
        }

        $langs = $this->getWebsiteLanguages($idShop);

        $licenseExists = 1;
        $sniperKey = SniperCommon::getConfig('sniper_key', $idShop, $this->idLang);
        $sniperIndex = SniperCommon::getConfig('sniper_index_name', $idShop, $this->idLang);
        if (empty($sniperKey) || empty($sniperIndex)) {
            $licenseExists = 0;
        }

        $this->context->smarty->assign(array(
            'module_version' => $this->module->version,
            'base_url' => $baseUrl,
            'sniper_token' => $ajaxToken,
            'sniper_shop_id'    => $idShop,
            'sniper_shop_lang'  => $this->idLang,
            'sniper_ps_version' => $psVersion,
            'sniper_search_url' => CDN_PROTOCOL.'://index.sniperfast.'.CDN_ENDPOINT.'/schedule',
            'sniper_stats_url' => CDN_PROTOCOL.'://api.sniperfast.'.CDN_ENDPOINT.'/stats',
            'sniper_save_url' => "{$baseUrl}modules/sniperfast/ajax/saveData.php",
            'sniper_save_app_url' => "{$baseUrl}modules/sniperfast/ajax/saveAppData.php",
            'sniper_key'        => $sniperKey,
            'sniper_index_name' => $sniperIndex,
            'last_job_id'       => SniperCommon::getConfig('last_job_id', $idShop, $this->idLang),
            'sniper_source_ip'  => $_SERVER['SERVER_ADDR'],
            'sniper_js_search'  => Configuration::get('PS_SEARCH_AJAX', null, null, $idShop),
            'sniper_ajax_instant'  => Configuration::get('PS_INSTANT_SEARCH', null, null, $idShop),
            'sniper_js_end'  => Configuration::get('PS_JS_DEFER', null, null, $idShop),
            'langs' => $langs,
            'url_change_language' => "{$baseUrl}modules/sniperfast/ajax/setLanguage.php",
            'sniper_bar_lookup' => SniperCommon::getConfig('sniper_bar_lookup', $idShop, $this->idLang),
            'sniper_flags' => "{$baseUrl}modules/sniperfast/views/img/flags/4x3/",
            'licenseExists' => $licenseExists
        ));
        $this->content .= $this->context->smarty->fetch($this->path.'/views/templates/admin/prestalia_header.tpl');

        if (empty($this->errors) && $this->somethingSubmitted) {
            $this->confirmations[] = $this->l('Configuration saved');
        }

        $this->context->smarty->assign(array(
            'url_module' => 'index.php?controller=AdminSniperfast',
            'bo_token' => Tools::getAdminTokenLite('AdminSniperfast'),
        ));

        $this->setActiveTab();
        $this->renderViewDefault();

        $this->context->smarty->assign(array(
            'maintenance_mode' => !(bool)Configuration::get('PS_SHOP_ENABLE'),
            'content'          => $this->content,
            'lite_display'     => $this->lite_display,
            'url_post'         => self::$currentIndex.'&token='.$this->token,
        ));

        if (Tools::version_compare(_PS_VERSION_, '1.6', '>=')) {
            $this->context->smarty->assign(array(
                'show_page_header_toolbar'  => $this->show_page_header_toolbar,
                'page_header_toolbar_title' => $this->page_header_toolbar_title,
                'title'                     => $this->page_header_toolbar_title,
                'toolbar_btn'               => $this->page_header_toolbar_btn,
                'page_header_toolbar_btn'   => $this->page_header_toolbar_btn,
            ));
        }
    }

    protected function renderViewDefault()
    {
        $error = '<p>'.$this->l('You need to correctly configure the module in order to access this tab').'</p>';
        $isValidShopAndGroup = true;
        if (in_array($this->context->shop->getContext(), array(Shop::CONTEXT_GROUP, Shop::CONTEXT_ALL))) {
            $error = $this->l('In order to use the module please select a shop from Prestashop dropdown menu.');
            $isValidShopAndGroup = false;
        }

        // Base template
        $template = $this->context->smarty->fetch($this->path.'/views/templates/admin/controller.tpl');
        if (!$isValidShopAndGroup) {
            $template = $error;
            $this->content .= $template;
            return;
        }

        if (Tools::isSubmit('deletesniperfast_app_banner')) {
            if (SniperCommonApp::deleteBanner((int)Tools::getValue('id_banner'))) {
                $this->somethingSubmitted = true;
            } else {
                $this->errors[] = $this->l('Could not delete banner');
            }
        }
        if (Tools::isSubmit('enabledsniperfast_app_banner')) {
            if (SniperCommonApp::changeActivationStatus((int)Tools::getValue('id_banner'), 'banner')) {
                $this->somethingSubmitted = true;
            } else {
                $this->errors[] = $this->l('Could not change banner status');
            }
        }
        if (Tools::isSubmit('deletesniperfast_app_slider')) {
            if (SniperCommonApp::deleteSlider((int)Tools::getValue('id_slider'))) {
                $this->somethingSubmitted = true;
            } else {
                $this->errors[] = $this->l('Could not delete slider');
            }
        }
        if (Tools::isSubmit('enabledsniperfast_app_slider')) {
            if (SniperCommonApp::changeActivationStatus((int)Tools::getValue('id_slider'), 'slider')) {
                $this->somethingSubmitted = true;
            } else {
                $this->errors[] = $this->l('Could not change slider status');
            }
        }
        if (Tools::isSubmit('deletesniperfast_app_notification_order')) {
            if (SniperCommonApp::deleteNotificationOrder((int)Tools::getValue('id_notification_order'))) {
                $this->somethingSubmitted = true;
            } else {
                $this->errors[] = $this->l('Could not delete notification');
            }
        }
        if (Tools::isSubmit('deletesniperfast_app_notification')) {
            if (SniperCommonApp::deleteNotification((int)Tools::getValue('id_notification'))) {
                $this->somethingSubmitted = true;
            } else {
                $this->errors[] = $this->l('Could not delete notification');
            }
        }
        if (Tools::isSubmit('enabledsniperfast_app_notification')) {
            if (SniperCommonApp::changeActivationStatus(
                (int)Tools::getValue('id_notification'),
                'notification_order'
            )) {
                $this->somethingSubmitted = true;
            } else {
                $this->errors[] = $this->l('Could not change notification status');
            }
        }
        if (Tools::isSubmit('enabledsniperfast_app_notification_order')) {
            if (SniperCommonApp::changeActivationStatus(
                (int)Tools::getValue('id_notification_order'),
                'notification_order'
            )) {
                $this->somethingSubmitted = true;
            } else {
                $this->errors[] = $this->l('Could not change notification status');
            }
        }

        if (Tools::isSubmit('updatesniperfast_app_banner') || Tools::isSubmit('add_banner')) {
            $template = $this->renderBannerForm();
        } elseif (Tools::isSubmit('updatesniperfast_app_slider') || Tools::isSubmit('add_slider')) {
            $template = $this->renderSliderForm();
        } elseif (Tools::isSubmit('updatesniperfast_app_notification') || Tools::isSubmit('add_notification')) {
            $template = $this->renderNotificationForm();
        } elseif (Tools::isSubmit('updatesniperfast_app_notification_order') ||
            Tools::isSubmit('add_notification_order')
        ) {
            $template = $this->renderNotificationOrderForm();
        } else {
            $generalOut        = $this->renderFormGeneralInfo();
            $generalOut        .= $this->renderFormGeneral();
            $featuresOut       = $this->renderFormFeatures();
            $attributesOut       = $this->renderFormAttributes();
            $cssOut            = $this->renderFormCss();
            // $productsOut       = $this->renderFormProducts();
            $productsOut       = $this->renderListProducts();
            // $categoriesOut     = $this->renderFormCategories();
            $categoriesOut     = $this->renderListCategories();
            // $manufacturersOut  = $this->renderFormManufacturers();
            $manufacturersOut  = $this->renderListManufacturers();
            // $suppliersOut      = $this->renderFormSuppliers();
            $suppliersOut      = $this->renderListSuppliers();
            $statsOut          = $this->renderFormStats();
            $exportOut         = $this->renderFormExport();

            $appOut            = $this->renderFormAppWelcome();
            $appOut            .= $this->renderFormAppLogo();
            $appOut            .= $this->renderFormApp();
            $appOut            .= $this->renderAddBanner();
            $appOut            .= $this->renderListBanner();
            $appOut            .= $this->renderAddSlider();
            $appOut            .= $this->renderListSlider();
            $appOut            .= $this->renderAddNotification();
            $appOut            .= $this->renderListNotification();
            $appOut            .= $this->renderAddNotificationOrder();
            $appOut            .= $this->renderListNotificationOrder();
            $appOut            .= $this->renderFormAppSocial();
            $appOut            .= $this->renderFormAppCms();
            $appOut            .= $this->renderFormAppContacts();
            $appOut            .= $this->renderFormAppLanguage();
            $synonimsOut       = $this->renderFormSynonims();

            // Generate output
            $template = str_replace('[[general_content]]', $generalOut, $template);
            $template = str_replace('[[features_content]]', $featuresOut, $template);
            $template = str_replace('[[attributes_content]]', $attributesOut, $template);
            $template = str_replace('[[synonims_content]]', $synonimsOut, $template);
            $template = str_replace('[[cssrules_content]]', $cssOut, $template);
            $template = str_replace('[[products_content]]', $productsOut, $template);
            $template = str_replace('[[categories_content]]', $categoriesOut, $template);
            $template = str_replace('[[manufacturers_content]]', $manufacturersOut, $template);
            $template = str_replace('[[suppliers_content]]', $suppliersOut, $template);
            $template = str_replace('[[stats_content]]', $statsOut, $template);
            $template = str_replace('[[export_content]]', $exportOut, $template);
            $template = str_replace('[[app_content]]', $appOut, $template);
        }

            $generalOut        = $this->renderFormGeneralInfo();
            $generalOut        .= $this->renderFormGeneral();
            $featuresOut       = $this->renderFormFeatures();
            $attributesOut       = $this->renderFormAttributes();
            $cssOut            = $this->renderFormCss();
            // $productsOut       = $this->renderFormProducts();
            $productsOut       = $this->renderListProducts();
            // $categoriesOut     = $this->renderFormCategories();
            $categoriesOut     = $this->renderListCategories();
            // $manufacturersOut  = $this->renderFormManufacturers();
            $manufacturersOut  = $this->renderListManufacturers();
            // $suppliersOut      = $this->renderFormSuppliers();
            $suppliersOut      = $this->renderListSuppliers();
            $statsOut          = $this->renderFormStats();
            $exportOut         = $this->renderFormExport();
            // $utilitiesOut      = $this->renderFormUtilities();
            // $utilitiesOut      .= $this->renderFormUtilitiesFrame();



        // } else {
            // $configurationOut = $error;
        // }

        // Set page title
        $this->toolbar_title = $this->l('Sniperfast');

        $this->content .= $template;
        $this->content .=
        $this->context->smarty->fetch(dirname(__FILE__).'/../../views/templates/admin/prestalia_footer.tpl');
    }

    /*
     * General form
     */
    protected function renderFormGeneralInfo()
    {
        return $this->context->smarty->fetch($this->path.'/views/templates/admin/job_status.tpl');
    }

    protected function renderFormGeneral()
    {
        $this->submit_action = 'submit_form_general';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormGeneralFields();
        $idShop = $this->context->shop->id;
        $lookupValues = array(
            'sniper_default',
            'sniper_server',
            'sniper_key',
            'sniper_index_name',
            'sniper_div_dsk',
            'sniper_div_mob',
            'only_in_stock',
            'sniper_currency',
            'image_format',
            'sniper_free_ship',
            'sniper_export_review',
            'sync_on_save',
            'hide_cat',
            'hide_manu'
        );

        foreach ($lookupValues as $lookupVal) {
            $this->fields_value[$lookupVal] =
                SniperCommon::getConfig($lookupVal, $idShop, $this->idLang);
        }

        $defaultDedicatedPage = $this->getDefaultDedicatedPageValues($idShop);
        $dediPageEnable = SniperCommon::getConfig('dedi_page_enable', $idShop, $this->idLang);
        $dediPageTitle = SniperCommon::getConfig('dedi_page_title', $idShop, $this->idLang);
        if (empty($dediPageTitle)) {
            $dediPageTitle = $defaultDedicatedPage['dedi_page_title'];
        }
        $dediPageRewrite = SniperCommon::getConfig('dedi_page_rewrite', $idShop, $this->idLang);
        if (empty($dediPageRewrite)) {
            $dediPageRewrite = $defaultDedicatedPage['dedi_page_rewrite'];
        }

        $this->fields_value['dedi_page_enable'] = $dediPageEnable;
        $this->fields_value['dedi_page_title'] = $dediPageTitle;
        $this->fields_value['dedi_page_rewrite'] = $dediPageRewrite;

        return parent::renderForm();
    }

    protected function getFormGeneralFields()
    {
        $isDebug = Tools::getIsset('prestalia_debug');
        $isDebug = !$isDebug;

        $hint1 = $this->l('The url you will see in your browser. Leave the default data if uncertain.');
        $review1 = $this->l('Please contact us and report the review system in use.');
        $review2 = $this->l('We will provide a dedicated solution to export the required data.');
        $syncSave1 = $this->l('If set to Yes, this option will enable product synchronization at save.');
        $syncSave2 = $this->l('Modified data will be immediately sent to the search results.');
        $img1 = $this->l('Set the image format that will be exported.');
        $img2 = $this->l('It is advisable to set a picture at maximum 250 px large.');
        $aspectEnabled = $this->l('If enabled,');
        $aspectManu = $this->l('this option will hide the manufacturer filter within the search results panel.');
        $aspectCat = $this->l('this option will hide the category filter within the search results panel.');

        $fields_form = array(
            'legend' => array(
                'title' => $this->l('General configuration'),
                'icon'  => 'icon-cogs',
            ),
            'tabs' => array(
                'general_data'       => $this->l('General configuration'),
                'dedicated_data'     => $this->l('Dedicated page'),
                'aspect_data'     => $this->l('Aspect'),
            ),
            'input' => array(
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Key'),
                    'name'      => 'sniper_key',
                    'class'     => 'input fixed-width-xl',
                    'disabled'  => $isDebug,
                    'required'  => true,
                    'tab'       => 'general_data'
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Index name'),
                    'name'      => 'sniper_index_name',
                    'class'     => 'input fixed-width-xl',
                    'disabled'  => $isDebug,
                    'required'  => true,
                    'tab'       => 'general_data'
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Desktop search bar'),
                    'name'      => 'sniper_div_dsk',
                    'class'     => 'input fixed-width-xl',
                    'required'  => true,
                    'tab'       => 'general_data'
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Mobile search bar'),
                    'name'      => 'sniper_div_mob',
                    'class'     => 'input fixed-width-xl',
                    'tab'       => 'general_data'
                ),
                array(
                    'type'      => 'switch',
                    'is_bool'   => true,
                    'label'     => $this->l('Export only products with stock'),
                    'desc'      => $this->l(''),
                    'name'      => 'only_in_stock',
                    'tab'       => 'general_data',
                    'values' => array(
                        array(
                            'id' => 'only_in_stock_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'only_in_stock_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    )
                ),
                array(
                    'type'    => 'select',
                    'label'   => $this->l('Currency'),
                    'name'    => 'sniper_currency',
                    'options' => array(
                        'query' => $this->getShopCurrencies(),
                        'id'    => 'id',
                        'name'  => 'name',
                    ),
                    'hint'      => $this->l('Prices will be sent in the selected currency.'),
                    'tab'       => 'general_data',
                ),
                array(
                    'type'    => 'select',
                    'label'   => $this->l('Image format'),
                    'name'    => 'image_format',
                    'hint'    => $img1.' '.$img2,
                    'tab'       => 'general_data',
                    'options' => array(
                        'query' => $this->getImageTypes(),
                        'id'    => 'id',
                        'name'  => 'name'
                    )
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Free shipping from'),
                    'name'      => 'sniper_free_ship',
                    'class'     => 'input fixed-width-xl',
                    'tab'       => 'general_data'
                ),
                array(
                    'type'      => 'switch',
                    'is_bool'   => true,
                    'label'     => $this->l('Export reviews'),
                    'desc'      => $review1.' '.$review2,
                    'name'      => 'sniper_export_review',
                    'tab'       => 'general_data',
                    'values' => array(
                        array(
                            'id' => 'sniper_export_review_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'sniper_export_review_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    )
                ),
                array(
                    'type'      => 'switch',
                    'is_bool'   => true,
                    'label'     => $this->l('Synchronize products on save'),
                    'desc'      => $syncSave1.' '.$syncSave2,
                    'name'      => 'sync_on_save',
                    'tab'       => 'general_data',
                    'values' => array(
                        array(
                            'id' => 'sync_on_save_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'sync_on_save_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    )
                ),
                array(
                    'type' => 'html',
                    'name' => 'configuration_fields',
                    'html_content' => '<hr>',
                    'tab' => 'general_data'
                ),
                array(
                    'type'      => 'switch',
                    'is_bool'   => true,
                    'label'     => $this->l('Enable dedicated page for search results'),
                    'desc'      => $this->l('Create a dedicated page to show search results after pressing Enter.'),
                    'name'      => 'dedi_page_enable',
                    'tab'       => 'dedicated_data',
                    'values' => array(
                        array(
                            'id' => 'dedi_page_enable_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'dedi_page_enable_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    )
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Title'),
                    'name'      => 'dedi_page_title',
                    'class'     => 'input fixed-width-xl',
                    'tab'       => 'dedicated_data',
                    'required'  => true
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Link rewrite'),
                    'name'      => 'dedi_page_rewrite',
                    'class'     => 'input fixed-width-xl',
                    'tab'       => 'dedicated_data',
                    'hint'      => $hint1
                ),
                array(
                    'type'      => 'switch',
                    'is_bool'   => true,
                    'label'     => $this->l('Hide categories'),
                    'hint'      => $aspectEnabled.' '.$aspectCat,
                    'name'      => 'hide_cat',
                    'tab'       => 'aspect_data',
                    'values' => array(
                        array(
                            'id' => 'hide_cat_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'hide_cat_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    )
                ),
                array(
                    'type'      => 'switch',
                    'is_bool'   => true,
                    'label'     => $this->l('Hide manufacturer'),
                    'hint'      => $aspectEnabled.' '.$aspectManu,
                    'name'      => 'hide_manu',
                    'tab'       => 'aspect_data',
                    'values' => array(
                        array(
                            'id' => 'hide_manu_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'hide_manu_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    )
                )
            ),
            'submit' => array(
                'id' => 'submit_form_general',
                'title' => $this->l('Save'),
            ),
        );

        return $fields_form;
    }

    /*
     * Synonims form
     */
    protected function renderFormSynonims()
    {
        $this->submit_action = 'submit_form_synonims';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormSynonimsFields();
        $idShop = $this->context->shop->id;

        if (file_exists($this->path.'/output/'.$idShop.'_'.$this->idLang.'_synonims.txt')) {
            $this->fields_value['sniper_synonims_text'] =
                Tools::file_get_contents($this->path.'/output/'.$idShop.'_'.$this->idLang.'_synonims.txt');
        } else {
            $this->fields_value['sniper_synonims_text'] = '';
        }

        return $this->renderForm();
    }

    protected function getFormSynonimsFields()
    {
        $fields_form = array(
            'legend' => array(
                'title' => $this->l('Index configuration'),
                'icon' => 'icon-edit'
            ),
            'input' => array(
                array(
                    'type'  => 'textarea',
                    'label' => $this->l('Synonims'),
                    'name'  => 'sniper_synonims_text',
                    'class' => 'sniper-synonims-textarea',
                ),
                array(
                    'type'  => 'file',
                    'label' => $this->l('Synonims file'),
                    'name'  => 'sniper_synonims_file'
                )
            ),
            'submit' => array(
                'id' => 'submit_form_synonims',
                'title' => $this->l('Save')
            )
        );

        return $fields_form;
    }

    /*
     * Features form
     */
    protected function renderFormFeatures()
    {
        $this->fields_value  = array();
        $this->submit_action = 'submit_form_features';
        $this->fields_form   = $this->getFormFeaturesFields();
        $idShop = $this->context->shop->id;
        $features_config = SniperCommon::getConfig('sniper_features', $idShop, $this->idLang);
        if ($features_config != '') {
            $this->fields_value['sniper_features[]'] = explode(',', $features_config);
        }

        return $this->renderForm();
    }

    protected function getFormFeaturesFields()
    {
        $idShop        = $this->context->shop->id;
        $fields_form = array(
            'legend' => array(
                'title' => $this->l('Features selector'),
                'icon' => 'icon-edit',
            ),
            'input' => array(
                array(
                    'type' => 'select',
                    'label' => $this->l('Select features you would like to show as facets'),
                    'name' => 'sniper_features[]',
                    'multiple' => true,
                    'options' => array(
                    'query' => Feature::getFeatures(
                        $this->idLang,
                        $idShop
                    ),
                    'id' => 'id_feature',
                    'name' => 'name'
                    ),
                ),
            ),
        'submit' => array(
            'id'    => 'submit_form_features',
            'title' => $this->l('Save')
            )
        );

        return $fields_form;
    }

    /*
    * Attributes form
    */
    protected function renderFormAttributes()
    {
        $this->fields_value  = array();
        $this->submit_action = 'submit_form_attributes';
        $this->fields_form   = $this->getFormAttributesFields();
        $idShop = $this->context->shop->id;
        $features_config = SniperCommon::getConfig('sniper_attributes', $idShop, $this->idLang);
        if ($features_config != '') {
            $this->fields_value['sniper_attributes[]'] = explode(',', $features_config);
        }

        return $this->renderForm();
    }

    protected function getFormAttributesFields()
    {
        $fields_form = array(
            'legend' => array(
                'title' => $this->l('Attributes selector'),
                'icon' => 'icon-edit',
            ),
            'input' => array(
                array(
                    'type' => 'select',
                    'label' => $this->l('Select attributes group you would like to show as facets'),
                    'name' => 'sniper_attributes[]',
                    'multiple' => true,
                    'options' => array(
                    'query' => AttributeGroup::getAttributesGroups($this->idLang),
                    'id' => 'id_attribute_group',
                    'name' => 'name'
                    ),
                ),
            ),
        'submit' => array(
            'id'    => 'submit_form_attributes',
            'title' => $this->l('Save')
            )
        );

        return $fields_form;
    }

    /*
     * Product list
     */
    protected function renderFormProducts()
    {
        $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }
        $this->context->smarty->assign(array(
            'sniperfastBoostRange' => $this->getBoostRange()
        ));
        $template = $this->context->smarty->fetch($this->path.'/views/templates/admin/products.tpl');

        return $template;
    }

    protected function renderListProducts()
    {
        $this->toolbar_title = $this->l('Products to export');
        $this->table         = 'product';
        $this->list_id       = 'product';
        $this->lang          = false;
        $this->className     = 'Sniperfast';
        $this->identifier    = 'id_product';
        $this->_orderBy      = 'id_product';
        $this->_orderWay     = 'ASC';
        $this->_filter       = '';
        $this->_filterHaving = null;

        $this->actions      = array();
        $this->bulk_actions = array(
            'enable' => array(
                'text' => $this->l('Enable selected'),
                'icon' => 'icon-check',
            ),
            'disable' => array(
                'text' => $this->l('Disable selected'),
                'icon' => 'icon-remove',
            ),
        );

        // List definition
        $this->fields_list = array(
            'id_product' => array(
                'title'        => $this->l('Product ID'),
                'type'         => 'int',
                'filter_key'   => 'a!id_product',
                'width'        => 100,
            ),
            'prod_name' => array(
                'title'        => $this->l('Name'),
                'filter_key'   => 'pl!name',
            ),
            // 'boost' => array(
                // 'title'        => $this->l('Product boost'),
                // 'type'         => 'int',
                // 'filter_key'   => 'ap!boost',
                // 'width'        => 100,
            // ),
            'enabled' => array(
                'title'        => $this->l('Export'),
                'type'         => 'bool',
                'filter_key'   => 'enabled',
                'havingFilter' => true,
                'callback'     => 'renderCheckboxProducts'
            )
        );

        $this->clearFilters();

        // Query
        $idShop   = (int)$this->context->shop->id;
        $idLangContext   = (int)$this->context->language->id;
        $endpoint = (int)$this->idLang;
        $dbprefix = _DB_PREFIX_;

        // All the variables in this query are already casted right above
        $this->_select = "pl.name as prod_name, IFNULL(ap.boost, 0) as boost,
            (
                CASE WHEN ap.enabled IS NULL THEN
                    (IFNULL(asup.enabled, 1) && IFNULL(am.enabled, 1) && IFNULL(ac.enabled, 1))
                    WHEN ap.enabled IS NOT NULL THEN ap.enabled
                END
            ) as enabled";
        $this->_join =
            "join {$dbprefix}product_shop as ps on ps.id_product = a.id_product
            join {$dbprefix}product_lang as pl on pl.id_product = a.id_product and pl.id_shop = ps.id_shop
            left join {$dbprefix}product_supplier as psu on psu.id_product = a.id_product
            left join {$dbprefix}sniperfast_product ap on ap.id_product = a.id_product
                and ap.id_shop = ps.id_shop and ap.endpoint = $endpoint
            left join {$dbprefix}sniperfast_supplier asup on asup.id_supplier = a.id_supplier
                and asup.id_shop = ps.id_shop and asup.endpoint = $endpoint
            left join {$dbprefix}sniperfast_manufacturer am on am.id_manufacturer = a.id_manufacturer
                and am.id_shop = ps.id_shop and am.endpoint = $endpoint
            left join {$dbprefix}sniperfast_category ac on ac.id_category = a.id_category_default
                and ac.id_shop = ps.id_shop and ac.endpoint = $endpoint
            LEFT JOIN {$dbprefix}shop AS t2 ON t2.id_shop = ap.id_shop";
        $this->_where =
            " AND ps.id_shop = $idShop
            AND pl.id_lang = $idLangContext";
        $this->_group =
            "GROUP BY a.`id_product`";

        $this->initToolbar();
        $this->toolbar_btn = array();

        // reload filter in case that something has been enabled/disabled
        $this->product_filter->prefetch();

        return $this->renderList();
    }

    public function renderCheckboxProducts($cellValue, $row)
    {
        unset($cellValue);
        $idProduct = (int)$row['id_product'];
        $status    = (int)$this->product_filter->isProductEnabled($idProduct);
        $idShop    = $this->context->shop->id;
        $token     = SniperCommon::getConfig('ajax_token', $idShop, $this->idLang);
        $baseUrl   = Tools::getHttpHost(true) . __PS_BASE_URI__;
        $ajaxUrl   = "{$baseUrl}modules/sniperfast/ajax/flipStatusProduct.php?"
            ."id_product={$idProduct}"
            ."&shop={$idShop}"
            ."&ep={$this->idLang}"
            ."&type=prod"
            ."&token={$token}";
        $iconClass = '';

        if ($status == 1) {
            $iconClass = 'sniperfast-icon-enabled';
        } elseif ($status == 0) {
            $iconClass = 'sniperfast-icon-disabled';
        }

        $outputTpl =
            $this->context->smarty->createTemplate(dirname(__FILE__).'/../../views/templates/admin/checkbox.tpl');
        $outputTpl->assign(array(
            'iconClass' => $iconClass,
            'ajaxUrl'   => $ajaxUrl,
            'status'    => $status,
            'idData'    => $idProduct,
            'dataType'  => 'product'
        ));

        $output = $outputTpl->fetch();
        return $output;
    }

    protected function renderAppLogo($appImgName)
    {
        $idShop    = $this->context->shop->id;
        $imgUrl    = SniperCommon::getAppConfig($appImgName, $idShop);
        if (empty($imgUrl)) {
            return '';
        }

        $extension = explode('.', $imgUrl);
        $extension = array_pop($extension);
        if (!file_exists(dirname(__FILE__).'/../../resources/app/'.$appImgName.'.'.$extension)) {
            return '';
        }

        $outputTpl =
            $this->context->smarty->createTemplate(dirname(__FILE__).'/../../views/templates/admin/app_img.tpl');
        $outputTpl->assign(array(
            'app_img_url' => $imgUrl
        ));

        $output = $outputTpl->fetch();
        return $output;
    }

    protected function processListProducts()
    {
        if (Tools::isSubmit('submitBulkenableproduct') && $products = Tools::getValue('productBox')) {
            foreach ($products as $idProduct) {
                $this->product_filter->enableProduct((int)$idProduct);
            }

            $this->somethingSubmitted = true;
        }

        if (Tools::isSubmit('submitBulkdisableproduct') && $products = Tools::getValue('productBox')) {
            foreach ($products as $idProduct) {
                $this->product_filter->disableProduct((int)$idProduct);
            }

            $this->somethingSubmitted = true;
        }
    }

/*
 * Category tree
 */
    protected function renderFormCategories()
    {
        $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }
        $this->context->smarty->assign(array(
            'sniperfastBoostRange' => $this->getBoostRange()
        ));
        $template = $this->context->smarty->fetch($this->path.'/views/templates/admin/categories.tpl');

        return $template;
    }

    protected function renderListCategories()
    {
        $this->toolbar_title = null;
        $this->table         = 'category';
        $this->list_id       = 'category';
        $this->lang          = false;
        $this->className     = 'Sniperfast';
        $this->identifier    = 'id_category';
        $this->_orderBy      = 'id_category';
        $this->_orderWay     = 'ASC';
        $this->_filter       = '';
        $this->_filterHaving = null;
        $this->_group = '';

        $this->actions      = array();
        $this->bulk_actions = array(
            'enable' => array(
                'text' => $this->l('Enable selected'),
                'icon' => 'icon-check',
            ),
            'disable' => array(
                'text' => $this->l('Disable selected'),
                'icon' => 'icon-remove',
            ),
        );

        // List definition
        $this->fields_list = array(
            'id' => array(
                'title'      => $this->l('ID'),
                'type'       => 'int',
                'filter_key' => 'a!id_category',
                'width'      => 100,
            ),
            'name' => array(
                'title'      => $this->l('Name'),
                'filter_key' => 't3!name',
            ),
            'category_path' => array(
                'title'      => $this->l('Category path'),
                'callback'   => 'renderCategoryPath',
                'search'     => false,
                'type'       => 'text',
            ),
            // 'boost' => array(
                // 'title'        => $this->l('Category boost'),
                // 'type'         => 'int',
                // 'filter_key'   => 't4!boost',
                // 'width'        => 100,
            // ),
            'enabled' => array(
                'title'        => $this->l('Export'),
                'type'         => 'bool',
                'filter_key'   => 'enabled',
                'havingFilter' => true,
                'callback'     => 'renderCheckboxCategories'
            )
        );

        $this->clearFilters();

        $dbprefix = _DB_PREFIX_;
        $idShop   = (int)$this->context->shop->id;
        $idLang   = (int)$this->context->language->id;
        $root_cat = (int)Configuration::get('PS_ROOT_CATEGORY');
        $home_cat = (int)Configuration::get('PS_HOME_CATEGORY');

        $this->_select =
            "a.id_category AS id, '' AS category_path,
            t3.name as name, IFNULL(t4.boost, 0) as boost,
            '' AS idealo_path,
            IFNULL(t4.enabled, 1) AS enabled";
        $this->_join   = "LEFT JOIN {$dbprefix}category_shop AS t2 ON t2.id_category = a.id_category
            LEFT JOIN {$dbprefix}category_lang AS t3 ON t3.id_category = a.id_category AND t3.id_shop = t2.id_shop
            LEFT JOIN {$dbprefix}sniperfast_category AS t4 ON t4.id_category = a.id_category
                AND t4.id_shop = t2.id_shop AND t4.endpoint = '$this->idLang'";

        $this->_where =
            " AND t2.id_shop = $idShop
            AND t3.id_lang = $idLang
            AND a.id_category NOT IN ($root_cat, $home_cat)";

        $this->initToolbar();
        $this->toolbar_btn = array();

        return $this->renderList();
    }

    public function renderCategoryPath($cellValue, $row)
    {
        unset($cellValue);
        if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {
            $path = (string)$this->getCategoriesPath(Tools::getPath('', $row['id_category']));
            $path = str_replace(Configuration::get('PS_NAVIGATION_PIPE'), ' > ', $path);
            $categoriesArray = explode('>', $path);
            if (count($categoriesArray) > 1) {
                array_shift($categoriesArray);
            }
            foreach ($categoriesArray as &$catArr) {
                $catArr = trim($catArr);
            }
            $categoriesArray = implode('>', $categoriesArray);
            return $categoriesArray;
        } else {
            return (string)$this->getCategoriesPath(Tools::getPath($row['id_category']));
        }
    }

    protected function getCategoriesPath($categories)
    {
        if (empty($categories)) {
            return "";
        }

        $categories = strip_tags($categories);
        $categories = html_entity_decode($categories);

        return $categories;
    }

    public function renderCheckboxCategories($cellValue, $row)
    {
        unset($cellValue);
        $idCategory = (int)$row['id_category'];
        $status    = (int)$this->product_filter->isCategoryEnabled($idCategory);
        $idShop    = $this->context->shop->id;
        $token     = SniperCommon::getConfig('ajax_token', $idShop, $this->idLang);
        $baseUrl   = Tools::getHttpHost(true) . __PS_BASE_URI__;
        $ajaxUrl   = "{$baseUrl}modules/sniperfast/ajax/flipStatusProduct.php?"
            ."id_category={$idCategory}"
            ."&shop={$idShop}"
            ."&ep={$this->idLang}"
            ."&type=cat"
            ."&token={$token}";
        $iconClass = '';

        if ($status == 1) {
            $iconClass = 'sniperfast-icon-enabled';
        } elseif ($status == 0) {
            $iconClass = 'sniperfast-icon-disabled';
        }

        $outputTpl =
            $this->context->smarty->createTemplate(dirname(__FILE__).'/../../views/templates/admin/checkbox.tpl');
        $outputTpl->assign(array(
            'iconClass' => $iconClass,
            'ajaxUrl'   => $ajaxUrl,
            'status'    => $status,
            'idData'    => $idCategory,
            'dataType'  => 'category'
        ));

        $output = $outputTpl->fetch();
        return $output;
    }

    protected function processListCategoryMapping()
    {
        if (Tools::isSubmit('submitBulkenablecategory')) {
            $categories = Tools::getValue('catmappingBox');
            if (empty($categories) || $categories === null || $categories === false) {
                $categories = Tools::getValue('categoryBox');
            }
            if (empty($categories) || $categories === null || $categories === false) {
                $categories = array();
            }

            foreach ($categories as $idCategory) {
                $this->product_filter->enableCategory((int)$idCategory);
            }

            $this->somethingSubmitted = true;
        }

        if (Tools::isSubmit('submitBulkdisablecategory')) {
            $categories = Tools::getValue('catmappingBox');
            if (empty($categories) || $categories === null || $categories === false) {
                $categories = Tools::getValue('categoryBox');
            }
            if (empty($categories) || $categories === null || $categories === false) {
                $categories = array();
            }

            foreach ($categories as $idCategory) {
                $this->product_filter->disableCategory((int)$idCategory);
            }

            $this->somethingSubmitted = true;
        }

        if (Tools::isSubmit('enabledcategory') && Tools::getIsset('id_category')) {
            $id_cat = (int)Tools::getValue('id_category');
            if ($this->product_filter->isCategoryEnabled($id_cat)) {
                $this->product_filter->disableCategory($id_cat);
            } else {
                $this->product_filter->enableCategory($id_cat);
            }
        }
    }

/*
 * Manufacturers list
 */
    protected function renderFormManufacturers()
    {
        $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }
        $this->context->smarty->assign(array(
            'sniperfastBoostRange' => $this->getBoostRange()
        ));
        $template = $this->context->smarty->fetch($this->path.'/views/templates/admin/manufacturers.tpl');

        return $template;
    }

    protected function renderListManufacturers()
    {
        $this->toolbar_title = $this->l('Filter Manufacturers');
        $this->table         = 'manufacturer';
        $this->list_id       = 'manufacturer';
        $this->lang          = false;
        $this->className     = 'Sniperfast';
        $this->identifier    = 'id_manufacturer';
        $this->_orderBy      = 'id_manufacturer';
        $this->_orderWay     = 'ASC';
        $this->_filter       = '';
        $this->_group       = '';

        $this->actions      = array();
        $this->bulk_actions = array(
            'enable' => array(
                'text' => $this->l('Enable selected'),
                'icon' => 'icon-check',
            ),
            'disable' => array(
                'text' => $this->l('Disable selected'),
                'icon' => 'icon-remove',
            ),
        );

        // List definition
        $this->fields_list = array(
            'id_manufacturer' => array(
                'title'      => $this->l('Manufacturer ID'),
                'filter_key' => 'a!id_manufacturer',
                'type'       => 'int',
                'col'        => 1,
            ),
            'name' => array(
                'title'      => $this->l('Name'),
                'filter_key' => 'a!name',
            ),
            // 'boost' => array(
                // 'title'        => $this->l('Manufacturer boost'),
                // 'type'         => 'int',
                // 'filter_key'   => 't3!boost',
                // 'width'        => 100,
            // ),
            'enabled' => array(
                'title'        => $this->l('Export'),
                'type'         => 'bool',
                'filter_key'   => 'enabled',
                'havingFilter' => true,
                'callback'     => 'renderCheckboxManufacturers'
            )
        );

        $this->clearFilters();

        $dbprefix = _DB_PREFIX_;
        $idShop   = (int)$this->context->shop->id;
        $endpoint = (int)$this->idLang;

        $this->_select =
            "a.id_manufacturer as id,
            a.name as name, IFNULL(t3.boost, 0) as boost,
            IFNULL(t3.enabled, 1) AS enabled";
        $this->_join =
            "LEFT JOIN {$dbprefix}manufacturer_shop AS t1 ON a.id_manufacturer = t1.id_manufacturer
            LEFT JOIN {$dbprefix}shop AS t2 ON t1.id_shop = t2.id_shop
            LEFT JOIN {$dbprefix}sniperfast_manufacturer AS t3
                ON t3.id_manufacturer = a.id_manufacturer
                AND t3.id_shop = t2.id_shop
                AND t3.endpoint = $endpoint";
        $this->_where =
            "AND t2.id_shop = {$idShop}";

        $this->initToolbar();
        $this->toolbar_btn = array();

        return $this->renderList();
    }

    public function renderCheckboxManufacturers($cellValue, $row)
    {
        unset($cellValue);
        $idManufacturer = (int)$row['id_manufacturer'];
        $status    = (int)$this->product_filter->isManufacturerEnabled($idManufacturer);
        $idShop    = $this->context->shop->id;
        $token     = SniperCommon::getConfig('ajax_token', $idShop, $this->idLang);
        $baseUrl   = Tools::getHttpHost(true) . __PS_BASE_URI__;
        $ajaxUrl   = "{$baseUrl}modules/sniperfast/ajax/flipStatusProduct.php?"
            ."id_manufacturer={$idManufacturer}"
            ."&shop={$idShop}"
            ."&ep={$this->idLang}"
            ."&type=manu"
            ."&token={$token}";
        $iconClass = '';

        if ($status == 1) {
            $iconClass = 'sniperfast-icon-enabled';
        } elseif ($status == 0) {
            $iconClass = 'sniperfast-icon-disabled';
        }

        $outputTpl =
            $this->context->smarty->createTemplate(dirname(__FILE__).'/../../views/templates/admin/checkbox.tpl');
        $outputTpl->assign(array(
            'iconClass' => $iconClass,
            'ajaxUrl'   => $ajaxUrl,
            'status'    => $status,
            'idData'    => $idManufacturer,
            'dataType'  => 'manufacturer'
        ));

        $output = $outputTpl->fetch();
        return $output;
    }

    protected function processListManufacturers()
    {
        if (Tools::isSubmit('enabledmanufacturer') && is_numeric(Tools::getValue('id_manufacturer'))) {
            $idManufacturer = (int)Tools::getValue('id_manufacturer');

            if ($this->product_filter->isManufacturerEnabled($idManufacturer)) {
                $this->product_filter->disableManufacturer($idManufacturer);
            } else {
                $this->product_filter->enableManufacturer($idManufacturer);
            }

            $this->somethingSubmitted = true;
        }

        if (Tools::isSubmit('submitBulkenablemanufacturer') && $manufacturers = Tools::getValue('manufacturerBox')) {
            foreach ($manufacturers as $idManufacturer) {
                $this->product_filter->enableManufacturer((int)$idManufacturer);
            }

            $this->somethingSubmitted = true;
        }

        if (Tools::isSubmit('submitBulkdisablemanufacturer') && $manufacturers = Tools::getValue('manufacturerBox')) {
            foreach ($manufacturers as $idManufacturer) {
                $this->product_filter->disableManufacturer((int)$idManufacturer);
            }

            $this->somethingSubmitted = true;
        }
    }

/*
 * Suppliers form
 */
    protected function renderFormSuppliers()
    {
        $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }
        $this->context->smarty->assign(array(
            'sniperfastBoostRange' => $this->getBoostRange()
        ));
        $template = $this->context->smarty->fetch($this->path.'/views/templates/admin/suppliers.tpl');

        return $template;
    }

    protected function renderListSuppliers()
    {
        $this->toolbar_title = $this->l('Filter Suppliers');
        $this->table         = 'supplier';
        $this->list_id       = 'supplier';
        $this->lang          = false;
        $this->className     = 'Sniperfast';
        $this->identifier    = 'id_supplier';
        $this->_orderBy      = 'id_supplier';
        $this->_orderWay     = 'ASC';
        $this->_filter       = '';

        $this->actions      = array();
        $this->bulk_actions = array(
            'enable' => array(
                'text' => $this->l('Enable selected'),
                'icon' => 'icon-check',
            ),
            'disable' => array(
                'text' => $this->l('Disable selected'),
                'icon' => 'icon-remove',
            ),
        );

        // List definition
        $this->fields_list = array(
            'id_supplier' => array(
                'title'      => $this->l('Supplier ID'),
                'type'       => 'int',
                'filter_key' => 'a!id_supplier',
                'col'        => 1,
            ),
            'name' => array(
                'title'      => $this->l('Name'),
                'filter_key' => 'a!name',
            ),
            // 'boost' => array(
                // 'title'        => $this->l('Supplier boost'),
                // 'type'         => 'int',
                // 'filter_key'   => 't3!boost',
                // 'width'        => 100,
            // ),
            'enabled' => array(
                'title'        => $this->l('Export'),
                'type'         => 'bool',
                'filter_key'   => 'enabled',
                'havingFilter' => true,
                'callback'     => 'renderCheckboxSuppliers'
            )
        );

        $this->clearFilters();

        $dbprefix = _DB_PREFIX_;
        $idShop   = (int)$this->context->shop->id;
        $endpoint = (int)$this->idLang;

        $this->_select =
            "a.id_supplier as id,
            a.name AS name, IFNULL(t3.boost, 0) as boost,
            IFNULL(t3.enabled, 1) AS enabled";
        $this->_join =
            "JOIN {$dbprefix}supplier_shop AS t1 ON t1.id_supplier = a.id_supplier
            LEFT JOIN {$dbprefix}shop AS t2 ON t2.id_shop = t1.id_shop
            LEFT JOIN {$dbprefix}sniperfast_supplier AS t3
                ON t3.id_supplier = a.id_supplier AND t3.id_shop = t1.id_shop AND t3.endpoint = $endpoint";
        $this->_where =
            "AND t1.id_shop = {$idShop}";

        return $this->renderList();
    }

    public function renderCheckboxSuppliers($cellValue, $row)
    {
        unset($cellValue);
        $idSupplier = (int)$row['id_supplier'];
        $status    = (int)$this->product_filter->isSupplierEnabled($idSupplier);
        $idShop    = $this->context->shop->id;
        $token     = SniperCommon::getConfig('ajax_token', $idShop, $this->idLang);
        $baseUrl   = Tools::getHttpHost(true) . __PS_BASE_URI__;
        $ajaxUrl   = "{$baseUrl}modules/sniperfast/ajax/flipStatusProduct.php?"
            ."id_supplier={$idSupplier}"
            ."&shop={$idShop}"
            ."&ep={$this->idLang}"
            ."&type=supp"
            ."&token={$token}";
        $iconClass = '';

        if ($status == 1) {
            $iconClass = 'sniperfast-icon-enabled';
        } elseif ($status == 0) {
            $iconClass = 'sniperfast-icon-disabled';
        }

        $outputTpl =
            $this->context->smarty->createTemplate(dirname(__FILE__).'/../../views/templates/admin/checkbox.tpl');
        $outputTpl->assign(array(
            'iconClass' => $iconClass,
            'ajaxUrl'   => $ajaxUrl,
            'status'    => $status,
            'idData'    => $idSupplier,
            'dataType'  => 'supplier'
        ));

        $output = $outputTpl->fetch();
        return $output;
    }

    protected function processListSuppliers()
    {
        if (Tools::isSubmit('enabledsupplier') && is_numeric(Tools::getValue('id_supplier'))) {
            $idSupplier = (int)Tools::getValue('id_supplier');

            if ($this->product_filter->isSupplierEnabled($idSupplier)) {
                $this->product_filter->disableSupplier($idSupplier);
            } else {
                $this->product_filter->enableSupplier($idSupplier);
            }

            $this->somethingSubmitted = true;
        }

        if (Tools::isSubmit('submitBulkenablesupplier') && $suppliers = Tools::getValue('supplierBox')) {
            foreach ($suppliers as $idSupplier) {
                $this->product_filter->enableSupplier((int)$idSupplier);
            }

            $this->somethingSubmitted = true;
        }

        if (Tools::isSubmit('submitBulkdisablesupplier') && $suppliers = Tools::getValue('supplierBox')) {
            foreach ($suppliers as $idSupplier) {
                $this->product_filter->disableSupplier((int)$idSupplier);
            }

            $this->somethingSubmitted = true;
        }
    }

    /*
     * Statistics form
     */
    protected function renderFormStats()
    {
        $idShop    = $this->context->shop->id;
        $firstDayOfMonth = new DateTime('first day of this month');
        $firstDayOfMonth = $firstDayOfMonth->format('Y-m-d');
        $yesterday = new DateTime('yesterday');
        $yesterday = $yesterday->format('Y-m-d');
        $this->context->smarty->assign(array(
            'sniper_first_day' => $firstDayOfMonth,
            'sniper_yesterday' => $yesterday,
            'sniper_excluded_ips' => SniperCommon::getConfig('excluded_ips', $idShop, $this->idLang)
        ));

        $template = $this->context->smarty->fetch($this->path.'/views/templates/admin/stats.tpl');
        return $template;
    }

    /*
     * Export form
     */
    protected function renderFormExport()
    {
        $idShop        = $this->context->shop->id;
        $idEmployee    = $this->context->employee->id;
        $ajaxToken     = SniperCommon::getConfig('ajax_token', $idShop, $this->idLang);

        $outputDir     = $this->path.'/output';
        $baseUrl       = _PS_BASE_URL_.__PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }
        $csvUrl        =
            "{$baseUrl}modules/sniperfast/ajax/getDatafeed.php?shop={$idShop}&lang={$this->idLang}".
            "&token={$ajaxToken}";
        $ajaxExportUrl =
            "{$baseUrl}modules/sniperfast/ajax/doExport.php?shop={$idShop}&lang={$this->idLang}".
            "&employee={$idEmployee}&token={$ajaxToken}";
        $lastExportTS  = SniperCommon::getConfig('last_export_date', $idShop, $this->idLang);

        if (empty($lastExportTS)) {
            $lastExportDate = $this->l('Never');
        } else {
            $lastExportDate = date("d/m/Y H:i:s", $lastExportTS);
        }

        // Create output dir if not exists
        if (!file_exists($outputDir)) {
            umask(0);
            mkdir($outputDir, 0777, true);
        }

        $this->context->smarty->assign(array(
            'max_execution_time' => ini_get('max_execution_time'),
            'memory_limit'       => ini_get('memory_limit'),
            'output_dir'         => $outputDir,
            'dir_writable'       => (bool)is_writable($outputDir),
            'csv_url'            => $csvUrl,
            'ajax_export_url'    => $ajaxExportUrl,
            'cron_export_url'    => $ajaxExportUrl.'&cron=1',
            'last_export_date'   => $lastExportDate,
            'last_export_status' => SniperCommon::getConfig('last_export_status', $idShop, $this->idLang)
        ));

        $template = $this->context->smarty->fetch($this->path.'/views/templates/admin/export_form.tpl');

        return $template;
    }

    /*
     * App welcome
     */
    protected function renderFormAppWelcome()
    {
        $template =
        $this->context->smarty->fetch($this->path.'/views/templates/admin/app_welcome.tpl');
        return $template;
    }

    /*
     * App form
     */
    protected function renderFormAppLogo()
    {
        $this->submit_action = 'submit_form_app_img';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormAppImagesFields();
        return parent::renderForm();
    }

    protected function getFormAppImagesFields()
    {
        $hintFormat = $this->l('Upload an image in jpeg or png format.');
        $hintLogo = $this->l('The size must be exactly 189x75 px.');
        $hintIconApple = $this->l('The size must be exactly 1024x1024 px.');
        $hintIconGoogle = $this->l('The size must be exactly 512x512 px.');
        $hintSplash = $this->l('The size must be exactly 2732x2732 px.');
        $hintSplashGoogle = $this->l('The size must be exactly 1024x500 px.');
        $fields_form = array(
            'legend' => array(
               'title' => $this->l('Images'),
               'icon' => 'icon-mobile',
            ),
            'input' => array(
               array(
                   'type' => 'html',
                   'name' => '',
                   'html_content' => $this->renderAppLogo('app_img_logo')
               ),
               array(
                   'type' => 'file',
                   'label' => $this->l('Logo'),
                   'hint' => $hintFormat.' '.$hintLogo,
                   'name' => 'app_img_logo'
               ),
               array(
                   'type' => 'html',
                   'name' => '',
                   'html_content' => $this->renderAppLogo('app_img_icon_apple')
               ),
               array(
                   'type' => 'file',
                   'label' => $this->l('Icon - App Store'),
                   'hint' => $hintFormat.' '.$hintIconApple,
                   'name' => 'app_img_icon_apple'
               ),
               array(
                   'type' => 'html',
                   'name' => '',
                   'html_content' => $this->renderAppLogo('app_img_icon_google')
               ),
               array(
                   'type' => 'file',
                   'label' => $this->l('Icon - Google Play'),
                   'hint' => $hintFormat.' '.$hintIconGoogle,
                   'name' => 'app_img_icon_google'
               ),
               array(
                   'type' => 'html',
                   'name' => '',
                   'html_content' => $this->renderAppLogo('app_img_splash')
               ),
               array(
                   'type' => 'file',
                   'label' => $this->l('Splash screen'),
                   'hint' => $hintFormat.' '.$hintSplash,
                   'name' => 'app_img_splash'
               ),
               array(
                   'type' => 'html',
                   'name' => '',
                   'html_content' => $this->renderAppLogo('app_img_splash_google')
               ),
               array(
                   'type' => 'file',
                   'label' => $this->l('Additional banner - Google Play'),
                   'hint' => $hintFormat.' '.$hintSplashGoogle,
                   'name' => 'app_img_splash_google'
               ),
            ),
            'submit' => array(
               'id'    => 'submit_form_app_img',
               'title' => $this->l('Save')
            )
        );
        return $fields_form;
    }

    protected function renderFormApp()
    {
        $idShop    = $this->context->shop->id;
        $sniperTemplateLoader = new SniperTemplateLoader($idShop);
        $appTplName = SniperCommon::getAppConfig('app_tpl_name', $idShop);
        if (empty($appTplName)) {
            $appTplName = 'White';
        }
        $outputTpl =
            $this->context->smarty->createTemplate($this->path.'/views/templates/admin/app_form.tpl');
        $outputTpl->assign(array(
            'availableTemplates' => $sniperTemplateLoader->getAvailableTemplates(),
            'templateColor' => $sniperTemplateLoader->getTemplateData($appTplName),
            'selectedTemplate' => $appTplName
        ));
        $template = $outputTpl->fetch();
        return $template;
    }

     /* Banner */
    protected function renderAddBanner()
    {
        $outputTpl =
            $this->context->smarty->createTemplate($this->path.'/views/templates/admin/add_button.tpl');
        $outputTpl->assign(array(
            'button_url' => self::$currentIndex.'&add_banner&token='.$this->token,
            'button_title' => $this->l('Add banner')
        ));
        $template = $outputTpl->fetch();
         return $template;
    }

    protected function renderListBanner()
    {
        $this->toolbar_title = $this->l('Banner');
        $this->table         = 'sniperfast_app_banner';
        $this->list_id       = 'sniperfast_app_banner';
        $this->lang          = false;
        $this->className     = 'Sniperfast';
        $this->identifier    = 'id_banner';
        $this->position_identifier = 'id_banner';
        $this->_orderBy      = false;
        $this->_orderWay     = false;
        $this->_defaultOrderBy = 'position';
        // $this->_filter       = '';
        // $this->_group        = '';

        $this->actions      = array();
        $this->bulk_actions = array(
            // 'enable' => array(
            //     'text' => $this->l('Enable selected'),
            //     'icon' => 'icon-check',
            // ),
            // 'disable' => array(
            //     'text' => $this->l('Disable selected'),
            //     'icon' => 'icon-remove',
            // ),
        );

        // List definition
        $this->fields_list = array(
            'id_banner' => array(
                'title'      => 'ID',
                'filter_key' => 'a!id_banner',
                'type'       => 'int',
                'col'        => 1,
            ),
            'language' => array(
                'title'      => $this->l('Language'),
                'callback'   => 'renderLanguageIsoFromId',
                'search'     => true,
                'type'       => 'text',
            ),
            'name' => array(
                'title'      => $this->l('Name'),
                'filter_key' => 'a!name',
            ),
            'position' => array(
                'title' => $this->l('Position'),
                'filter_key' => 'a!position',
                'position' => 'position',
                'class' => 'fixed-width-md'
            ),
            'random' => array(
                'title'      => $this->l('Random'),
                'type'       => 'bool',
                // 'active'     => 'enabled',
                'filter_key' => 'random',
            ),
            'enabled' => array(
                'title'      => $this->l('Sync'),
                'type'       => 'bool',
                'active'     => 'enabled',
                'filter_key' => 'enabled',
            )
        );

        $this->clearFilters();
        $this->_select =
            "a.id_banner as id,
            '' as language,
            a.name as name,
            a.position as position,
            a.random as random,
            a.enabled as enabled";
        $this->_join = "";
        $this->_where = "";

        $this->initToolbar();
        $this->toolbar_btn = array();

        return $this->renderListBannerAction();
    }

    protected function renderBannerForm()
    {
        $this->submit_action = 'submit_form_app_banner';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormBanner();
        $idShop = $this->context->shop->id;
        $idBanner = Tools::getValue('id_banner');

        $valueMapping = array(
            'id_banner' => 'app_banner_id',
            'id_lang' => 'app_banner_lang',
            'name' => 'app_banner_title',
            'image' => 'app_banner_logo',
            'source' => 'app_banner_source',
            'source_data' => 'app_banner_source_data',
            'random' => 'app_banner_random'
        );

        if (!empty($idBanner)) {
            $appBannerValues = SniperCommonApp::getAppBanner($idBanner, $idShop);
            foreach ($appBannerValues as $key => $val) {
                if (!isset($valueMapping[$key])) {
                    continue;
                }
                if ($key != 'image') {
                    $this->fields_value[$valueMapping[$key]] = $val;
                } else {
                    foreach ($this->fields_form['input'] as &$input) {
                        if ($input['name'] == 'app_banner_logo') {
                            $input['html_content'] = $this->renderBannerLogo($val);
                        }
                    }
                }
            }
        }

        return parent::renderForm();
    }

    protected function getFormBanner()
    {
        $idShop = $this->context->shop->id;
        $fields_form = array(
            'legend' => array(
                'title' => $this->l('Banner'),
                'icon' => 'icon-mobile',
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => 'ID',
                    'name' => 'app_banner_id',
                    'readonly' => true,
                    'class'     => 'input fixed-width-sm'
                ),
                array(
                    'type' => 'select',
                    'label'     => $this->l('Language'),
                    'name'      => 'app_banner_lang',
                    'options' => array(
                        'query' => $this->getWebsiteLanguages($idShop),
                        'id'    => 'id',
                        'name'  => 'name'
                    ),
                    'class'     => 'input fixed-width-xl',
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Title'),
                    'name'      => 'app_banner_title',
                    'class'     => 'input fixed-witdh-xxl'
                ),
                array(
                    'type' => 'html',
                    'name' => 'app_banner_logo',
                    'html_content' => ''
                ),
                array(
                    'type' => 'file',
                    'label' => $this->l('Image'),
                    'name' => 'app_banner_image'
                ),
                array(
                    'type'      => 'select',
                    'label'     => $this->l('Source'),
                    'hint'      => '',
                    'name'      => 'app_banner_source',
                    'options' => array(
                        'query' => $this->getBannerSource(),
                        'id'    => 'id',
                        'name'  => 'name'
                    ),
                    'class'     => 'input fixed-width-xl',
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Data'),
                    'name'      => 'app_banner_source_data',
                    'class'     => 'input fixed-witdh-xxl'
                ),
                array(
                    'type'      => 'switch',
                    'is_bool'   => true,
                    'label'     => $this->l('Random'),
                    'desc'      => $this->l(''),
                    'name'      => 'app_banner_random',
                    'values' => array(
                        array(
                            'id' => 'app_banner_random_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'app_banner_random_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    )
                ),
            ),
            'submit' => array(
                'id'    => 'submit_form_app_banner',
                'title' => $this->l('Save')
            )
        );

        return $fields_form;
    }

    /* Slider */
    protected function renderAddSlider()
    {
        $outputTpl =
            $this->context->smarty->createTemplate($this->path.'/views/templates/admin/add_button.tpl');
        $outputTpl->assign(array(
            'button_url' => self::$currentIndex.'&add_slider&token='.$this->token,
            'button_title' => $this->l('Add slider')
        ));
        $template = $outputTpl->fetch();

        return $template;
    }

    protected function renderListSlider()
    {
        $this->toolbar_title = $this->l('Slider');
        $this->table         = 'sniperfast_app_slider';
        $this->list_id       = 'sniperfast_app_slider';
        $this->lang          = false;
        $this->className     = 'Sniperfast';
        $this->identifier    = 'id_slider';
        $this->position_identifier = 'id_slider';
        $this->_orderBy      = false;
        $this->_orderWay     = false;
        $this->_defaultOrderBy = 'position';
        // $this->_filter       = '';
        // $this->_group        = '';

        $this->actions      = array();
        $this->bulk_actions = array(
            // 'enable' => array(
            //     'text' => $this->l('Enable selected'),
            //     'icon' => 'icon-check',
            // ),
            // 'disable' => array(
            //     'text' => $this->l('Disable selected'),
            //     'icon' => 'icon-remove',
            // ),
        ) ;

        // List definition
        $this->fields_list = array(
            'id_slider' => array(
                'title'      => 'ID',
                'filter_key' => 'a!id_slider',
                'type'       => 'int',
                'col'        => 1,
            ),
            'language' => array(
                'title'      => $this->l('Language'),
                'callback'   => 'renderLanguageIsoFromId',
                'search'     => true,
                'type'       => 'text',
            ),
            'name' => array(
                'title'      => $this->l('Name'),
                'filter_key' => 'a!name',
            ),
            'position' => array(
                'title' => $this->l('Position'),
                'filter_key' => 'a!position',
                'position' => 'position',
                'class' => 'fixed-width-md'
            ),
            'random' => array(
                'title'      => $this->l('Random'),
                'type'       => 'bool',
                // 'active'     => 'enabled',
                'filter_key' => 'random',
            ),
            'enabled' => array(
                'title'      => $this->l('Sync'),
                'type'       => 'bool',
                'active'     => 'enabled',
                'filter_key' => 'enabled',
            ),
        );

        $this->clearFilters();

        $this->_select =
            "a.id_slider as id,
            '' as language,
            a.name as name,
            a.position as position,
            a.random as random,
            a.enabled as enabled";
        $this->_join = "";
        $this->_where = "";

        $this->initToolbar();
        $this->toolbar_btn = array();

        return $this->renderListBannerAction();
    }

    protected function renderSliderForm()
    {
        $this->submit_action = 'submit_form_app_slider';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormSlider();
        $idShop = $this->context->shop->id;
        $idSlider = Tools::getValue('id_slider');

        $valueMapping = array(
            'id_slider' => 'app_slider_id',
            'id_lang' => 'app_slider_lang',
            'name' => 'app_slider_title',
            'source' => 'app_slider_source',
            'source_data' => 'app_slider_source_data',
            'random' => 'app_slider_random'
        );

        if (!empty($idSlider)) {
            $appSliderValues = SniperCommonApp::getAppSlider($idSlider, $idShop);
            foreach ($appSliderValues as $key => $val) {
                if (!isset($valueMapping[$key])) {
                    continue;
                }
                $this->fields_value[$valueMapping[$key]] = $val;
            }
        }

        return parent::renderForm();
    }

    protected function getFormSlider()
    {
        $idShop = $this->context->shop->id;
        $fields_form = array(
            'legend' => array(
                'title' => $this->l('Slider'),
                'icon' => 'icon-mobile',
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => 'ID',
                    'name' => 'app_slider_id',
                    'readonly' => true,
                    'class'     => 'input fixed-width-sm'
                ),
                array(
                    'type' => 'select',
                    'label'     => $this->l('Language'),
                    'name'      => 'app_slider_lang',
                    'options' => array(
                        'query' => $this->getWebsiteLanguages($idShop),
                        'id'    => 'id',
                        'name'  => 'name'
                    ),
                    'class'     => 'input fixed-width-xl',
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Title'),
                    'name'      => 'app_slider_title',
                    'class'     => 'input fixed-witdh-xxl'
                ),
                array(
                    'type'      => 'select',
                    'label'     => $this->l('Source'),
                    'hint'      => '',
                    'name'      => 'app_slider_source',
                    'options' => array(
                        'query' => $this->getSliderSource(),
                        'id'    => 'id',
                        'name'  => 'name'
                    ),
                    'class'     => 'input fixed-width-xl',
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Data'),
                    'name'      => 'app_slider_source_data',
                    'class'     => 'input fixed-witdh-xxl'
                ),
                array(
                    'type'      => 'switch',
                    'is_bool'   => true,
                    'label'     => $this->l('Random'),
                    'desc'      => $this->l(''),
                    'name'      => 'app_slider_random',
                    'values' => array(
                        array(
                            'id' => 'app_slider_random_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'app_slider_random_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    )
                ),
            ),
            'submit' => array(
                'id'    => 'submit_form_app_slider',
                'title' => $this->l('Save')
            )
        );

        return $fields_form;
    }

    /* Notification */
    protected function renderAddNotification()
    {
        $outputTpl =
           $this->context->smarty->createTemplate($this->path.'/views/templates/admin/add_button.tpl');
        $outputTpl->assign(array(
           'button_url' => self::$currentIndex.'&add_notification&token='.$this->token,
           'button_title' => $this->l('Add notification')
        ));
        $template = $outputTpl->fetch();

        return $template;
    }

    protected function renderListNotification()
    {
        $this->toolbar_title = $this->l('Notification');
        $this->table         = 'sniperfast_app_notification';
        $this->list_id       = 'sniperfast_app_notification';
        $this->lang          = false;
        $this->className     = 'Sniperfast';
        $this->identifier    = 'id_notification';
        $this->position_identifier = 'id_notification';
        $this->_orderBy      = 'id_notification';
        $this->_orderWay     = 'ASC';

        $this->actions      = array();
        $this->bulk_actions = array(
          // 'enable' => array(
          //     'text' => $this->l('Enable selected'),
          //     'icon' => 'icon-check',
          // ),
          // 'disable' => array(
          //     'text' => $this->l('Disable selected'),
          //     'icon' => 'icon-remove',
          // ),
        );

        // List definition
        $this->fields_list = array(
          'id_notification' => array(
              'title'      => 'ID',
              'filter_key' => 'a!id_notification',
              'type'       => 'int',
              'col'        => 1,
          ),
          'language' => array(
              'title'      => $this->l('Language'),
              'callback'   => 'renderLanguageIsoFromId',
              'search'     => true,
              'type'       => 'text',
          ),
          'device' => array(
              'title'      => $this->l('Device'),
              'callback'   => 'renderDeviceFromId',
              'search'     => true,
              'type'       => 'text',
          ),
          'title' => array(
              'title'      => $this->l('Title'),
              'filter_key' => 'a!title',
          ),
          'enabled' => array(
              'title'      => $this->l('Sync'),
              'type'       => 'bool',
              'active'     => 'enabled',
              'filter_key' => 'enabled',
          ),
        );

        $this->clearFilters();

        $this->_select =
          "a.id_notification as id,
          '' as language,
          '' as device,
          device as device_code,
          a.title as title,
          a.body as body,
          a.enabled as enabled";
        $this->_join = "";
        $this->_where = "";

        $this->initToolbar();
        $this->toolbar_btn = array();

        return $this->renderListBannerAction();
    }

    protected function renderNotificationForm()
    {
        $this->submit_action = 'submit_form_app_notification';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormNotification();
        $idShop = $this->context->shop->id;
        $idNotification = Tools::getValue('id_notification');

        $valueMapping = array(
          'id_notification' => 'app_notification_id',
          'id_lang' => 'app_notification_lang',
          'title' => 'app_notification_title',
          'body' => 'app_notification_body',
          'time' => 'app_notification_time',
          'source' => 'app_notification_status',
          'topic' => 'app_notification_topic'
        );

        if (!empty($idNotification)) {
            $appNotificationValues = SniperCommonApp::getAppNotification($idNotification, $idShop);
            foreach ($appNotificationValues as $key => $val) {
                if (!isset($valueMapping[$key])) {
                    continue;
                }

                $this->fields_value[$valueMapping[$key]] = $val;
            }
        }

        return parent::renderForm();
    }

    protected function getFormNotification()
    {
        $idShop = $this->context->shop->id;
        $fields_form = array(
          'legend' => array(
              'title' => $this->l('Notification'),
              'icon' => 'icon-mobile',
          ),
          'input' => array(
              array(
                  'type' => 'text',
                  'label' => 'ID',
                  'name' => 'app_notification_id',
                  'readonly' => true,
                  'class'     => 'input fixed-width-sm'
              ),
              array(
                  'type' => 'select',
                  'label'     => $this->l('Language'),
                  'name'      => 'app_notification_lang',
                  'options' => array(
                      'query' => $this->getWebsiteLanguages($idShop),
                      'id'    => 'id',
                      'name'  => 'name'
                  ),
                  'class'     => 'input fixed-width-xl',
              ),
              array(
                  'type'      => 'text',
                  'label'     => $this->l('Title'),
                  'name'      => 'app_notification_title',
                  'class'     => 'input fixed-witdh-xxl'
              ),
              array(
                  'type'      => 'text',
                  'label'     => $this->l('Body'),
                  'name'      => 'app_notification_body',
                  'class'     => 'input fixed-witdh-xxl'
              ),
              // array(
              //     'type' => 'html',
              //     'label'     => $this->l('Time'),
              //     'name' => 'app_notification_time',
              //     'html_content' => ''
              // ),
              array(
                  'type'      => 'switch',
                  'is_bool'   => true,
                  'label'     => $this->l('Send now / Send again'),
                  'desc'      => $this->l(''),
                  'name'      => 'app_notification_send_now',
                  'values' => array(
                      array(
                          'id' => 'app_notification_send_now_on',
                          'value' => 1,
                          'label' => $this->l('Enabled'),
                      ),
                      array(
                          'id' => 'app_notification_send_now_off',
                          'value' => 0,
                          'label' => $this->l('Disabled'),
                      ),
                  )
              ),
          ),
          'submit' => array(
              'id'    => 'submit_form_app_notification',
              'title' => $this->l('Save')
          )
        );

        return $fields_form;
    }

    /* Notification - Order */
    protected function renderAddNotificationOrder()
    {
        $outputTpl =
           $this->context->smarty->createTemplate($this->path.'/views/templates/admin/add_button.tpl');
        $outputTpl->assign(array(
           'button_url' => self::$currentIndex.'&add_notification_order&token='.$this->token,
           'button_title' => $this->l('Add order notification')
        ));
        $template = $outputTpl->fetch();

        return $template;
    }

    protected function renderListNotificationOrder()
    {
        $this->toolbar_title = $this->l('Order notification');
        $this->table         = 'sniperfast_app_notification_order';
        $this->list_id       = 'sniperfast_app_notification_order';
        $this->lang          = false;
        $this->className     = 'Sniperfast';
        $this->identifier    = 'id_notification_order';
        $this->position_identifier = 'id_notification_order';
        $this->_orderBy      = 'id_notification_order';
        $this->_orderWay     = 'ASC';

        $this->actions      = array();
        $this->bulk_actions = array(
          // 'enable' => array(
          //     'text' => $this->l('Enable selected'),
          //     'icon' => 'icon-check',
          // ),
          // 'disable' => array(
          //     'text' => $this->l('Disable selected'),
          //     'icon' => 'icon-remove',
          // ),
        );

        // List definition
        $this->fields_list = array(
          'id_notification_order' => array(
              'title'      => 'ID',
              'filter_key' => 'a!id_notification_order',
              'type'       => 'int',
              'col'        => 1,
          ),
          'language' => array(
              'title'      => $this->l('Language'),
              'callback'   => 'renderLanguageIsoFromId',
              'search'     => true,
              'type'       => 'text',
          ),
          'order_state' => array(
              'title'      => $this->l('Order state'),
              'callback'   => 'renderOrderStateFromId',
              'search'     => true,
              'type'       => 'text',
          ),
          'name' => array(
              'title'      => $this->l('Name'),
              'filter_key' => 'a!name',
          ),
          'enabled' => array(
              'title'      => $this->l('Sync'),
              'type'       => 'bool',
              'active'     => 'enabled',
              'filter_key' => 'enabled',
          ),
        );

        $this->clearFilters();

        $this->_select =
          "a.id_notification_order as id,
          '' as language,
          status as status,
          '' as order_state,
          a.name as name,
          a.enabled as enabled";
        $this->_join = "";
        $this->_where = "";

        $this->initToolbar();
        $this->toolbar_btn = array();

        return $this->renderListBannerAction();
    }

    protected function renderNotificationOrderForm()
    {
        $this->submit_action = 'submit_form_app_notification_order';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormNotificationOrder();
        $idShop = $this->context->shop->id;
        $idNotification = Tools::getValue('id_notification_order');

        $valueMapping = array(
          'id_notification_order' => 'app_notification_order_id',
          'id_lang' => 'app_notification_order_lang',
          'name' => 'app_notification_order_title',
          // 'time' => 'app_notification_order_time',
          'status' => 'app_notification_order_status'
        );

        if (!empty($idNotification)) {
            $appNotificationValues = SniperCommonApp::getAppNotificationOrder($idNotification, $idShop);
            foreach ($appNotificationValues as $key => $val) {
                if (!isset($valueMapping[$key])) {
                    continue;
                }
                // if ($key != 'time') {
                    $this->fields_value[$valueMapping[$key]] = $val;
                // } else {
                //     $hourMinute = explode(':', $val);
                //     $hour = '';
                //     $minute = '';
                //     if (count($hourMinute) == 2) {
                //         $hour = (int)$hourMinute[0];
                //         $minute = (int)$hourMinute[1];
                //     }
                //     foreach ($this->fields_form['input'] as &$input) {
                //         if ($input['name'] == 'app_notification_order_time') {
                //             $input['html_content'] = $this->getHourMinuteWidget($hour, $minute);
                //         }
                //     }
                // }
            }
        }

        return parent::renderForm();
    }

    protected function getFormNotificationOrder()
    {
        $idShop = $this->context->shop->id;
        $fields_form = array(
          'legend' => array(
              'title' => $this->l('Notification'),
              'icon' => 'icon-mobile',
          ),
          'input' => array(
              array(
                  'type' => 'text',
                  'label' => 'ID',
                  'name' => 'app_notification_order_id',
                  'readonly' => true,
                  'class'     => 'input fixed-width-sm'
              ),
              array(
                  'type' => 'select',
                  'label'     => $this->l('Language'),
                  'name'      => 'app_notification_order_lang',
                  'options' => array(
                      'query' => $this->getWebsiteLanguages($idShop),
                      'id'    => 'id',
                      'name'  => 'name'
                  ),
                  'class'     => 'input fixed-width-xl',
              ),
              array(
                  'type'      => 'text',
                  'label'     => $this->l('Title'),
                  'name'      => 'app_notification_order_title',
                  'class'     => 'input fixed-witdh-xxl'
              ),
              // array(
              //     'type' => 'html',
              //     'label'     => $this->l('Time'),
              //     'name' => 'app_notification_order_time',
              //     'html_content' => ''
              // ),
              array(
                  'type'      => 'select',
                  'label'     => $this->l('Source'),
                  'hint'      => '',
                  'name'      => 'app_notification_order_status',
                  'options' => array(
                      'query' => $this->getOrderStatus(),
                      'id'    => 'id',
                      'name'  => 'name'
                  ),
                  'class'     => 'input fixed-width-xl',
              ),
          ),
          'submit' => array(
              'id'    => 'submit_form_app_notification_order',
              'title' => $this->l('Save')
          )
        );

        return $fields_form;
    }

    /* Social */

    protected function renderFormAppSocial()
    {
        $this->submit_action = 'submit_form_app_social';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormAppSocialFields();
        $idShop = $this->context->shop->id;

        foreach ($this->fields_form['input'] as $input) {
            $this->fields_value[$input['name']] = SniperCommon::getAppConfig($input['name'], $idShop);
        }

        return parent::renderForm();
    }

    protected function getFormAppSocialFields()
    {
        $fields_form = array(
           'legend' => array(
               'title' => $this->l('Social'),
               'icon' => 'icon-mobile',
           ),
           'input' => array(
               array(
                   'type'      => 'text',
                   'label'     => 'Facebook',
                   'name'      => 'app_social_fb',
                   'class'     => 'input'
               ),
               array(
                   'type'      => 'text',
                   'label'     => 'Instagram',
                   'name'      => 'app_social_ig',
                   'class'     => 'input'
               ),
               array(
                   'type'      => 'text',
                   'label'     => 'Twitter',
                   'name'      => 'app_social_tw',
                   'class'     => 'input'
               ),
               array(
                   'type'      => 'text',
                   'label'     => 'YouTube',
                   'name'      => 'app_social_yt',
                   'class'     => 'input'
               )
           ),
           'submit' => array(
               'id'    => 'submit_form_app_social',
               'title' => $this->l('Save')
           )
        );

        return $fields_form;
    }

    protected function processFormAppImg()
    {
        if (!Tools::isSubmit('submit_form_app_img') && !Tools::isSubmit('submitAddform_app_img')) {
            return false;
        }

        $idShop = $this->context->shop->id;
        $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }

        $extension = '';
        $resLogo = $this->uploadAppImage('app_img_logo', $this->errors, $extension);
        if ($resLogo) {
            $url = "{$baseUrl}modules/sniperfast/resources/app/app_img_logo.".$extension;
            SniperCommon::setAppConfig('app_img_logo', $url, $idShop);
        }
        $resIconApple = $this->uploadAppImage('app_img_icon_apple', $this->errors, $extension);
        if ($resIconApple) {
            $url = "{$baseUrl}modules/sniperfast/resources/app/app_img_icon_apple.".$extension;
            SniperCommon::setAppConfig('app_img_icon_apple', $url, $idShop);
        }
        $resIconGoogle = $this->uploadAppImage('app_img_icon_google', $this->errors, $extension);
        if ($resIconGoogle) {
            $url = "{$baseUrl}modules/sniperfast/resources/app/app_img_icon_google.".$extension;
            SniperCommon::setAppConfig('app_img_icon_google', $url, $idShop);
        }
        $resSplash = $this->uploadAppImage('app_img_splash', $this->errors, $extension);
        if ($resSplash) {
            $url = "{$baseUrl}modules/sniperfast/resources/app/app_img_splash.".$extension;
            SniperCommon::setAppConfig('app_img_splash', $url, $idShop);
        }
        $resSplashGoogle = $this->uploadAppImage('app_img_splash_google', $this->errors, $extension);
        if ($resSplashGoogle) {
            $url = "{$baseUrl}modules/sniperfast/resources/app/app_img_splash_google.".$extension;
            SniperCommon::setAppConfig('app_img_splash_google', $url, $idShop);
        }

        if (empty($this->errors)) {
            $this->somethingSubmitted = true;
        }
    }

    protected function processFormAppBanner()
    {
        if (!Tools::isSubmit('submit_form_app_banner') && !Tools::isSubmit('submitAddform_app_banner')) {
            return false;
        }
        $idShop = $this->context->shop->id;
        $idBanner = Tools::getValue('app_banner_id');
        $bannerLang = Tools::getValue('app_banner_lang');
        $bannerTitle = Tools::getValue('app_banner_title');
        $bannerSource = Tools::getValue('app_banner_source');
        $bannerSourceData = Tools::getValue('app_banner_source_data');
        $bannerRandom = Tools::getValue('app_banner_random');

        $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
        if (Configuration::get('PS_SSL_ENABLED')) {
            $baseUrl = str_replace('http://', 'https://', $baseUrl);
        }

        $extension = '';
        $bannerLogo = '';
        $idBannerImg = $idBanner;
        if (empty($idBannerImg)) {
            $sql = "SELECT MAX(id_banner)
                FROM "._DB_PREFIX_."sniperfast_app_banner";
            $maxId = (int)Db::getInstance()->getValue($sql);
            $idBannerImg = $maxId + 1;
        }

        $bannerImpageUploadSuccess = $this->uploadAppImage('app_banner_image', $this->errors, $extension, $idBannerImg);
        if ($bannerImpageUploadSuccess) {
            $bannerLogo = "{$baseUrl}modules/sniperfast/resources/app/app_banner_image".$idBannerImg.".".$extension;
        }

        SniperCommonApp::setAppBanner(
            $idBanner,
            $idShop,
            $bannerLang,
            $bannerTitle,
            $bannerLogo,
            $bannerSource,
            $bannerSourceData,
            $bannerRandom
        );

        if (empty($this->errors)) {
            $this->somethingSubmitted = true;
        }
    }

    protected function processFormAppSlider()
    {
        if (!Tools::isSubmit('submit_form_app_slider') && !Tools::isSubmit('submitAddform_app_slider')) {
            return false;
        }
        $idShop = $this->context->shop->id;
        $idSlider = Tools::getValue('app_slider_id');
        $sliderLang = Tools::getValue('app_slider_lang');
        $sliderTitle = Tools::getValue('app_slider_title');
        $sliderSource = Tools::getValue('app_slider_source');
        $sliderSourceData = Tools::getValue('app_slider_source_data');
        $sliderRandom = Tools::getValue('app_slider_random');

        SniperCommonApp::setAppSlider(
            $idSlider,
            $idShop,
            $sliderLang,
            $sliderTitle,
            $sliderSource,
            $sliderSourceData,
            $sliderRandom
        );

        $this->somethingSubmitted = true;
    }

    protected function processFormAppNotification()
    {
        if (!Tools::isSubmit('submit_form_app_notification') &&
            !Tools::isSubmit('submitAddform_app_notification')) {
            return false;
        }

        $idShop = $this->context->shop->id;
        $idNotification = Tools::getValue('app_notification_id');
        $notificationLang = Tools::getValue('app_notification_lang');
        $notificationTitle = Tools::getValue('app_notification_title');
        $notificationBody = Tools::getValue('app_notification_body');
        $notificationSendNow = (int)Tools::getValue('app_notification_send_now');
        $notificationDate = '';
        $topic = '';

        $idNotification = SniperCommonApp::setAppNotification(
            $idNotification,
            $idShop,
            $notificationLang,
            $notificationTitle,
            $notificationBody,
            $notificationDate,
            $topic
        );

        if (!$notificationSendNow) {
            $this->somethingSubmitted = true;
            return;
        }

        $notificationHandler = new SniperNotificationHandler($idShop, $this->idLang, $idNotification);
        $res = $notificationHandler->sendNotification();

        if ($res['success']) {
            $this->somethingSubmitted = true;
            $notificationHandler->setMessageId($idNotification, $idShop, $res['result']);
        } else {
            switch ($res['msg']) {
                case 'invalid_key':
                    $this->errors[] = $this->l('Invalid key');
                    break;
                case 'invalid_index':
                    $this->errors[] = $this->l('Invalid index');
                    break;
                case 'invalid_title':
                    $this->errors[] = $this->l('Title can\'t be empty or is not valid');
                    break;
                case 'invalid_body':
                    $this->errors[] = $this->l('Body can\'t be empty or is not valid');
                    break;
            }
        }
    }

    protected function processFormAppNotificationOrder()
    {
        if (!Tools::isSubmit('submit_form_app_notification_order') &&
            !Tools::isSubmit('submitAddform_app_notification_order')) {
            return false;
        }
        $idShop = $this->context->shop->id;
        $idNotification = Tools::getValue('app_notification_order_id');
        $notificationLang = Tools::getValue('app_notification_order_lang');
        $notificationTitle = Tools::getValue('app_notification_order_title');
        $notificationStatus = Tools::getValue('app_notification_order_status');
        $notificationHour = Tools::getValue('app_notification_hour');
        $notificationMinute = Tools::getValue('app_notification_minutes');
        $notificationTime = '';

        if ($notificationHour !== false && $notificationHour != '-') {
            if ($notificationMinute == '-') {
                $notificationMinute = '00';
            }
            $notificationTime = $notificationHour.':'.$notificationMinute;
        }

        SniperCommonApp::setAppNotificationOrder(
            $idNotification,
            $idShop,
            $notificationLang,
            $notificationTitle,
            $notificationTime,
            $notificationStatus
        );

        $this->somethingSubmitted = true;
    }

    protected function processFormAppSocial()
    {
        if (!Tools::isSubmit('submit_form_app_social') && !Tools::isSubmit('submitAddform_app_social')) {
            return false;
        }
        $idShop = $this->context->shop->id;

        $fbUrl = Tools::getValue('app_social_fb', '');
        SniperCommon::setAppConfig('app_social_fb', trim($fbUrl), $idShop);
        $igUrl = Tools::getValue('app_social_ig', '');
        SniperCommon::setAppConfig('app_social_ig', trim($igUrl), $idShop);
        $twUrl = Tools::getValue('app_social_tw', '');
        SniperCommon::setAppConfig('app_social_tw', trim($twUrl), $idShop);
        $ytUrl = Tools::getValue('app_social_yt', '');
        SniperCommon::setAppConfig('app_social_yt', trim($ytUrl), $idShop);

        $this->somethingSubmitted = true;
    }

    protected function processFormAppCms()
    {
        $idShop = $this->context->shop->id;

        if (!Tools::isSubmit('submit_form_app_cms') && !Tools::isSubmit('submitAddform_app_cms')) {
            return false;
        }

        $submittedCms = Tools::getValue('sniper_app_cms');
        $commaSeparatedCms = '';

        if ($submittedCms) {
            foreach ($submittedCms as $tmp) {
                $commaSeparatedCms.=$tmp.',';
            }
            $commaSeparatedCms = rtrim($commaSeparatedCms, ',');
        };

        SniperCommon::setAppConfig('sniper_app_cms', $commaSeparatedCms, $idShop, $this->idLang);
        $this->somethingSubmitted = true;
    }

    protected function processFormAppContacts()
    {
        $idShop = $this->context->shop->id;

        if (!Tools::isSubmit('submit_form_app_contacts') && !Tools::isSubmit('submitAddform_app_contacts')) {
            return false;
        }

        $idShop = $this->context->shop->id;

        $whatsapp = Tools::getValue('app_contact_whatsapp', '');
        if (empty($whatsapp) || Validate::isPhoneNumber($whatsapp)) {
            SniperCommon::setAppConfig('app_contact_whatsapp', trim($whatsapp), $idShop);
        }
        $phone = Tools::getValue('app_contact_phone', '');
        if (empty($phone) || Validate::isPhoneNumber($phone)) {
            SniperCommon::setAppConfig('app_contact_phone', trim($phone), $idShop);
        }
        $email = Tools::getValue('app_contact_email', '');
        if (empty($email) || Validate::isEmail($email)) {
            SniperCommon::setAppConfig('app_contact_email', trim($email), $idShop);
        }
        $maps = Tools::getValue('app_contact_gmaps', '');
        $mapCoordinates = null;
        if (!empty($maps)) {
            $mapCoordinates = explode(',', $maps);
            if (count($mapCoordinates) == 2) {
                $lat = preg_replace('/[^0-9.]/', '', $mapCoordinates[0]);
                $long = preg_replace('/[^0-9.]/', '', $mapCoordinates[1]);
                if (!empty($lat) && !empty($long)) {
                    $mapCoordinates = $lat.','.$long;
                } else {
                    $mapCoordinates = '';
                }
            } else {
                $mapCoordinates = '';
            }
        } else {
            $mapCoordinates = trim($maps);
        }
        SniperCommon::setAppConfig('app_contact_gmaps', $mapCoordinates, $idShop);


        $this->somethingSubmitted = true;
    }

    protected function processFormAppLanguage()
    {
        if (!Tools::isSubmit('submit_form_app_language') && !Tools::isSubmit('submitAddform_app_language')) {
            return false;
        }

        $idShop = $this->context->shop->id;
        $idLang = (int)Tools::getValue('app_lang_selection');

        $appDefaultLanguage = Tools::getValue('app_default_language');
        SniperCommon::setAppConfig('app_default_language', $appDefaultLanguage, $idShop);

        if ($idLang < 1) {
            $this->errors = $this->l('Invalid language');
            return false;
        }

        $websiteLanguages = $this->getWebsiteLanguages($idShop, true);
        $lang = '';
        foreach ($websiteLanguages as $wLang) {
            if ($wLang['id'] == $idLang) {
                $lang = $wLang['iso_code'];
                break;
            }
        }

        if (empty($lang)) {
            $this->errors = $this->l('Invalid language');
            return false;
        }
        if (empty($_FILES['app_language_file']['name'])) {
            return false;
        }
        $validFileTypes = array(
            'json'
        );
        $filename = $_FILES['app_language_file']["name"];
        $extension = explode(".", $filename);
        $extension = array_pop($extension);
        $extension = Tools::strtolower($extension);
        if (!in_array(Tools::strtolower($extension), $validFileTypes)) {
            $this->errors = $this->l('The file you are trying to upload is not a valid translation file');
            return false;
        }

        $max_file_size = min(
            $this->convertPHPSizeToBytes(ini_get('post_max_size')),
            $this->convertPHPSizeToBytes(ini_get('upload_max_filesize'))
        );
        $uploadedFileSize = $_FILES['app_language_file']['size'];
        if ($uploadedFileSize > $max_file_size) {
            $this->errors = $this->l('File size is bigger than your max upload file size');
            return false;
        }

        $tempFolder = dirname(__FILE__).'/../../resources/app/translations/';
        $fileSettingsName = $idShop.'_'.$lang.'.'.$extension;

        if ($_FILES['app_language_file']['error'] == UPLOAD_ERR_OK) {
            if (!move_uploaded_file(
                $_FILES['app_language_file']['tmp_name'],
                $tempFolder.$fileSettingsName
            )) {
                $this->errors = $this->l('Cannot upload file');
                return false;
            }
        } else {
            $this->errors = $this->l('File upload error');
        }
        $this->somethingSubmitted = true;
        return true;
    }

    protected function renderFormAppCms()
    {
        $this->fields_value  = array();
        $this->submit_action = 'submit_form_app_cms';
        $this->fields_form   = $this->getFormCmsFields();
        $idShop = $this->context->shop->id;
        $cmsConfig = SniperCommon::getAppConfig('sniper_app_cms', $idShop);
        if ($cmsConfig != '') {
            $this->fields_value['sniper_app_cms[]'] = explode(',', $cmsConfig);
        }

        return $this->renderForm();
    }

    protected function getFormCmsFields()
    {
        $fields_form = array(
            'legend' => array(
                'title' => $this->l('CMS selector'),
                'icon' => 'icon-edit',
            ),
            'input' => array(
                array(
                    'type' => 'select',
                    'label' => $this->l('CMS pages'),
                    'name' => 'sniper_app_cms[]',
                    'multiple' => true,
                    'options' => array(
                    'query' => CMS::listCms($this->idLang, false, true),
                    'id' => 'id_cms',
                    'name' => 'meta_title'
                    ),
                ),
            ),
        'submit' => array(
            'id'    => 'submit_form_app_cms',
            'title' => $this->l('Save')
            )
        );

        return $fields_form;
    }

    protected function renderFormAppContacts()
    {
        $this->fields_value  = array();
        $this->submit_action = 'submit_form_app_contacts';
        $this->fields_form   = $this->getFormContactsFields();
        $idShop = $this->context->shop->id;

        foreach ($this->fields_form['input'] as $input) {
            $this->fields_value[$input['name']] = SniperCommon::getAppConfig($input['name'], $idShop);
        }

        return parent::renderForm();
    }

    protected function getFormContactsFields()
    {
        $fields_form = array(
            'legend' => array(
                'title' => $this->l('Contact details'),
                'icon' => 'icon-edit',
            ),
            'input' => array(
                array(
                    'type'      => 'text',
                    'label'     => $this->l('WhatsApp number'),
                    'name'      => 'app_contact_whatsapp',
                    'class'     => 'input fixed-width-xxl'
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Phone number'),
                    'name'      => 'app_contact_phone',
                    'class'     => 'input fixed-width-xxl'
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('E-mail address'),
                    'name'      => 'app_contact_email',
                    'class'     => 'input fixed-width-xxl'
                ),
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Google Maps coordinates'),
                    'name'      => 'app_contact_gmaps',
                    'class'     => 'input fixed-width-xxl'
                )
            ),
        'submit' => array(
            'id'    => 'submit_form_app_contacts',
            'title' => $this->l('Save')
            )
        );

        return $fields_form;
    }

    /*
     * language form
     */
    protected function renderFormAppLanguage()
    {
        $this->submit_action = 'submit_form_app_language';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormAppLanguageFields();
        $idShop = $this->context->shop->id;

        foreach ($this->fields_form['input'] as $input) {
            $this->fields_value[$input['name']] = SniperCommon::getAppConfig($input['name'], $idShop);
        }

        return parent::renderForm();
    }

    protected function getFormAppLanguageFields()
    {
        $idShop = $this->context->shop->id;
        $hintFormat = $this->l('Upload a file in json format.');
        $hintWhich = $this->l('Select which language the file you are uploading belongs to.');
        $hintDefault =
            $this->l('Select which file to use in case a language does not have the respective translations.');
        $fields_form = array(
            'legend' => array(
               'title' => $this->l('Language'),
               'icon' => 'icon-globe',
            ),
            'input' => array(
               array(
                   'type' => 'select',
                   'label' => $this->l('App default language'),
                   'name' => 'app_default_language',
                   'options' => array(
                       'query' => $this->getCurrentlyAvailableLanguages(),
                       'id'    => 'id',
                       'name'  => 'name'
                   ),
                   'hint' => $hintDefault,
                   'class'     => 'input fixed-width-xl',
               ),
               array(
                   'type' => 'select',
                   'label' => $this->l('Language'),
                   'name' => 'app_lang_selection',
                   'options' => array(
                       'query' => $this->getWebsiteLanguages($idShop),
                       'id'    => 'id',
                       'name'  => 'name'
                   ),
                   'hint' => $hintWhich,
                   'class'     => 'input fixed-width-xl',
               ),
               array(
                   'type' => 'file',
                   'label' => $this->l('Translation file'),
                   'hint' => $hintFormat,
                   'name' => 'app_language_file'
               )
            ),
            'submit' => array(
               'id'    => 'submit_form_app_language',
               'title' => $this->l('Save')
            )
        );
        return $fields_form;
    }

    public function processPosition()
    {
        if (Tools::isSubmit('action') && Tools::getValue('action') == 'updatePositions' && Tools::isSubmit('banner')) {
            $banners = Tools::getValue('banner');
            $idBanner = Tools::getValue('id');
            $page = Tools::getValue('page');
            $pagination = Tools::getValue('selected_pagination');
            $position = 0;
            foreach ($banners as $banner) {
                $explBanner = explode('_', $banner);
                if ($explBanner[2] == $idBanner) {
                    break;
                }
                $position++;
            }

            $position += ($page - 1) * $pagination;

            SniperCommonApp::updatePositions(
                $idBanner,
                'banner',
                $position,
                Tools::getValue('way')
            );
        }

        if (Tools::isSubmit('action') && Tools::getValue('action') == 'updatePositions' && Tools::isSubmit('slider')) {
            $sliders = Tools::getValue('slider');
            $idSlider = Tools::getValue('id');
            $page = Tools::getValue('page');
            $pagination = Tools::getValue('selected_pagination');
            $position = 0;

            foreach ($sliders as $slider) {
                $explSlider = explode('_', $slider);
                if ($explSlider[2] == $idSlider) {
                    break;
                }
                $position++;
            }

            $position += ($page - 1) * $pagination;

            SniperCommonApp::updatePositions(
                $idSlider,
                'slider',
                $position,
                Tools::getValue('way')
            );
        }
    }

    /*
     * CSS form
     */
    protected function renderFormCss()
    {
        $this->submit_action = 'submit_form_css';
        $this->fields_value  = array();
        $this->fields_form   = $this->getFormCssFields();
        $idShop = $this->context->shop->id;

        if (file_exists($this->path.'/views/css/'.$idShop.'_'.$this->idLang.'_custom.css')) {
            $this->fields_value['sniper_css_text'] =
                Tools::file_get_contents($this->path.'/views/css/'.$idShop.'_'.$this->idLang.'_custom.css');
        } else {
            $this->fields_value['sniper_css_text'] = '';
        }

        $this->fields_value['sniper_result_height'] =
            SniperCommon::getConfig('sniper_result_height', $idShop, $this->idLang);
        $this->fields_value['sniper_result_width'] =
            SniperCommon::getConfig('sniper_result_width', $idShop, $this->idLang);
        $this->fields_value['sniper_result_align'] =
            SniperCommon::getConfig('sniper_result_align', $idShop, $this->idLang);

        return $this->renderForm();
    }

    protected function getFormCssFields()
    {
        $height1 = $this->l('If the search results appear too high or too low, change their position here.');
        $height2 = $this->l('Higher numbers will make them appear in a lower position.');
        $height3 = $this->l('Lower numbers will make them appear in a higher position.');
        $width1 = $this->l('Set how wide the search results will appear.');
        $align1 = $this->l('Set where the search results will appear horizontally.');
        $align2 = $this->l('If your search bar is placed on the right select Right.');
        $align3 = $this->l('If your search bar is placed on the left select Left.');
        $align4 = $this->l('Select Center to keep results in the middle of the screen.');

        $fields_form = array(
            'legend' => array(
                'title' => $this->l('CSS rules'),
                'icon' => 'icon-edit',
            ),
            'input' => array(
                array(
                    'type'      => 'text',
                    'label'     => $this->l('Results panel vertical position'),
                    'hint'      => $height1.' '.$height2.' '.$height3,
                    'name'      => 'sniper_result_height',
                    'class'     => 'input fixed-width-xl',
                ),
                array(
                    'type'      => 'select',
                    'label'     => $this->l('Results panel width'),
                    'hint'      => $width1,
                    'name'      => 'sniper_result_width',
                    'options' => array(
                        'query' => $this->getWidthValues(),
                        'id'    => 'id',
                        'name'  => 'name'
                    ),
                    'class'     => 'input fixed-width-xl',
                ),
                array(
                    'type'      => 'select',
                    'label'     => $this->l('Results panel alignment'),
                    'hint'      => $align1.' '.$align2.' '.$align3.' '.$align4,
                    'name'      => 'sniper_result_align',
                    'options' => array(
                        'query' => $this->getAlignmentValues(),
                        'id'    => 'id',
                        'name'  => 'name'
                    ),
                    'class'     => 'input fixed-width-xl',
                ),
                array(
                    'type'  => 'textarea',
                    'label' => $this->l('Custom CSS'),
                    'name'  => 'sniper_css_text',
                    'class' => 'sniper-css-textarea',
                )
            ),
            'submit' => array(
                'id'    => 'submit_form_css',
                'title' => $this->l('Save')
            )
        );

        return $fields_form;
    }

    protected function clearFilters()
    {
        /* Reset filters */
        if (Tools::isSubmit('submitResetproduct')) {
            $this->processResetFilters('product');
        }
        if (Tools::isSubmit('submitResetmanufacturer')) {
            $this->processResetFilters('manufacturer');
        }
        if (Tools::isSubmit('submitResetsupplier')) {
            $this->processResetFilters('supplier');
        }
    }

    public function renderList()
    {
        $this->processFilter();

        if (!($this->fields_list && is_array($this->fields_list))) {
            return false;
        }

        $this->getList($this->context->language->id);

        // If list has 'active' field, we automatically create bulk action
        if (isset($this->fields_list) && is_array($this->fields_list) && array_key_exists('active', $this->fields_list)
            && !empty($this->fields_list['active'])) {
            if (!is_array($this->bulk_actions)) {
                $this->bulk_actions = array();
            }

            $this->bulk_actions = array_merge(array(
                'enableSelection' => array(
                    'text' => $this->l('Enable selection'),
                    'icon' => 'icon-power-off text-success'
                ),
                'disableSelection' => array(
                    'text' => $this->l('Disable selection'),
                    'icon' => 'icon-power-off text-danger'
                ),
                'divider' => array(
                    'text' => 'divider'
                )
            ), $this->bulk_actions);
        }

        $helper = new HelperList();

        // Empty list is ok
        if (!is_array($this->_list)) {
            $this->displayWarning($this->l('Bad SQL query', 'Helper').'<br />'.htmlspecialchars($this->_list_error));
            return false;
        }

        $this->setHelperDisplay($helper);

        // For compatibility reasons, we have to check standard actions in class attributes
        foreach ($this->actions_available as $action) {
            if (!in_array($action, $this->actions) && isset($this->$action) && $this->$action) {
                $this->actions[] = $action;
            }
        }

        $helper->is_cms = $this->is_cms;
        $helper->sql = $this->_listsql;
        $list = $helper->generateList($this->_list, $this->fields_list);

        return $list;
    }

    public function renderListBannerAction()
    {
        $this->addRowAction('edit');
        $this->addRowAction('delete');

        return parent::renderList();
    }

    /* Select the correct tab depending on what values are passed by GET and POST */
    protected function setActiveTab()
    {
        $values = self::getAllValues();

        // Key is the keyword while Value is the tab name. Note that partial words are allowed
        $keywords = array(
            // 'product'      => 'products',
            'general'      => 'general',
            'synonim'      => 'synonims',
            'config'       => 'configuration',
            'form_css'     => 'cssrules',
            'features'     => 'features',
            'attributes'   => 'attributes',

            // 'categor' is not a mistake.
            // Since partial words are allowed, we can use a
            // single keyword to refer to 'categorY' and 'categorIES'
            'categor'      => 'categories',
            'manufacturer' => 'manufacturers',
            'supplier'     => 'suppliers',
            'product'      => 'products',
            'app'          => 'app'
        );

        // Fallback
        $this->context->smarty->assign('tab', 'general');

        // If there are values...
        if (is_array($values) && count($values) > 0) {
            // ...iterate through them...
            foreach (array_keys($values) as $key) {
                // ...match the value against each keyword
                $key = Tools::strtolower($key);
                foreach ($keywords as $keyword => $tab) {
                    // ...if the value matches the keyword set the related tab and exit
                    if (preg_match("/.*{$keyword}.*/", $key)) {
                        $this->context->smarty->assign('tab', $tab);
                        break 2;
                    }
                }
            }
        }
    }

    public static function getAllValues()
    {
        return $_POST + $_GET;
    }

    protected function getImageTypes()
    {
        $out = array();
        $psImgTypes = ImageType::getImagesTypes(null, true);

        foreach ($psImgTypes as $type) {
            $out[] = array(
                'id' => $type['id_image_type'],
                'name' => "{$type['name']} - {$type['width']}x{$type['height']} px",
            );
        }

        return $out;
    }

    protected function getAlignmentValues()
    {
        $out = array();
        $out[] = array(
            'id' => 'center',
            'name' => $this->l('Center')
        );
        $out[] = array(
            'id' => 'left',
            'name' => $this->l('Left')
        );
        $out[] = array(
            'id' => 'right',
            'name' => $this->l('Right')
        );

        return $out;
    }

    protected function getWidthValues()
    {
        $out = array();
        $out[] = array(
            'id' => '100',
            'name' => $this->l('Full screen')
        );
        $out[] = array(
            'id' => '90',
            'name' => $this->l('90%')
        );
        $out[] = array(
            'id' => '80',
            'name' => $this->l('80%')
        );
        $out[] = array(
            'id' => '70',
            'name' => $this->l('70%')
        );
        $out[] = array(
            'id' => '60',
            'name' => $this->l('60%')
        );
        $out[] = array(
            'id' => '50',
            'name' => $this->l('50%')
        );

        return $out;
    }

    protected function getValidSynonims($synonims, $idShop, $idLang)
    {
        $synonimLines  = explode("\n", $synonims);
        $synonimLines  = array_filter(array_map('trim', $synonimLines));
        $cleanSynonims = '';
        $wrongSynonims = '';

        foreach ($synonimLines as $line) {
            if ($line == '') {
                continue;
            }
            if (Tools::strpos($line, 'php') !== false || Tools::strpos($line, '<?') !== false) {
                continue;
            } elseif (Tools::strpos($line, '=>') !== false) {
                // Arrow
                $lineEx  = explode('=>', $line);
                $lineEx  = array_filter(array_map('trim', $lineEx));
                if ($lineEx[0] == '' || $lineEx[1] == '') {
                    $wrongSynonims .= $line."\n";
                    continue;
                }
                $firstHalf = explode(',', $lineEx[0]);
                $firstHalf = array_filter(array_map('trim', $firstHalf));
                $firstHalf = $this->cleanRow($firstHalf);
                if (empty($firstHalf)) {
                    $wrongSynonims .= $line."\n";
                    continue;
                }
                $firstHalf = implode(',', $firstHalf);

                $secondHalf = explode(',', $lineEx[1]);
                $secondHalf = array_filter(array_map('trim', $secondHalf));
                $secondHalf = $this->cleanRow($secondHalf);
                if (empty($secondHalf)) {
                    $wrongSynonims .= $line."\n";
                    continue;
                }
                $secondHalf = implode(',', $secondHalf);
                $cleanSynonims .= $firstHalf.'=>'.$secondHalf."\n";
            } elseif (Tools::strpos($line, ',') !== false) {
                // Comma only
                $lineEx = explode(',', $line);
                $lineEx = array_filter(array_map('trim', $lineEx));
                $lineEx = $this->cleanRow($lineEx);
                if (empty($lineEx)) {
                    $wrongSynonims .= $line."\n";
                    continue;
                }
                $lineEx = implode(',', $lineEx);
                $cleanSynonims .= $lineEx."\n";
            } else {
                // No arrow nor comma
                $wrongSynonims .= $line."\n";
            }
        }

        $returnVal = 1;
        if (!empty($wrongSynonims)) {
            file_put_contents($this->path.'/log/'.$idShop.'_'.$idLang.'_synonims_error.txt', $wrongSynonims);
            $returnVal = 0;
        }
        if (empty($cleanSynonims)) {
            $returnVal = -1;
        } else {
            file_put_contents($this->path.'/output/'.$idShop.'_'.$idLang.'_synonims.txt', $cleanSynonims);
        }

        return $returnVal;
    }

    protected function getValidCss($css, $idShop, $idLang)
    {
        $cssLines  = explode("\n", $css);
        $cssLines  = array_filter(array_map('trim', $cssLines));
        $cleanCss = '';

        foreach ($cssLines as $line) {
            if (Tools::strpos($line, '<?') !== false) {
                continue;
            } else {
                $cleanCss .= $line."\n";
            }
        }
        $returnVal = 1;
        if (!empty($cleanCss)) {
            file_put_contents($this->path.'/views/css/'.$idShop.'_'.$idLang.'_custom.css', $cleanCss);
        }

        return $returnVal;
    }

    protected function cleanRow($data)
    {
        $cleanRow = array();
        foreach ($data as $d) {
            if (empty($d)) {
                continue;
            }
            $cleanRow[] = $d;
        }

        return $cleanRow;
    }

    protected function getWebsiteLanguages($idShop, $onlyIds = false)
    {
        $languages = Language::getLanguages(true, $idShop);
        $languageForMenu = array();

        foreach ($languages as $lang) {
            $l = array();
            $l['id']    = $lang['id_lang'];
            $l['iso_code'] = $lang['iso_code'];
            if (!$onlyIds) {
                $l['name']  = $lang['name'];
                $l['selected'] = 0;
                if ($lang['id_lang'] == $this->idLang) {
                    $l['selected'] = 1;
                }
            }
            $languageForMenu[] = $l;
        }
        return $languageForMenu;
    }

    protected function getCurrentlyAvailableLanguages()
    {
        $availableTranslationFiles = glob(_PS_MODULE_DIR_.'sniperfast/resources/app/translations/*.json');
        $availableTranslations = array();

        foreach ($availableTranslationFiles as $file) {
            $file = explode('/', $file);
            $file = array_pop($file);
            $file = explode('.', $file);
            $fileName = array_shift($file);
            if (Tools::strpos($fileName, '_') !== false) {
                continue;
            }
            $availableTranslations[] = array(
                'id' => $fileName,
                'name' => $fileName
            );
        }
        return $availableTranslations;
    }

    protected function getDefaultDedicatedPageValues($idShop)
    {
        $websiteLanguages = $this->getWebsiteLanguages($idShop, true);
        $psDefaultLanguage = Configuration::get('PS_LANG_DEFAULT', null, null, $idShop);
        $defaultDedicatedPage = array(
            'dedi_page_title' => '',
            'dedi_page_rewrite' => ''
        );

        $dedicatedPageIdDefault = SniperCommon::getConfig('dedi_page_id', $idShop, $psDefaultLanguage);
        if (!empty($dedicatedPageIdDefault)) {
            $defaultDedicatedPage['dedi_page_title'] =
                SniperCommon::getConfig('dedi_page_title', $idShop, $psDefaultLanguage);
            $defaultDedicatedPage['dedi_page_rewrite'] =
                SniperCommon::getConfig('dedi_page_rewrite', $idShop, $psDefaultLanguage);
            return $defaultDedicatedPage;
        }

        foreach ($websiteLanguages as $idLang) {
            if ($idLang == $psDefaultLanguage) {
                continue;
            }
            $dedicatedPageIdDefault = SniperCommon::getConfig('dedi_page_id', $idShop, $idLang);
            if (empty($dedicatedPageIdDefault)) {
                continue;
            }
            $defaultDedicatedPage['dedi_page_title'] =
                SniperCommon::getConfig('dedi_page_title', $idShop, $idLang);
            $defaultDedicatedPage['dedi_page_rewrite'] =
                SniperCommon::getConfig('dedi_page_rewrite', $idShop, $idLang);
            break;
        }
        return $defaultDedicatedPage;
    }

    protected function getBoostRange()
    {
        $boostRange = array();
        for ($i = 10; $i >= -10; $i--) {
            $boostRange[] = $i;
        }
        return $boostRange;
    }

    protected function uploadAppImage($appImgName, &$errorMsg, &$extension, $customText = '')
    {
        if (empty($_FILES[$appImgName]["name"])) {
            return false;
        }
        $validFileTypes = array(
            'jpg',
            'jpeg',
            'png'
        );
        $filename = $_FILES[$appImgName]["name"];
        $extension = explode(".", $filename);
        $extension = array_pop($extension);
        $extension = Tools::strtolower($extension);
        if (!in_array(Tools::strtolower($extension), $validFileTypes)) {
            $errorMsg[] = $this->l('The file you are trying to upload is not a valid image file');
            return false;
        }

        $max_file_size = min(
            $this->convertPHPSizeToBytes(ini_get('post_max_size')),
            $this->convertPHPSizeToBytes(ini_get('upload_max_filesize'))
        );
        $uploadedFileSize = $_FILES[$appImgName]['size'];
        if ($uploadedFileSize > $max_file_size) {
            $errorMsg[] = $this->l('File size is bigger than your max upload file size');
            return false;
        }

        $tempFolder = dirname(__FILE__).'/../../resources/app/';
        $fileSettingsName = $appImgName.$customText.'.'.$extension;

        if ($_FILES[$appImgName]['error'] == UPLOAD_ERR_OK) {
            if (!move_uploaded_file(
                $_FILES[$appImgName]['tmp_name'],
                $tempFolder.$fileSettingsName
            )) {
                $errorMsg[] = $this->l('Cannot upload file');
                return false;
            }
        } else {
            $errorMsg[] = $this->l('File upload error');
        }

        return true;
    }

    private function convertPHPSizeToBytes($sSize)
    {
        if (is_numeric($sSize)) {
            return $sSize;
        }
        $sSuffix = Tools::substr($sSize, -1);
        $iValue = Tools::substr($sSize, 0, -1);
        switch (Tools::strtoupper($sSuffix)) {
            case 'p':
            case 'P':
                $iValue *= 1024;
                // no break
            case 't':
            case 'T':
                $iValue *= 1024;
                // no break
            case 'g':
            case 'G':
                $iValue *= 1024;
                // no break
            case 'm':
            case 'M':
                $iValue *= 1024;
                // no break
            case 'k':
            case 'K':
                $iValue *= 1024;
                break;
        }
        return $iValue;
    }

    private function renderBannerLogo($bannerUrl)
    {
        if (empty($bannerUrl)) {
            return '';
        }

        $outputTpl =
            $this->context->smarty->createTemplate(dirname(__FILE__).'/../../views/templates/admin/app_banner.tpl');
        $outputTpl->assign(array(
            'app_banner_url' => $bannerUrl
        ));

        $output = $outputTpl->fetch();
        return $output;
    }

    private function getBannerSource()
    {
        $out = array(
            array(
                'id' => 0,
                'name' => $this->l('None')
            ),
            array(
                'id' => 1,
                'name' => $this->l('Best seller')
            ),
            array(
                'id' => 2,
                'name' => $this->l('Category')
            ),
            array(
                'id' => 3,
                'name' => $this->l('Manufacturer')
            ),
            array(
                'id' => 4,
                'name' => $this->l('Product')
            ),
            array(
                'id' => 5,
                'name' => $this->l('New arrivals (Category)')
            ),
            array(
                'id' => 6,
                'name' => $this->l('New arrivals (Manufacturer)')
            ),
            array(
                'id' => 7,
                'name' => $this->l('New arrivals (Products)')
            )
        );

        return $out;
    }

    private function getSliderSource()
    {
        $out = array(
            array(
                'id' => 1,
                'name' => $this->l('Best seller')
            ),
            array(
                'id' => 2,
                'name' => $this->l('Category')
            ),
            array(
                'id' => 3,
                'name' => $this->l('Manufacturer')
            ),
            array(
                'id' => 4,
                'name' => $this->l('Products')
            ),
            array(
                'id' => 5,
                'name' => $this->l('New arrivals (Category)')
            ),
            array(
                'id' => 6,
                'name' => $this->l('New arrivals (Manufacturer)')
            ),
            array(
                'id' => 7,
                'name' => $this->l('New arrivals (Products)')
            )
        );

        return $out;
    }

    public function renderLanguageIsoFromId($cellValue, $row)
    {
        unset($cellValue);
        return Language::getIsoById($row['id_lang']);
    }

    public function renderOrderStateFromId($cellValue, $row)
    {
        unset($cellValue);
        $states = OrderState::getOrderStates((int)$this->context->language->id);
        foreach ($states as $state) {
            if ($state['id_order_state'] == $row['status']) {
                return $state['name'];
            }
        }
    }

    public function renderDeviceFromId($cellValue, $row)
    {
        unset($cellValue);
        $deviceOs = SniperCommonApp::getDeviceOs();
        $deviceOs[0] = $this->l('All');

        if (isset($deviceOs[$row['device_code']])) {
            return $deviceOs[$row['device_code']];
        }

        return $deviceOs[0];
    }

    public function getOrderStatus()
    {
        $states = OrderState::getOrderStates((int)$this->context->language->id);
        if (empty($states)) {
            return array();
        }

        $statesOut = array();
        foreach ($states as $state) {
            $stateTemp = array(
                'id' => $state['id_order_state'],
                'name' => $state['name']
            );
            $statesOut[] = $stateTemp;
        }

        return $statesOut;
    }

    public function getHourMinuteWidget($hour, $minute)
    {
        $outputTpl =
           $this->context->smarty->createTemplate($this->path.'/views/templates/admin/app_hour.tpl');
        $outputTpl->assign(array(
            'hour' => $hour,
            'minute' => $minute
        ));
        $template = $outputTpl->fetch();
        return $template;
    }

    protected function getShopCurrencies()
    {
        $currencies = Currency::getCurrenciesByIdShop($this->context->shop->id);
        $defaultCurrency = Currency::getDefaultCurrency();
        $out = array();
        $defaultCurrencyId = false;
        if ($defaultCurrency != false) {
            $defaultCurrencyId = $defaultCurrency->id;
            $out[] = array(
                'id'    => $defaultCurrencyId,
                'name'  => $defaultCurrency->name
            );
        }

        foreach ($currencies as $currency) {
            if ($currency['id_currency'] == $defaultCurrencyId
             || $currency['active'] == 0
             || $currency['deleted'] == 1
            ) {
                continue;
            }

            $out[] = array(
                'id'   => $currency['id_currency'],
                'name' => $currency['name']
            );
        }
        return $out;
    }
}
