<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @version   1.0.0
*  @author    Prestalia <prestalia.it>
*  @copyright 2015 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\ExclusiveLock\Locks;

use Prestalia\ExclusiveLock\LockInterface;

class FlockLock implements LockInterface, \Serializable
{
    private $handle;
    private $lockfile_path;
    private $is_acquired = false;

    public function __construct($lockfile)
    {
        try {
            $this->handle = fopen($lockfile, 'w+');
        } catch (\Exception $e) {
            throw new \RuntimeException("Could not open lockfile '$lockfile'. Reason: " . $e->getMessage());
        }

        $this->lockfile_path = $lockfile;
    }

    public function __destruct()
    {
        $this->release();
        fclose($this->handle);
    }

    public function serialize()
    {
        // make the lock not serializable
        return null;
    }

    public function unserialize($data)
    {
        // make the lock not serializable
        return null;
    }

    public function acquire($blocking = false)
    {
        if ($this->is_acquired)  {
            return true;
        }

        $mask = LOCK_EX | LOCK_NB;
        if ($blocking) {
            $mask = LOCK_EX;
        }

        if (!flock($this->handle, $mask)) {
            return false;
        }

        $this->is_acquired = true;
        return true;
    }

    public function release()
    {
        if (!$this->is_acquired) {
            return true;
        }

        if (!flock($this->handle, LOCK_UN)) {
            throw new \RuntimeException("Could not release lock '$this->lockfile_path'");
        }

        $this->is_acquired = false;
        return true;
    }

    public function isAcquired()
    {
        return $this->is_acquired;
    }
}
