<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

class Sniperfast extends Module implements WidgetInterface
{
    public static $definition = array();
    protected $config_form = false;
    protected $idLang;

    public function __construct()
    {
        require_once dirname(__FILE__).'/autoload.php';

        $this->name = 'sniperfast';
        $this->tab = 'front_office_features';
        $this->version = '1.4.3';
        $this->author = 'Prestalia';
        $this->module_key    = '6528d1c630912f2a83b36dfe8f82b4b4';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->controllers = array(
            'AdminSniperfast'
        );

        parent::__construct();

        $this->displayName = $this->l('SniperFast');
        $this->description = $this->l('The best search engine for your shop.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall this module?');

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
        $this->idLang   = $this->context->language->id;
    }

    public function install()
    {
        SniperCommon::installTab(
            'AdminCatalog',
            'AdminSniperfast',
            $this->l('SniperFast'),
            $this->name
        );
        require dirname(__FILE__).'/installer/install.php';

        return parent::install() &&
            $this->registerHook('displayHeader') &&
            $this->registerHook('actionProductAdd') &&
            $this->registerHook('actionProductAttributeUpdate') &&
            $this->registerHook('actionProductUpdate') &&
            $this->registerHook('actionOrderStatusPostUpdate') &&
            $this->registerHook('displayAdminOrderTabShip') &&
            $this->registerHook('OrderConfirmation');
    }

    public function uninstall()
    {
        SniperCommon::uninstallTab('AdminSniperfast');
        require dirname(__FILE__).'/installer/uninstall.php';

        return parent::uninstall();
    }

    public function getContent()
    {
        // There's no configuration to show. Just redirect to the module controller
        Tools::redirectAdmin(
            'index.php?controller=AdminSniperfast&token='
            . Tools::getAdminTokenLite('AdminSniperfast')
        );
    }

    public function hookDisplayHeader()
    {
        $idShop = $this->context->shop->id;
        $idLang = $this->idLang;
        $sourceData = SniperCommon::getConfig('source_data_file', $idShop, $idLang);
        $cssPath = $this->_path.'/views/css/'.$idShop.'_app.css';

        $isCheckout = $this->context->controller->php_self == 'order'
            || $this->context->controller->php_self == 'order-confirmation';
        $isApp = $this->context->cookie->in_app;

        if ($isCheckout && $isApp) {
            $this->context->controller->registerJavascript(
                'sniperfast-app-js',
                'modules/'.$this->name.'/views/js/sniperfast_app.js',
                array('position' => 'bottom', 'priority' => 150)
            );

            $this->context->controller->addCSS($cssPath);
        }

        if (empty($sourceData)) {
            return;
        }

        $this->context->controller->registerStylesheet(
            'sniperfast-css',
            CDN_PROTOCOL.'://cdn.sniperfast.'.CDN_ENDPOINT.'/125/sniperfast.css',
            array('server' => 'remote', 'priority' => 150)
        );
        $this->context->controller->addJQuery();
        $this->context->controller->addJQueryUI('ui.slider');
        $this->context->controller->registerJavascript(
            'modules-sniperfast-js',
            'modules/'.$this->name.'/views/js/'.$idShop.'_'.$idLang.'_'.$sourceData.'.js',
            array('position' => 'bottom', 'priority' => 150)
        );

        $this->context->controller->registerJavascript(
            'sniperfast-search-js',
            CDN_PROTOCOL.'://cdn.sniperfast.'.CDN_ENDPOINT.'/125/sniperfast.js',
            array('server' => 'remote', 'position' => 'bottom', 'priority' => 150)
        );

        if (file_exists(dirname(__FILE__).'/views/js/'.$idShop.'_'.$idLang.'_custom.js')) {
            $this->context->controller->addJS(dirname(__FILE__).'/views/js/'.$idShop.'_'.$idLang.'_custom.js');
            $this->context->controller->registerJavascript(
                'modules-snipercustom-js',
                'modules/'.$this->name.'/views/js/'.$idShop.'_'.$idLang.'_custom.js',
                array('position' => 'bottom', 'priority' => 150)
            );
        }

        $this->context->controller->registerJavascript(
            'sniperfast-jqtouch-js',
            'https://cdnjs.cloudflare.com/ajax/libs/jqueryui-touch-punch/0.2.3/jquery.ui.touch-punch.min.js',
            array('server' => 'remote', 'position' => 'bottom', 'priority' => 150)
        );

        if (file_exists(dirname(__FILE__).'/views/css/'.$idShop.'_'.$idLang.'_custom.css')) {
            $this->context->controller->addCSS($this->_path.'views/css/'.$idShop.'_'.$idLang.'_custom.css');
        }

        if (file_exists(dirname(__FILE__).'/views/css/'.$idShop.'_'.$idLang.'_sniperfast.css')) {
            $this->context->controller->addCSS($this->_path.'views/css/'.$idShop.'_'.$idLang.'_sniperfast.css');
        }
    }

    public function hookActionOrderStatusPostUpdate($params)
    {
        $db = Db::getInstance();
        $dbp = _DB_PREFIX_;
        $idShop = $this->context->shop->id;

        // Get id order
        $idOrder = 0;
        if (!isset($params['id_order'])) {
            return;
        } else {
            $idOrder = $params['id_order'];
        }

        $sql = "SELECT id_customer, reference, id_lang FROM {$dbp}orders
            WHERE id_order = ".(int)$idOrder;
        $orderDetails = $db->executeS($sql);
        $orderDetails = array_shift($orderDetails);

        // Get id customer
        $idCustomer = $orderDetails['id_customer'];
        $idLang = $orderDetails['id_lang'];
        $orderReference = $orderDetails['reference'];

        $newOrderStatus = $params['newOrderStatus'];
        $currentState = $newOrderStatus->id;

        $sql = "SELECT * FROM {$dbp}sniperfast_app_notification_order".
            " WHERE id_shop = ".(int)$idShop.
            " AND id_lang = ".(int)$idLang.
            " AND status = ".(int)$currentState.
            " AND enabled = 1";
        $res = $db->executeS($sql);

        if (empty($res)) {
            return;
        } else {
            $notificationData = array_shift($res);
            $notificationData['name'] = str_replace('[[order_number]]', $orderReference, $notificationData['name']);
        }

        $notificationHandler =
            new SniperNotificationHandler($idShop, $idLang, $notificationData['id_notification_order']);
        $customerTokens = $notificationHandler->getCustomerToken($idShop, $idCustomer);
        if (empty($customerTokens)) {
            return;
        }
        $res = $notificationHandler->sendOrderNotification($notificationData, $customerTokens);
        if (!$res['success']) {
            return;
        }

        $notificationHandler->setOrderMessageId($notificationData['id_notification_order'], $idShop, $res['result']);
    }

    public function renderWidget($hookName = null, array $configuration = array())
    {
        $this->smarty->assign($this->getWidgetVariables($hookName, $configuration));
        return $this->display(dirname(__FILE__), 'views/templates/front/searchBar_17.tpl');
    }

    public function getWidgetVariables($hookName = null, array $configuration = array())
    {
        unset($hookName);
        unset($configuration);
    }

    public function hookActionProductAdd($params)
    {
        $idShop = $this->context->shop->id;

        // Id product
        $idProduct = 1;
        if (array_key_exists('id_product', $params) && $params['id_product'] > 0) {
            $idProduct = (int)$params['id_product'];
        } elseif (array_key_exists('product', $params)) {
            $idProduct = $params['product']->id;
        } else {
            return;
        }

        // Last update
        $lastUpdate = SniperCommon::getLastIncrementalUpdate($idShop);
        $lastUpdate = explode('_', $lastUpdate);
        $lastIdProduct = $lastUpdate[0];
        $lastUpdateDate = $lastUpdate[1];
        $currentTime = time();

        if ($currentTime - $lastUpdateDate < 10) {
            if ($lastIdProduct != $idProduct) {
                SniperCommon::addProductToQueue($idProduct, $currentTime, $idShop);
            }
            return;
        } else {
            SniperCommon::setLastIncrementalUpdate($currentTime, $idProduct, $idShop);
        }

        $queuedProductIds = array();
        $queuedProductIds = SniperCommon::getQueuedProducts($idShop);
        if (empty($queuedProductIds)) {
            $queuedProductIds[] = $idProduct;
        }

        // Sales cache
        $sql = "SELECT id_product, quantity FROM "._DB_PREFIX_."product_sale
            WHERE id_product IN (".pSQL(implode(',', $queuedProductIds)).")";
        $sales = Db::getInstance()->executeS($sql);
        $id_sales_cache = array();
        if (!empty($sales)) {
            foreach ($sales as $tmp) {
                $id_sales_cache[$tmp['id_product']] = $tmp['quantity'];
            }
        }

        // Configured languages
        $configuredLanguages = SniperCommon::getConfiguredLanguages($idShop);
        $productFound = false;
        foreach ($configuredLanguages as $idLang) {
            /* Get config from DB */
            $config = SniperCommon::getConfigKeys($idShop, $idLang);
            if (empty($config)) {
                continue;
            }

            $context = Context::getContext();
            $context->cart = null;
            $context->customer = null;
            $context->language = new Language($idLang, $idLang);
            if (Tools::version_compare(_PS_VERSION_, '1.6.1.0', '<')) {
                $context->shop = new Shop($idShop, null, $idShop);
            } else {
                $context->shop = new Shop($idShop, $idLang, $idShop);
            }

            if (empty($config['sniper_currency'])) {
                $context->currency = Currency::getDefaultCurrency();
            } else {
                $context->currency = new Currency($config['sniper_currency'], $idLang, $idShop);
            }

            // Get if https is requested for images
            $config['use_https'] = Configuration::get('PS_SSL_ENABLED');
            $baseUrl    = _PS_BASE_URL_.__PS_BASE_URI__;
            if (Configuration::get('PS_SSL_ENABLED')) {
                $baseUrl = str_replace('http://', 'https://', $baseUrl);
            }
            $config['base_url'] = $baseUrl;

            $sniperfastProductExporter = new SniperfastProductExporter($idShop, $idLang, $config, $id_sales_cache);

            $stats = array(
                'products_to_export'       => 0,
                'skipped_already_exported' => 0,
                'skipped_price'            => 0,
                'skipped_quantity'         => 0,
                'skipped_inactive'         => 0,
                'skipped_no_attributes'    => 0
            );
            $sniperfastProductFilter = new SniperfastProductFilter((int)$idShop, (int)$idLang);
            $sniperfastProductFilter->prefetch($queuedProductIds);
            $productsToSend = array();
            foreach ($queuedProductIds as $queuedId) {
                if (!$productFound && $queuedId == $idProduct) {
                    $productFound;
                }
                if ($sniperfastProductFilter->isProductEnabled((int)$queuedId)) {
                    $productData = $sniperfastProductExporter->exportData($queuedId, $stats);
                    if (!empty($productData)) {
                        $productsToSend[] = $productData;
                        if (count($productsToSend) >= 50) {
                            $sniperfastProductExporter->sendData($productsToSend);
                            $productsToSend = array();
                        }
                    }
                }
            }
            if (!$productFound) {
                $productsToSend[] = $sniperfastProductExporter->exportData($idProduct, $stats);
            }

            if (count($productsToSend) > 0) {
                $sniperfastProductExporter->sendData($productsToSend);
            }
        }

        $sql = "DELETE FROM "._DB_PREFIX_."sniperfast_queue
            WHERE id_product IN (".pSQL(implode(',', $queuedProductIds)).")
            AND id_shop = ".(int)$idShop;
        Db::getInstance()->execute($sql);
    }

    public function hookActionProductAttributeUpdate($params)
    {
        $this->hookActionProductAdd($params);
    }

    public function hookActionProductUpdate($params)
    {
        $this->hookActionProductAdd($params);
    }

    public function hookOrderConfirmation($params)
    {
        $idShop = $this->context->shop->id;
        $idLang = $this->context->language->id;

        if (Validate::isLoadedObject($params['order']) && $this->context->cookie->in_app) {
            $sql = 'REPLACE INTO '._DB_PREFIX_.'sniperfast_app_orders
                VALUES ('.(int)$params['order']->id.','.(int)$idShop.')';
            Db::getInstance()->execute($sql);
            $orderCurrency = new Currency($params['order']->id_currency);
            $orderAddress = new Address($params['order']->id_address_delivery);
            $orderCountryIso = Country::getIsoById($orderAddress->id_country);
            $orderDetailFull = $params['order']->getProducts();
            $orderDetail = array();
            foreach ($orderDetailFull as $tmp) {
                $detail = array();
                $detail['id_product'] = $tmp['product_id'];
                $detail['id_product_attribute'] = $tmp['product_attribute_id'];
                $detail['product_name'] = $tmp['product_name'];
                $detail['product_quantity'] = $tmp['product_quantity'];
                $detail['total_price_tax_incl'] = round($tmp['total_price_tax_incl'], 2);
                $orderDetail[] = $detail;
            }
            unset($orderDetailFull);

            $orderData = array(
                'id_order'     => (int)$params['order']->id,
                'id_customer'  => (int)$params['order']->id_customer,
                'total_amount' => (float)$params['order']->total_paid_tax_incl,
                'iso_currency' => $orderCurrency->iso_code,
                'iso_country'  => $orderCountryIso,
                'order_detail' => $orderDetail
            );
            SniperCommon::sendOrderDetails($idShop, $idLang, $orderData);
        }
    }

    public function hookDisplayAdminOrderTabShip($params)
    {
        if (array_key_exists('order', $params)) {
            $id_order = (int)$params['order']->id;
        } else {
            return false;
        }
        $sql = 'SELECT id_order, id_shop FROM '._DB_PREFIX_.'sniperfast_app_orders
                WHERE id_order = '.(int)$id_order.'
                AND id_shop = '.(int)$this->context->shop->id;
        $data = Db::getInstance()->executeS($sql);
        if (!empty($data)) {
            return $this->display(dirname(__FILE__), 'views/templates/hook/order.tpl');
        }
    }
}
